-- $Source: /home/dsd/d/aa/monoBANK/AA/ada.sequential_io.spc,v $ 
-- $Revision: 1.6 $ $Date: 96/03/15 10:29:32 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

pragma Suppress( Elaboration_Check );

with Ada.IO_Exceptions;
with System.RTS.TGT.Basic_IO;

pragma ada_child;

generic

    type Element_Type (<>) is private;

package Ada.Sequential_IO is
    
    pragma Suppress( Elaboration_Check );

    type File_Type is limited private;
    
    type File_Mode is (In_File, Out_File, Append_File);
    
    -- File management
    
    procedure Create(File : in out File_Type;
                     Mode : in File_Mode := Out_File;
		     Name : in String := "";
		     Form : in String := "");
		     
    --| Effect
    --|
    --| Establishes a new external file with the given name and form,
    --| and associates this external file with the given file.
    --| The given file is left open. Raises STATUS_ERROR if the given
    --| file is already open. Raises NAME_ERROR if the string given
    --| as NAME does not allow the identification of an external
    --| file.
		     
    procedure Open(File : in out File_Type;
                   Mode : in File_Mode;
		   Name : in String;
		   Form : in String := "");
		   
    --| Effect
    --|
    --| Associates the given file with an existing external file having
    --| the given name and form, and sets the current mode of the file to
    --| the given mode. The given file is left open.
    --|
    --| The exception STATUS_ERROR is raised if the given file is
    --| already open. The exception NAME_ERROR is raised if the
    --| string given as NAME does not allow the identification of
    --| an external file.
		   
    procedure Close  (File : in out File_Type);
    
    --| Effect
    --|
    --| Severs the association between the given file and its
    --| associated external file. The given file is left closed.
    --|
    --| The exception STATUS_ERROR is raised if the given file is
    --| not open.
    
    procedure Delete (File : in out File_Type);
    
    --| Effect
    --|
    --| Deletes the external file associated with the given file.
    --| The given file is closed, and the external file ceases to
    --| exist.
    --|
    --| The exception STATUS_ERROR is raised if the given file is not
    --| open. The exception USE_ERROR is raised if deletion of the
    --| external file is not supported by the environment.
    
    procedure Reset  (File : in out File_Type; Mode : in File_Mode);
    procedure Reset  (File : in out File_Type);
    
    --| Effect
    --|
    --| Resets the given file so that reading from or writing to its
    --| elements can be restarted from the beginning of the file. If
    --| a MODE parameter is supplied, the current mode of the given
    --| file is set to the given mode.
    --|
    --| The exception STATUS_ERROR is raised if the file is not open.
    --| The exception USE_ERROR is raised if the environment does not
    --| support resetting for the external file and, also, if the
    --| environment does not support resetting to the specified mode
    --| for the external file.
    
    function Mode (File : in File_Type) return File_Mode;
    
    --| Effect
    --|
    --| Returns the current mode of the given file.
    --|
    --| The exception STATUS_ERROR is raised if the file is not open.
    
    function Name (File : in File_Type) return String;
    
    --| Effect
    --|
    --| Returns a string which uniquely identifies the external file
    --| currently associated with the given file (and may thus be
    --| used in an open operation).
    --|
    --| The exception STATUS_ERROR is raised if the file is not open.
    
    function Form (File : in File_Type) return String;
    
    --| Effect
    --|
    --| Returns the form string for the external file
    --| currently associated with the given file.
    --|
    --| The exception STATUS_ERROR is raised if the file is not open.
    
    function Is_Open (File : in File_Type) return Boolean;
    
    --| Effect
    --|
    --| Returns TRUE if the given file is open, otherwise returns
    --| false.
    
    -- Input and output operations
    
    procedure Read  (File : in File_Type; Item : out Element_Type);
    
    --| Effect
    --|
    --| Operates on a file of mode IN_FILE. Reads an element from
    --| the given file, and returns the value of this element in the
    --| ITEM parameter.
    --|
    --| The exception MODE_ERROR is raised if the mode is not IN_FILE.
    --| The exception END_ERROR is raised if no more elements can be
    --| read from the given file. The exception DATA_ERROR is
    --| raised if the element read cannot be interpreted as a
    --| value of the type ELEMENT_TYPE.
    
    procedure Write (File : in File_Type; Item : in Element_Type);
    
    --| Effect
    --|
    --| Operates on a file of mode OUT_FILE. Writes the value of
    --| ITEM to the given file.
    --|
    --| The exception MODE_ERROR is raised if the mode is not
    --| OUT_FILE. The exception USE_ERROR is raised if the
    --| capacity of the external file is exceeded.
    
    function End_Of_File (File : in File_Type) return Boolean;
    
    --| Effect
    --|
    --| Operates on a file of mode IN_FILE. Returns TRUE if no more
    --| elements can be read from the given file; otherwise returns
    --| false.
    
    -- Exceptions
    
    Status_Error    : exception renames Ada.IO_Exceptions.Status_Error;
    Mode_Error      : exception renames Ada.IO_Exceptions.Mode_Error;
    Name_Error      : exception renames Ada.IO_Exceptions.Name_Error;
    Use_Error       : exception renames Ada.IO_Exceptions.Use_Error;
    Device_Error    : exception renames Ada.IO_Exceptions.Device_Error;
    End_Error       : exception renames Ada.IO_Exceptions.End_Error;
    Data_Error      : exception renames Ada.IO_Exceptions.Data_Error;
    
private
    
    type File_Record;
    
    type File_Type is access File_Record;
      
    type File_Record is record
	Mode             : File_Mode;
	End_Of_File      : boolean := false;
	Host_File        : System.RTS.TGT.Basic_IO.File_Type;
    end record;
    
end Ada.Sequential_IO;

with Ada.Sequential_IO;
generic package Sequential_IO renames Ada.Sequential_IO;

