-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.lexer.spc,v $
-- $Revision: 1.2 $ -- $Date: 95/10/10 13:18:29 $ -- $Author: akira $
--        Copyright (C) 1995 by Intermetrics, Inc.   

-------------------------------------------------------------
--                        System.RTS.Lexer
-------------------------------------------------------------

package System.RTS.Lexer is

    pragma Suppress( Elaboration_Check );

-------------------------------------------------------------
--| Overview
--|   System.RTS.Lexer provides a finite-state-lexer capability geared
--| toward lexing integers, enumerals, and reals.
--|
--| type TOKEN - abstraction of interesting properties of
--|      characters from the point of view of the FSA. The
--|      TOKEN values are predefined here and are not changeable.
--|
--| type Char_To_Token_Mapping - a mapping of characters to tokens;
--|      forms a partition of characters into classes. Different
--|      FSA's can provide different mappings of characters to tokens.
--|
--| type STATE - states of the FSA.  Predefined "special" reserved
--|      states are START=0, ACCEPTED=1, ERROR=2.  Each FSA defines
--|      other "interesting" states.
--|
--| type STATE_TRANSITIONS - a mapping of
--|      <current_state,current_token> ==> next_state
--|      Each FSA defines its own state_transitions table.
--|
--| type token_transitions - intermediate type decl used to
--|      construct STATE_TRANSITIONS
--|
--| generic function LEXER - "lexes" a stream of characters according
--|      to a state table and token table.  The stream of chars
--|      comes from the generic formal function NEXT.
--|
--| procedure RTS_LEX_STRING - effectively an instantiation of
--|      LEXER, where NEXT provides the characters from a string,
--|      and lexing begins AFTER all leading blanks have been
--|      skipped.  (Used by GET(string) routines of int, enum,
--|      float and fixed io.)
--|
-------------------------------------------------------------

    type TOKEN is (Letter,      -- used only by enumerations
      Digit, Underscore,        -- used by enumerations, floats, integers
      Sign,     -- plus '+' or minus '-'
      Sharp, Colon,     -- indicate start of based literals
      ExDigit,  -- extended digit, a letter from a to z
        -- Note: g..z are syntactically correct and semantically incorrect
      Exp,      -- exponent indicator (or extended digit) 'E' or 'e'
      Decimal,  -- radix point '.', commonly misnamed "decimal point"
      Quote,    -- quotes around character enumerals
      Other_Printing,   -- Weirdo characters
      Blank,    -- Either a space or a horizontal tab (for numeric IO)
      Space,    -- Only a space (matters in enumeration IO)
      HTab,     -- Only a horizontal tab (matters in enumeration IO)
      Delim);   -- Any Ascii character that is not part of the searched
                -- element is a delimiter.

    type CHAR_TO_TOKEN_MAPPING is array (character) of TOKEN;
    -- currently this packing does not work
    -- pragma PACK (Char_To_Token_Mapping);

      -- These states are a pseudo-enumeration type.  The State values
      -- do not matter, so long as the values are distinct.
    type STATE is new integer range 0 .. 127;   --| a small range

      -- These 3 states are common for all input.  States greater than 2
      -- mean different things for enumeration IO, float IO and integer IO
    START       : constant State := 0;
    ACCEPTED    : constant State := 1;
    ERROR       : constant State := 2;

    type token_transitions is array (token) of State;
        -- currently this packing does not work
        -- pragma PACK(Token_Transitions);

    type STATE_TRANSITIONS is array (state range <>) of token_transitions;

    -------------------------------------------------------------
    generic
        with function Next return character;
    function  LEXER(        --| Parses input for text_io
        State_Table: State_Transitions; --|  array of state transitions
        Char_To_Token_Table: Char_To_Token_Mapping
          --| tells what token each character should be translated into.
    ) return State; --| Final State: either ACCEPTED or ERROR
    -------------------------------------------------------------
        --| Overview
        --| Instantiations of this generic function  use
        --| Char_To_Token_Table to translate each character
        --| into a token, and use that token
        --| as one index into the State_Table (the other index is the
        --| current state) to determine the next state of the
        --| "finite state machine".

        --| Algorithm:
        --| Loops until current state is either Accepted or Error.

        --| Requires
        --| Function Next reads file or string and provide characters.
        --| Next must raise End_Error where appropriate.

    -------------------------------------------------------------
    procedure RTS_LEX_STRING(FROM : STRING;
                        STATE_TABLE: STATE_TRANSITIONS;
                        CHAR_TO_TOKEN_TABLE: CHAR_TO_TOKEN_MAPPING;
                        LAST  : out NATURAL
                            -- resulting lexical unit withOUT
                            -- trailing blanks is in
                            --     FROM (FROM'first .. LAST)
                        );
    --+-----------------------------------------------------
    --| Effect
    --|   Lexes the next token in FROM according to STATE_TABLE and
    --|   CHAR_TO_TOKEN_TABLE.  If the token is Accepted according to the
    --|   tables, then then token without trailing blanks
    --|   is returned in FROM(FROM'first .. LAST).  Otherwise,
    --|   DATA_ERROR or END_ERROR are raised.
    --|
    --|  The effect is as specified for GET(string) procedures in enum_io,
    --|   integer_io, fixed_io, and float_io: as to when
    --|   DATA_ERROR and END_ERROR are raised, and which character is at
    --|   FROM(LAST + 1).  LAST is not defined if an exception is raised.
    --|

end System.RTS.Lexer;
