-- $Source: /home/dsd/d/aa/monoBANK/AA/j_system.rts.float_io_support.bdy,v $
-- $Revision: 1.2 $ $Date: 96/04/10 14:00:51 $ $Author: maclaren $
--        Copyright (C) 1995 by Intermetrics, Inc.   

with Ada.IO_Exceptions; use Ada.IO_Exceptions;
with java.lang.Math; use java.lang.Math;

package body System.RTS.Float_IO_Support is

    pragma Suppress( Elaboration_Check );

    function FLOAT_IMAGE (   --| Returns string corresponding to rules for
                             --| FLOAT_IO.PUT RM 14.3.8
                          ITEM : in TGT.Longest_Float;
				               --| value to cvt to string
                          FORE : in Natural;   --| size of integerpart
                          AFT  : in Natural;   --| size of fractional part
                          EXP  : in Natural)   --| size of exponent
             return STRING is

    --+-----------------------------------------------------
    --| Overview
    --|  Returns the string representation of ITEM, as specified by
    --| the formatting arguments:
    --|
    --|     fore: number of chars for leading spaces, '-' if negative, and
    --|           digits before '.'
    --|     aft : number of digits past '.'
    --|     exp : if non-zero, then number of spaces for exponent sign and
    --|           digits; also means there will only be 1 digit before '.'
    --|           If exp=0, then '.' appears in its REAL place.
    --|
    --| If there is insufficient room in any of these fields, the minimum
    --| necessary amount is used.
    --|
    --| Algorithm
    --| (1) X is used as a temporary to be converted.  It is ALWAYS
    --|     non-positive.
    --| (2) Adjust X to be in range -10.0 < X <= -1.0, and EXPONENT
    --|     holding correct power to raise 10 to in order to = ITEM.
    --| (3) Apply rounding of .5 to last digit that will be printed.
    --|     (Note. It does NOT work to round only that digit before
    --|     it is going to be printed - you have to let the rounding
    --|     propagate all the way up; e.g. -9.99999 => -10.0000
    --| (4) X may have been rounded up to -10.0, so re-adjust so it
    --|     is strictly less than -10 (in magnitude)
    --| (5) Compute the length of the resulting string.
    --| (6) Allocate a string of the proper length, and fill it in.
    --|

        use type TGT.Longest_Float;

        X : TGT.Longest_Float := ITEM;

        EXPONENT : integer := 0; -- Exponent (base 10) corresponding to X

        -- Invariant:  abs (item) = abs (x * 10 ** exponent)
        --      after rounding is true of: abs(item rounded in last print pos)

        -- Sizes of various parts of the result string
        result_size,              -- consists of:
        spaces,                   -- optional leading spaces
        sign_size,                -- sign if negative
        fore_digits,              -- integer part
                                  --     '.'
        aft_digits,               -- fractional part
        exp_size   : integer;     -- if exp > 0, then made up of
                                  --     'E' (:'+",'-':)
        exp_zeros  : integer := 0; --     optional leading zeros
        exp_digits : integer := 0; --     exponent value

	temporary  : integer := 0;

        function max (a, b: integer)
          return integer renames Integer'Max;

        -------------------------------------------------------------
        function LOG10 (arg : Integer) return Integer
          -- Computes log10(abs arg), where log10(0) == 0;
        is
          Numerator : integer := abs arg;  -- Starts as abs of arg
                                -- gets divided by 10 in loop, counting
                                -- number of trips around the loop
        begin
        for Result in 0 .. 100 loop
            Numerator := Numerator / 10;

            if Numerator <= 0 then
                return Result;
            end if;
        end loop;
        end log10;

    begin
        -- (0) Do some initializations:
        aft_digits := max (1, AFT);  -- Always at least 1 digit past '.'

        -- (1) Make X be negative and set sign_size:
        if X >= 0.0 then
            X := - X;
            sign_size := 0; -- No leading '+'
        else
            sign_size := 1; -- Need a leading '-'
        end if;

        -- (2) If X is non-zero, adjust X and EXPONENT such that
        --            -10.0 < X <= -1.0
        if X = 0.0 then
            -- X = 0.0, EXPONENT = 0 (due to initialization), and it is true
            -- that item = x * 10 ** exponent.
            -- X is not in the range -10 .. -1, but subsequent calculations
            -- will work OK anyway.
            null;

        else -- X /= 0.0
            -- (2.1) If X has magnitude >= 10, then repeatedly "divide" by 10.
            -- (If X has a smaller magnitude, then the while loop won't be
            -- entered.)
            while X <= -10.0 loop
                X := X / 10.0;
                EXPONENT := EXPONENT + 1;
            end loop;

            -- (2.2) If X has magnitude < 1, then repeatedly multiply by 10.
            -- (If X has a larger magnitude, then the while loop won't be
            -- entered.)
            while X > -1.0 loop
                X := X * 10.0;
                EXPONENT := EXPONENT - 1;
            end loop;

            -- (3) Figure out which digit will be the last one printed, and
            --     add 0.5 to that position in order to correctly ROUND the
            --     result. (Note, add -0.5 since X is negative.)
            --
            --     e.g., X is currently of the form: 9.123456
            --     if exp/=0, then the last digit is AFT_DIGITS past '.',
            --           corresponding to 10 ** -aft_digits
            --
            --     if exp=0, then the result will be X shifted by EXPONENT
            --           positions (+ => left, - => right), so the last
            --           digit is in position AFT_DIGITS + EXPONENT. For
            --           example aft_digits=4 and exponent=2, the string is
            --             "912.3456" and last digit was in position 6== 4+2
            --           same aft but exponent=-2 gives
            --             "0.0912" last digit in position 2 == 4+(-2)


	    temporary := 
	              (-(AFT_DIGITS + boolean'pos(EXP = 0) * EXPONENT));
            X := X - (0.5 * TGT.Longest_Float(java.lang.Math.pow(
	                         Long_Float(10.0), 
			         Long_Float(temporary))));

            -- (4) This might have caused X to round "up" to -10.0, e.g., if
            --     ITEM = -9.9999999999; in which case X and EXPONENT need
            --     adjustment.

            if X <= -10.0 then  -- X can't really be < -10.0, but this is safe
                X := X / 10.0;
                EXPONENT := EXPONENT + 1;
            end if;

        end if;   -- X = 0.0

        -- (5) Now X is either 0.0, or is in range -10.0 < X <= -1.0; X has
        --     been rounded in the last digit; and EXPONENT is correct.
        --     Now determine the size of each part of the result string:

        if EXP = 0 then
            fore_digits := max (1,    -- always has at least 1 FORE digit
                           1 + EXPONENT);  -- e.g., item=10.0 has exponent=1
                                          -- needs 2 digits...
            exp_size    := 0;    -- no exponent stuff

        else
            fore_digits := 1;    -- scientific notation with 1 integer digit

            exp_digits  := log10 (EXPONENT) + 1; -- add 1 for "ones" place

            exp_zeros   := max (0, EXP - (exp_digits + 1));
                  -- EXP accounts for exp_sign, leading zeros, and digits
                  -- If more space is needed than given by EXP, then EXP is
                  -- overridden.
            exp_size    := 2 + exp_zeros + exp_digits;  -- 'E' + Sign

         end if; -- EXP /= 0

         spaces := max (0, FORE - (sign_size + fore_digits));
                   -- spaces fills up that part of FORE not taken up by
                   -- the sign or integer digits.

         result_size := spaces + sign_size + fore_digits + 1 -- '.'
                        + aft_digits + exp_size;

         --(6) Make a string and fill it in
         declare
             RESULT : string (1 .. result_size);
             index  : integer range 1 .. result_size + 1 := 1;
                      -- tracks where next char should be put into result

             period_position : constant integer :=
                   spaces + sign_size + fore_digits + 1;
                   -- Where to put '.' ; when index=period_pos, '.' is added

             leading_zeros_size : integer := 0;  -- number of leading zeros
                        -- needed if exp=0 and exponent < 0

             char0 : constant := character'pos ('0');

             Number_To_Round : TGT.Longest_Float;
                   -- temp to hold each float number in cvt loop
             digit : integer range 0 .. 9 := 0;
                   -- temp to hold each integer digit in cvt loop

             exp_sign : character := '+';   -- character for exponent

             -------------------------------------------------------------
             procedure add (c : character)  -- Adds c to result in next pos.
             is
             begin
                 RESULT (index) := c;
                 index := index + 1;
             end add;

         -------------------------------------------------------------
         begin
             -- (6.0) Add leading spaces
             for sp in 1 .. spaces loop
                 add (' ');
             end loop;

             -- (6.1) Add sign
             if ITEM < 0.0 then add ('-'); end if;

             -- (6.2) If exp=0 and exponent < 0, then we need EXPONENT number
             --       of leading zeros (one before '.', the rest after)
             --       but not more than fore_digits + aft_digits.
             --       (NB. that FOR loop will only get entered when exponent<0
             --       Also remember to check for adding '.' !
             if EXP = 0  and then EXPONENT < 0 then
                 leading_zeros_size := --|   min(- exponent, foredig+aft_dig)
                     - max (EXPONENT, - (fore_digits + aft_digits) );

                 for z in 1 .. leading_zeros_size loop
                     if index = period_position then add ('.'); end if;
                     add ('0');
                 end loop;
             end if;

             -- (6.3) We've finally arrived at the point we were waiting for:
             --       putting the interesting digits into the result string.
             --          The mechanism is to slice off the integer part of X
             --       (which is between -9 and -1), get the char value of
             --       the digit, and add that to the string.  Before doing
             --       the cvt-to-int of X it is necessary to reduce the
             --       magnitude by 0.5 (because cvt-to-int first INCREASES
             --       the magnitude by 0.5 to perform rounding -- if we
             --       didn't reduce by 0.5, half the digits in result
             --       would be 1 value too high!!
             --          Note that this loop works correctly when ITEM (and X)
             --       is 0.0 - it outputs a '0' each time around, for the
             --       desired number of digits.
             --            Remember to check if it's time to add '.' ...


             for d in integer range
                      1 .. fore_digits + aft_digits -- total num digits
                           - leading_zeros_size     -- minus zeros printed
             loop

                 if index = period_position then add ('.'); end if;

                 Number_To_Round := X + 0.5;
                 if Number_To_Round > 0.0 then
                     --  -0.5 < X <= 0.0, so adding 0.5 resulted in a POSITIVE
                     --  number, which will have 0.5 added to it during
                     --  conversion, bringing it dangerously close to +1.0 .
                     --  (This is what happens when  X = 0.0 !!) Therefore,
                     -- revert to the original non-positive value of X.

                     Number_To_Round := X;
                 end if;

                 digit := - integer (Number_To_Round);  -- digit is positive

                 add (character'val(char0 + digit));

                 X := (X + TGT.Longest_Float(digit)) * 10.0;
                      -- reduce magnitude of X by its
                      -- integer part, and bring up next digit to the
                      -- integer position.

                 -- ASSERT Integer part of X is now between -9 and zero

             end loop;

             -- (6.4) if exp=0, then we're all done, else the exp_part has to
             --       be added

             if EXP = 0 then
                 null;
                 -- ASSERT: result is all filled up, ie,
                 -- index = result'last+1
             else
                 -- (6.5) Fill in EXP part.
                 -- a. 1 + exp_digits is the size of the string that will be
                 --    returned by integer'image(EXPONENT) -- < contains
                 --    space for sign, then digits> -- so assign that to
                 --    a slice at the END of result.  Note there will be the
                 --    extra (:space,'-':) sign that is not desired; but that
                 --    will be replaced by the REAL sign, or by zeros.
                 -- b. add E, the sign, and any zeros.
			
                 result ((result'last - exp_digits) .. result'last) :=
                   integer'image (EXPONENT);
                     -- note beginning of slice starts at:
                     -- last - (1+exp_digits) + 1 == last - exp_digits

                 add ('E');

                 if EXPONENT < 0 then exp_sign := '-'; end if;
                 add (exp_sign);

                 for z in 1 .. exp_zeros loop
                     add ('0');
                 end loop;
             end if;  -- EXP /= 0

             -- (7) We're all done!  Return result.

             return RESULT;
         end;
    end FLOAT_IMAGE;

    -------------------------------------------------------------


    function FLOAT_VALUE (IMAGE : string) return TGT.Longest_Float
    --+-----------------------------------------------------
    --| Overview
    --| Returns the floating point value corresponding to IMAGE.
    --|
    --| Requires:
    --|   IMAGE is lexically correct (and therefore also non-null)
    --|         (leading and trailing blanks are OK)
    --|
    --| Exceptions
    --|   DATA_ERROR if based number and
    --|      (1) base is not in range 2 .. 16
    --|   or (2) any of "digits" don't belong to the base
    --|   or (3) value is too big to hold in FLOAT.  Numeric_Error is
    --|          converted to DATA_ERROR.

    --| Algorithm
    --|   1. It is first necessary to determine if there is a base, and if so,
    --|      what it is.  This is done by scanning from the front until a
    --|      base indicator (:'#', ':':) is seen, or a '.' is encountered.
    --|      Along the way, if a '-' sign is seen, its parity is noted (the
    --|      initial assumption is positive number).
    --|        If this is a based number, then integer'value is used to
    --|      get the value, which is checked for being in range 2 .. 16.
    --|        Another side effect of the scan is to determine where the
    --|      first float digit is.
    --|
    --|   2. The second loop consumes the floating value,
    --|      accumulating a NEGATIVE
    --|      value of the correct magnitude.  This is done
    --|      because if this is
    --|      a machine with two's complement floating point,
    --|      you can't represent
    --|      abs FLOAT'FIRST (which is embarassing).
    --|        Each time a hex digit is encountered, it is first checked
    --|      to be in range of the base; then the current result is
    --|      multiplied by BASE and the digit value is added in.
    --|      Protection against Overflow: if the incoming image has 200
    --|      digits, RESULT will probably overflow, since it will be
    --|      multiplied by BASE 200 times (base**200). However, for
    --|      all digits past the first 10-20 non-leading-zero digits,
    --|      their addition to the current result has no effect, since
    --|      by then RESULT is too big.  Thus after a while it isn't
    --|      necessary to continue multiplying by base.  Scheme:
    --|         Lower_Limit is 1 multiple of BASE away from float'first,
    --|         when result is "bigger" in magnitude than L_L, then
    --|         it doesn't get multiplied by BASE.  Instead,
    --|         Skipped_Exponent is incremented to remember how many
    --|         times RESULT was NOT multiplied by BASE; this is
    --|         later added to Exponent for adjustment.
    --|
    --|        Boolean E_is_Exponent determines whether 'E'
    --|         is a hex digit in the
    --|      based part, or signals beginning of Exponent.
    --|        When the '.' is seen, the number of fractional digits is
    --|        counted (negative number of them kept in Exponent),
    --|        because that is how
    --|      many extra times RESULT has been multiplied by BASE -- this
    --|      is fixed by reducing the final Exponent value.


    --|       Note 1. The float value must be accumulated in a FLOAT
    --|       instead of an integer because the integer can easily
    --|       overflow for big float values.
    --|
    --|       Note 2. The hex values for each char are picked up as
    --|       integers so the illegality check DIGIT >= BASE does
    --|       an exact match.  If FLOAT were used for digit, then
    --|       it might pass the test when digit was "less than" BASE
    --|       in 1 low-order bit.

    is

    type int_val_type is array (character) of hex_values;
    int_val : constant int_val_type := int_val_type'(
                    -- each legal hex character is mapped into its
                    -- hex value; all others are mapped into
                    -- BIGGER number.  Note syntax permits ANY char,
                    -- but of course the legal ones must correspond to
                    -- to the current base.
      '0' =>  0, '1' =>  1, '2' =>  2, '3' =>  3, '4' =>  4,
      '5' =>  5, '6' =>  6, '7' =>  7, '8' =>  8, '9' =>  9,
      'a' | 'A' => 10, 'b' | 'B' => 11, 'c' | 'C' => 12,
      'd' | 'D' => 13, 'e' | 'E' => 14, 'f' | 'F' => 15,
      others => 16        -- beyond range of legal hex digits
         );

	use type TGT.Longest_Float;

        subtype digit is character range '0' .. '9';
        subtype upper_case is character range 'A' .. 'Z';
        subtype lower_case is character range 'a' .. 'z';

        first_digit_pos : natural := 0;  -- position of first digit in IMAGE

        base : TGT.Longest_Float;
        ibase : integer := 10;  -- moved into base later

        sign : TGT.Longest_Float := -1.0;  -- NOTE sign = -1.0 means '+' sign
        exponent : integer := 0;  -- number of "aft" digits

        Lower_Limit : TGT.Longest_Float;  
			      -- float'first/base.  Stop mult
                              -- result by base when result is more negative
        skipped_exponent : integer := 0;  -- number of times RESULT is not
          -- multiplied by BASE if RESULT gets bigger than Lower_Limit

        E_is_exponent : boolean := TRUE;
          -- false when between # and #, otherwise TRUE.  Tells how to
          -- interpret 'e' or 'E'

        RESULT : TGT.Longest_Float := 0.0;
        exponent_part : TGT.Longest_Float;  -- used to compute base**exp
        hex_digit : integer;  -- picks up character's hex value
        float_hex_digit : TGT.Longest_Float;  -- and its float value
	saw_radix_point : boolean := false;
    begin
	
        for index in IMAGE'range loop
            case IMAGE(index)
            is
            when '-' =>
                sign := + 1.0;  -- remember, +1.0 means negative
                                -- default value is for positive number
            when digit =>
                if first_digit_pos = 0 then
                      -- This is the first digit we've seen, so remember it.
                    first_digit_pos := index;
                end if;

            when '.' =>
	        saw_radix_point := true;
                exit;  -- We've determined that there is no BASE, so stop

            when '#' | ':' =>
                  --End of the base and beginning of float digits
        ibase := integer'value (IMAGE(first_digit_pos .. index -1));

                if ibase not in legal_base then
                    raise DATA_ERROR;
                end if;

                first_digit_pos := index + 1;  -- where to start float num
                E_is_exponent := FALSE;  -- we're about to scan based num
                exit;

            when others =>
                null;

            end case;
        end loop;


        base := TGT.Longest_Float (ibase);
        Lower_Limit := TGT.Longest_Float'first / base;
            -- Conditions:
            --  1. If there is a base, it has been set.
            --  2. first_digit_pos is where to start getting float digits
            --  3. If it is a positive number, sign = -1.0, else it is +1.0
            --  4. E_is_exponent is false if we're starting a based number,
            --     else it's true

            -- Now loop over the float-digit part of IMAGE, stopping at
            -- trailing # or :, E or e, or stop at end of the string.
            -- Take note of where the '.' appears, so aft digits can be
            -- counted in EXPONENT.

        if first_digit_pos = 0 then
            first_digit_pos := IMAGE'first;
        end if;



        for index in first_digit_pos .. IMAGE'last loop
            case IMAGE(index)
            is

            when '_' | ' ' | ascii.HT =>
                null;

            when '.' =>
	        saw_radix_point := true;
                Exponent := 0;  -- reset to zero

            when '#' | ':' =>
                -- We've reached the end of the base part.
                -- Next comes 'E', or end of string
                E_is_Exponent := TRUE;

            when digit | upper_case | lower_case =>
              hex_digit := int_val (IMAGE(index));
                if hex_digit = 14      -- i.e., 'E' or 'e'
                and then E_is_Exponent
                then
                    -- This is beginning of exponent, grab from here to
                    -- end of IMAGE for the value of the exponent
                    Exponent := Exponent +
                        integer'value (IMAGE (index + 1 .. IMAGE'last));
                    exit;
                else
                    -- Check that this is a good digit and put into result
                    if hex_digit >= ibase then
                        raise DATA_ERROR;
                    elsif RESULT <= Lower_Limit then
                        Skipped_Exponent := Skipped_Exponent + 1;
                    else
                        begin
                            float_hex_digit := TGT.Longest_Float(hex_digit);
                            RESULT := RESULT * base - float_hex_digit;

                        exception
                            when NUMERIC_ERROR =>
                              raise DATA_ERROR;
                        end;
                        Exponent := Exponent - 1;
                    end if;  -- hex_digit >= ibase
                end if;  -- E is exponent

            when others =>
                -- there shouldn't be any others if the IMAGE was OK
                raise DATA_ERROR;
            end case;
        end loop;


	if not saw_radix_point then
	    Exponent := 0;  -- reset to zero
	end if;

        begin

            Exponent := Exponent + Skipped_Exponent;

              -- In order to maintain accuracy of the final result,
              -- we must NOT raise base to a negative exponent, as
              -- that causes multiple divides.  Instead, raise base
              -- to the abs Exponent, and either multiply or divide
              -- by the resulting exponent_part.


            exponent_part := TGT.Longest_Float(
			java.lang.Math.pow(Long_Float(base),
	  	      	   	           Long_Float (abs Exponent)));


            if Exponent < 0 then
                RESULT := RESULT / exponent_part;
            else
                RESULT := RESULT * exponent_part;
            end if;
            RESULT := sign * RESULT;
            return RESULT;
        exception
            when NUMERIC_ERROR =>
              raise DATA_ERROR;
            end;

    end FLOAT_VALUE;

end System.RTS.Float_IO_Support;
