-- ****************************************************************************
--
--  COPYRIGHT NOTICE:
--    This software is protected as an unpublished work under the Copyright
--    Act of 1976. Created 1986 (C). All rights reserved. Alsys.
--    "Alsys" is a trademark of Alsys, S.A.
--
--  PACKAGE SPECIFICATION: DOS
--  --------------------------
--
--  NT-DOS Version 1.00 07 Sep 1994, 09:19
--
-- ****************************************************************************


with win32, Win32.WinBase, Win32.Windef, Win32.Winnt;
with SYSTEM;
with Unchecked_Conversion;
package DOS is

-- ****************************************************************************
--
--  Outline:
--  --------
--
--     This package simulates the interface to DOS services by making calls to
--     the win32 API.  Most of the DOS 3.10 functions are supported.
--
--  Contents:
--  ---------
--
--     (0)  Types Definition
--     (1)  Error Handling
--     (2)  Note on String Arguments
--     (3)  Note on Character Type
--     (4)  Note on Drive Letters
--     (5)  Attribute Definitions
--     (6)  Open Mode Definitions
--     (7)  File Handle Definitions
--     (8)  Type Definitions
--     (9)  INT 21 Interface Functions
--    (10)  Environment Access Routines
--    (11)  Absolute Disk Access Routines
--    (12)  Control/Break Intercept
--    (13)  Conversion Routines
--
--  Note: references of the form TR-6-23 are to the DOS Version 3.10
--  Technical Reference Manual, IBM publication 6138536, Feb 1985.
--
-- ****************************************************************************

   package WinBase renames Win32.Winbase;
   package Winnt   renames Win32.Winnt;
   package WinDef  renames Win32.WinDef;
   
   -- package Winnt renames Win32
--	    *************************
--	    * (0)  Types Definition *
--	    *************************

   subtype BYTE_8 is WIN32.BYTE;
   subtype WORD_16 is WIN32.WORD;
   subtype INTEGER_32 is INTEGER;



--	    ***********************
--	    * (1)  Error Handling *
--	    ***********************

-- The following exception is signalled if any DOS function signals an error
-- It is also raised for a disk full condition on a write (which DOS does not
-- consider to be an error).

   DOS_ERROR: exception;



--	    *********************************
--	    * (2)  Note on String Arguments *
--	    *********************************

-- In contexts where DOS would normally require an ASCIIZ string, the parameter
-- passed by the Ada program is a normal Ada string. No terminator is required,
-- for example the string can either be exactly the right length, or right blank
-- filled. Alternatively a null terminator may be included, in which case any
-- characters following the NUL character in the string are ignored.



--	    *******************************
--	    * (3)  Note on Character Type *
--	    *******************************

-- For all interface procedures which have parameters or return results of type
-- CHARACTER (or STRING), the corresponding DOS function can in fact handle the
-- full range of ASCII characters (with codes 0-255). For example, a file name
-- can contain accented letters which have codes greater than 127. If the
-- calling program is compiled with checks off, no difficulties arise. Even if
-- checks are on, no checks are performed as long as the only operations are
-- assignment of character values.



--	    ******************************
--	    * (4)  Note on Drive Letters *
--	    ******************************

-- Several of the DOS calls need a drive to be specified or return a drive. The
-- DOS usage is inconsistent in that sometimes 0 means drive A and sometimes 1
-- means drive A. To avoid this confusion, all drive specifications in the Ada
-- interface use an upper case letter to specify a drive. When a drive parameter
-- is supplied, the convention is that '@' (which is the default value of the
-- parameter) means the current drive.

   subtype DRIVE_LETTER is CHARACTER range '@' .. 'Z';



--	    ******************************
--	    * (5)  Attribute Definitions *
--	    ******************************

-- Several of the functions require or return a file attribute value, as defined
-- on TR-5-11. The following definitions can be used to construct such values.
-- These are all flag definitions. If the flag is on, then the corresponding
-- attribute is set. The definition ATTR_NORMAL can be used when no attribute
-- is to be set. The type definition here is provided to document clearly where
-- attribute values of this type appear.

   subtype ATTR_TYPE is WIN32.DWORD;
   subtype SHORT_ATTR_TYPE is WIN32.BYTE;

   ATTR_READ_ONLY: constant := WinNT.FILE_ATTRIBUTE_READONLY;
   ATTR_HIDDEN: constant := WinNT.FILE_ATTRIBUTE_HIDDEN;
   ATTR_SYSTEM: constant := WinNT.FILE_ATTRIBUTE_SYSTEM;
-- ATTR_VOLUME: constant := 0; -- not supported in NT-DOS
   ATTR_DIREC: constant := WinNT.FILE_ATTRIBUTE_DIRECTORY;
   ATTR_ARCHIVE: constant := WinNT.FILE_ATTRIBUTE_ARCHIVE;
   ATTR_NORMAL: constant := WinNT.FILE_ATTRIBUTE_NORMAL;



--	    ******************************
--	    * (6)  Open Mode Definitions *
--	    ******************************

-- The following definitions can be used to construct the MODE parameter for
-- the OPEN_FILE routine. See TR-6-113/121 for usage details on this parameter.
-- The type definition here is provided to document clearly where open mode
-- code values of this type appear.

   subtype OPEN_TYPE is WIN32.DWORD;
   use type WIN32.DWORD;

   OPEN_READ_ONLY:  OPEN_TYPE := WinNT.Generic_Read;
   OPEN_WRITE_ONLY: OPEN_TYPE := WinNT.Generic_Write;
   OPEN_READ_WRITE: OPEN_TYPE := WinNT.Generic_Read + WinNT.Generic_Write;
   OPEN_EXCLUSIVE:  OPEN_TYPE := 0;
   OPEN_DENY_WRITE: OPEN_TYPE := -WinNT.File_Share_Write-1; -- not WinNT.File_Share_Write;
   OPEN_DENY_READ:  OPEN_TYPE := -WinNT.File_Share_Read-1;  -- not WinNT.File_Share_Read;
   OPEN_DENY_NONE:  OPEN_TYPE := WinNT.File_Share_Read + WinNT.File_Share_Write;
-- OPEN_NO_INHERIT: OPEN_TYPE := 0; -- not supported in NT-DOS



--	    ********************************
--	    * (7)  File Handle Definitions *
--	    ********************************

-- The following definitions are for the standard file handles which are
-- automatically opened and available for use with READ and WRITE (TR-6-6-46)
-- Note that in practice handles are integers in the range 0..19, and can for
-- example be used to index arrays. However, such usage does not conform to
-- to the official DOS specification, which does not guarantee that file
-- handle values have any particular range. The type definition here is
-- provided to document clearly where handle values are used.

   -- subtype HANDLE_TYPE is Win32.HFile;
   subtype HANDLE_TYPE is WinNT.Handle;
 
   function To_Handle is new Unchecked_Conversion (Source => Win32.Dword,
                                                   Target => WinNT.Handle);
   STANDARD_INPUT: constant HANDLE_TYPE :=
      To_HANDLE (WINBASE.STD_INPUT_HANDLE);
   STANDARD_OUTPUT: constant HANDLE_TYPE :=
      To_HANDLE (WINBASE.STD_OUTPUT_HANDLE);
   STANDARD_ERROR : constant HANDLE_TYPE :=
      To_HANDLE (WINBASE.STD_ERROR_HANDLE);
-- STANDARD_AUX : constant HANDLE_TYPE := 0 -- not supported in NT-DOS
-- STANDARD_PRINT : constant HANDLE_TYPE := 0 -- not supported in NT-DOS



--	    *************************
--	    * (8)  Type Definitions *
--	    *************************

-- The following record definitions are used to pass date and time values

   type DATE_RECORD is
      record
	 YEAR  : INTEGER range 1980..2099;
	 MONTH : INTEGER range 1..12;
	 DAY   : INTEGER range 1..31;
      end record;

   for DATE_RECORD use
      record
	 YEAR  at 0 range 0 .. 15;
	 MONTH at 2 range 0 .. 15;
	 DAY   at 4 range 0 .. 15;
      end record;

   type TIME_RECORD is
      record
	 HOURS	    : INTEGER range 0..23;
	 MINUTES    : INTEGER range 0..59;
	 SECONDS    : INTEGER range 0..59;
	 HUNDREDTHS : INTEGER range 0..99;
      end record;

   for TIME_RECORD use
      record
	 HOURS	    at 0 range 0 .. 15;
	 MINUTES    at 2 range 0 .. 15;
	 SECONDS    at 4 range 0 .. 15;
	 HUNDREDTHS at 6 range 0 .. 15;
      end record;

   subtype DAY_OF_WEEK_TYPE is INTEGER range 0..6;  -- 0=Sunday, 6=Saturday


-- The following types are packed dates and times, as used in file directory
-- entries (functions SET_FILE_DATE_TIME and GET_FILE_DATE_TIME, and also in
-- FIND_FIRST and FIND_NEXT). They may be converted to and from the record
-- structures TIME_RECORD and DATE_RECORD using the PACK and UNPACK functions.


-- Packed date (see DOS-TR-5-12)

--     <  most signif byte  >	 < least signif byte  >
--     15 14 13 12 11 10 9  8	 7  6  5  4  3	2  1  0
--     y  y  y	y  y  y  y  m	 m  m  m  d  d	d  d  d
--
-- yyyyyyy is the binary number of years (0-119 for 1980-2099)
-- mmmmmm is the binary number of months (1-12)
-- ddddd is the binary number of days (1-31)

   subtype PACKED_DATE is WORD_16;


-- Packed time (see DOS-TR-5-13)

--     <  most signif byte  >	 < least signif byte  >
--     15 14 13 12 11 10 9  8	 7  6  5  4  3	2  1  0
--     h  h  h	h  h  m  m  m	 m  m  m  x  x	x  x  x
--
-- hhhhh is the binary number of hours (0-23)
-- mmmmmm is the binary numbner of minutes (0-59)
-- xxxxx is the binary number of two second increments (0-29)

   subtype PACKED_TIME is WORD_16;


-- Record used to pass flags to IOCTL_SET_DEVICE_INFO and to return flags
-- from IOCTL_GET_DEVICE_INFO. See TR-6-134 for details of these flags.

   type DEVICE_INFO_RECORD is
      record
	 BIT15	   : BOOLEAN;	-- Bit 15 (reserved)
	 CTRL	   : BOOLEAN;	-- Bit 14 (true if control strings processed)
	 NETWORK   : BOOLEAN;	-- Bit 13 (true if device is on network)
	 BIT12	   : BOOLEAN;	-- Bit 12 (reserved)
	 BIT11	   : BOOLEAN;	-- Bit 11 (reserved)
	 BIT10	   : BOOLEAN;	-- Bit 10 (reserved)
	 BIT9	   : BOOLEAN;	-- Bit 9 (reserved)
	 BIT8	   : BOOLEAN;	-- Bit 8 (reserved)
	 ISDEV	   : BOOLEAN;	-- Bit 7 (true if device, false if disk file)
	 EOF	   : BOOLEAN;	-- Bit 6 (true if at end of file)
	 BINARY    : BOOLEAN;	-- Bit 5 (true if binary (raw) mode)
	 BIT4	   : BOOLEAN;	-- Bit 4 (reserved)
	 ISCLK	   : BOOLEAN;	-- Bit 3 (true if clock device)
	 ISNUL	   : BOOLEAN;	-- Bit 2 (true if NUL device)
	 ISCOT	   : BOOLEAN;	-- Bit 1 (true if console output device)
	 ISCIN	   : BOOLEAN;	-- Bit 0 (true if console input device)
      end record;


   for DEVICE_INFO_RECORD use
      record
	 BIT15	   at 1 range 7 .. 7;
	 CTRL	   at 1 range 6 .. 6;
	 NETWORK   at 1 range 5 .. 5;
	 BIT12	   at 1 range 4 .. 4;
	 BIT11	   at 1 range 3 .. 3;
	 BIT10	   at 1 range 2 .. 2;
	 BIT9	   at 1 range 1 .. 1;
	 BIT8	   at 1 range 0 .. 0;
	 ISDEV	   at 0 range 7 .. 7;
	 EOF	   at 0 range 6 .. 6;
	 BINARY    at 0 range 5 .. 5;
	 BIT4	   at 0 range 4 .. 4;
	 ISCLK	   at 0 range 3 .. 3;
	 ISNUL	   at 0 range 2 .. 2;
	 ISCOT	   at 0 range 1 .. 1;
	 ISCIN	   at 0 range 0 .. 0;
      end record;



--	    ***********************************
--	    * (9)  INT 21 Interface Functions *
--	    ***********************************

-- The following section contains INT 21 interface definitions, arranged by
-- function code. For further details on these functions see TR-6-49/178



--	    ************************************
--	    * Function 00H - Program Terminate *
--	    ************************************

-- No interface is provided to this function, use function 4C (EXIT) instead



--	    *******************************************
--	    * Function 01H - Keyboard Input with Echo *
--	    *******************************************

-- Waits for character to be read from standard input (TR-6-50). Two calls are
-- needed (the first returning 0) for special function keys.

   function READ_KBD_ECHO return CHARACTER;



--	    *********************************
--	    * Function 02H - Display Output *
--	    *********************************

-- Outputs character to standard output device

   procedure DISPLAY_CHAR (CHR : CHARACTER);



--	    **********************************
--	    * Function 03H - Auxiliary Input *
--	    **********************************

-- Reads one character from the standard auxiliary device

-- function AUXIN return CHARACTER; -- not supported in NT-DOS



--	    ***********************************
--	    * Function 04H - Auxiliary Output *
--	    ***********************************

-- Writes one character to the standard auxiliary device

-- procedure AUXOUT (CHR : CHARACTER); -- not supported in NT-DOS



--	    *********************************
--	    * Function 05H - Printer Output *
--	    *********************************

-- Outputs character to standard printer device

-- procedure PRINT_CHAR (CHR : CHARACTER); -- not supported in NT-DOS



--	    *************************************
--	    * Function 06H - Direct Console I/O *
--	    *************************************

-- Direct console I/O differs from the corresponding normal (non-direct) calls
-- only in that there is no test for a Ctrl-Break check.

-- Print character direct (cf DISPLAY_CHAR). Note: the character value 16#FF#
-- canot be printed in direct mode, it is replaced with a space (16#20#).

-- procedure DISPLAY_CHAR_DIRECT (CHR : CHARACTER); -- not supported in NT-DOS


-- Read keyboard direct with echo (cf READ_KBD_ECHO)

   function READ_KBD_DIRECT_ECHO return CHARACTER;



--	    ****************************************
--	    * Function 07H - Direct Keyboard Input *
--	    ****************************************

-- Read character from keyboard. This is like function 08H, except that there
-- is no test for Ctrl-Break (or like function 06H without the echo)

   function READ_KBD_DIRECT_NO_ECHO return CHARACTER;



--	    *********************************
--	    * Function 08H - Keyboard Input *
--	    *********************************

-- Read character from keyboard. This is like function 01H, except that the
-- character is not echoed to the screen (there is a test for Ctrl-Break).

   function READ_KBD_NO_ECHO return CHARACTER;



--	    *******************************
--	    * Function 09H - Print String *
--	    *******************************

-- No interface is provided to this function. Use function 40H (WRITE) with
-- handle 1 (standard_output) instead. This has the advantage of not needing
-- to worry about the special case of the $ sign.



--	    ******************************************
--	    * Function 0AH - Buffered Keyboard Input *
--	    ******************************************

-- Read specified number of characters from keyboard. The returned string is
-- no longer than the given count, but may be shorter if fewer than the
-- maximum number of characters are entered before the carriage return.
-- The string returned does not include the terminating carriage return.

-- The maximum value of MAX_KEYS is 254. Larger values are treated as
-- if 254 had been specified. Note that the count does not include the
-- terminating carriage return used to end the entry of the string.

   function READ_KBD_BUFFERED (MAX_KEYS : INTEGER) return STRING;



--	    *************************************
--	    * Function 0BH - Check Input Status *
--	    *************************************

-- Check if a character is available from the keyboard (standard input), also
-- does a standard Ctrl-Break test. Return TRUE if a key is available.

   function KBD_DATA_AVAILABLE return BOOLEAN;



--	    ***************************************************
--	    * Function 0CH - Clear Buffer and Invoke Function *
--	    ***************************************************

-- Keyboard read functions with flush of keyboard buffer before starting.
-- Note: many "resident" programs intefere with the keyboard buffer, and
-- may result in the flushing action of these routines being ineffective.


-- Read keyboard buffered with flush (cf READ_KBD_BUFFERED)

   function READF_KBD_BUFFERED (MAX_KEYS : INTEGER) return STRING;


-- Read keyboard with echo and flush (cf READ_KBD_ECHO)

   function READF_KBD_ECHO return CHARACTER;


-- Read keyboard with no echo and flush (cf READ_KBD_NO_ECHO)

   function READF_KBD_NO_ECHO return CHARACTER;


-- Read keyboard direct with echo and flush (cf READ_KBD_DIRECT_ECHO)

   function READF_KBD_DIRECT_ECHO return CHARACTER;


-- Read keyboard direct with no echo and flush (cf READ_KBD_DIRECT_NO_ECHO)

   function READF_KBD_DIRECT_NO_ECHO return CHARACTER;



--	    *****************************
--	    * Function 0DH - Disk Reset *
--	    *****************************

-- Flush all file buffers

-- procedure DISK_RESET; -- not supported in NT-DOS



--	    *******************************
--	    * Function 0EH - Select Drive *
--	    *******************************

-- Select specified drive as the current drive.

   procedure SELECT_DRIVE (DRIVE : DRIVE_LETTER);


-- Get number of drives. Warning: this is not a very useful number in most
-- versions of DOS, it does not correspond to the actual number of drives.

   function NUMBER_OF_DRIVES return INTEGER;



--	    ****************************
--	    * Function 0FH - Open File *
--	    ****************************

-- No interface is provided to this function, use 3D (OPEN_FILE) instead



--	    *****************************
--	    * Function 10H - Close File *
--	    *****************************

-- No interface is provided to this function, use 3E (CLOSE_FILE) instead



--	    *****************************************
--	    * Function 11H - Search for First Entry *
--	    *****************************************

-- No interface is provided to this function, use 4E (FIND_FIRST) instead



--	    ****************************************
--	    * Function 12H - Search for Next Entry *
--	    ****************************************

-- No interface is provided to this function, use 4F (FIND_NEXT)



--	    ******************************
--	    * Function 13H - Delete File *
--	    ******************************

-- No interface is provided to this function, use 41 (DELETE_FILE) instead



--	    **********************************
--	    * Function 14H - Sequential Read *
--	    **********************************

-- No interface is provided to this function, use 3F (READ) instead



--	    ***********************************
--	    * Function 15H - Sequential Write *
--	    ***********************************

-- No interface is provided to this function, use 40 (WRITE) instead



--	    ******************************
--	    * Function 16H - Create File *
--	    ******************************

-- No interface is provided to this function, use 3C (CREATE_FILE) instead



--	    ******************************
--	    * Function 17H - Rename File *
--	    ******************************

-- No interface is provided to this function, use 56 (RENAME_FILE) instead



--	    *******************************
--	    * Function 19H - Current Disk *
--	    *******************************

-- Returns currently selected drive

   function CURRENT_DRIVE return DRIVE_LETTER;



--	    ********************************************
--	    * Function 1AH - Set Disk Transfer Address *
--	    ********************************************

-- No interface is provided to this function.



--	    *************************************************************
--	    * Function 1BH - Allocation Table Information Current Drive *
--	    *************************************************************

-- No interface is provided to this function (use function 1CH with DRIVE='@')



--	    ****************************************************************
--	    * Function 1CH - Allocation Table Information, Specified Drive *
--	    ****************************************************************

-- Returns information about the allocation table for the specified drive.
-- Note that the first parameter returned is the media description byte
-- itself, not a pointer to the byte.

   procedure ALLOCATION_INFO (
      DRIVE                  : DRIVE_LETTER := '@';
      MEDIA_DESCRIPTION_BYTE : out WORD_16;
      NUM_ALLOCATION_UNITS   : out WORD_16;
      NUM_SECTORS_ALLOC_UNIT : out WORD_16;
      PHYSICAL_SECTOR_SIZE   : out WORD_16);



--	    ******************************
--	    * Function 21H - Random Read *
--	    ******************************

-- No interface is provided to this function, use 3F (READ) instead



--	    *******************************
--	    * Function 22H - Random Write *
--	    *******************************

-- No interface is provided to this function, use 40 (WRITE) instead



--	    ****************************
--	    * Function 23H - File Size *
--	    ****************************

-- No interface is provided to this function, use 41 (FILE_SIZE) instead



--	    ********************************************
--	    * Function 24H - Set Relative Record Field *
--	    ********************************************

-- No interface is provided to this function, use 41 (LSEEK) instead



--	    ***************************************
--	    * Function 25H - Set Interrupt Vector *
--	    ***************************************

-- No interface is provided to this function



--	    *********************************************
--	    * Function 26H - Create New Program Segment *
--	    *********************************************

-- No interface is provided to this function, use 4B (EXEC) instead



--	    ************************************
--	    * Function 27H - Random Block Read *
--	    ************************************

-- No interface is provided to this function, use 3F (READ) instead



--	    *************************************
--	    * Function 28H - Random Block Write *
--	    *************************************

-- No interface is provided to this function, use 40 (WRITE) instead



--	    *********************************
--	    * Function 29H - Parse Filename *
--	    *********************************

-- No interface is provided to this function, since it is for use with FCB's



--	    ***************************
--	    * Function 2AH - Get Date *
--	    ***************************

-- Return current date

   function DATE return DATE_RECORD;


-- Return day of week. Note: although earlier versions of the DOS manual
-- have not documented the day of week return, it does in fact work in all
-- versions of DOS from 2.0 on and is thus fully implemented.

   function DAY_OF_WEEK return DAY_OF_WEEK_TYPE;



--	    ***************************
--	    * Function 2BH - Set Date *
--	    ***************************

-- Sets the date. If the date is invalid, the set operation is ignored. The
-- function form of this call indicates whether the set succeeded (TRUE
-- returned), or failed due to an invalid date (FALSE returned).

   procedure SET_DATE (DATE : DATE_RECORD);
   function  SET_DATE (DATE : DATE_RECORD) return BOOLEAN;



--	    ***************************
--	    * Function 2CH - Get Time *
--	    ***************************

-- Returns the time: hours, minutes, seconds and hundredths of seconds

   function TIME return TIME_RECORD;



--	    ***************************
--	    * Function 2DH - Set Time *
--	    ***************************

-- Sets the time. If the time is invalid, the set operation is ignored. The
-- function form of this call indicates whether the set succeeded (TRUE result
-- returned), or failed due to an invalid time (FALSE result returned).

   procedure SET_TIME (TIME : TIME_RECORD);
   function  SET_TIME (TIME : TIME_RECORD) return BOOLEAN;



--	    ******************************************
--	    * Function 2EH - Set/Reset Verify Switch *
--	    ******************************************

-- Sets the verify switch. The SWITCH argument is FALSE for verify mode
-- off (DOS setting 0) and TRUE for BREAK ON (DOS setting 1).

-- procedure SET_VERIFY_SWITCH (SWITCH : BOOLEAN); -- not supported in NT-DOS



--	    ********************************************
--	    * Function 2FH - Get Disk Transfer Address *
--	    ********************************************

-- No interface is provided to this function



--	    *****************************************
--	    * Function 30H - Get DOS Version Number *
--	    *****************************************

-- Returns the DOS version number

-- procedure GET_VERSION (MAJOR : out INTEGER; MINOR : out INTEGER);
   -- not supported in NT-DOS


--	    **********************************************
--	    * Function 31H - Terminate and Stay Resident *
--	    **********************************************

-- No interface is provided to this function



--	    ***********************************
--	    * Function 33H - Ctrl-Break Check *
--	    ***********************************

-- Get the state of BREAK (Ctrl-Break checking). The returned result is FALSE
-- for BREAK OFF mode (DOS setting 0) and TRUE for BREAK ON (DOS setting 1)

-- function  CTRL_BREAK_ON return BOOLEAN; -- not supported in NT-DOS


-- Set the state of BREAK (Ctrl-Break checking). The SWITCH argument is FALSE
-- for BREAK OFF mode (DOS setting 0) and TRUE for BREAK ON (DOS setting 1)

-- procedure SET_CTRL_BREAK_STATE (SWITCH : BOOLEAN); -- not supported in NT-DOS



--	    *****************************
--	    * Function 35H - Get Vector *
--	    *****************************

-- No interface is provided to this function



--	    **************************************
--	    * Function 36H - Get Disk Free Space *
--	    **************************************

-- Get disk free space for specified drive. The sectors per cluster value
-- (SEC_PER_CLUST) is set to -1 if the specified drive letter is invalid.

   procedure GET_DISK_FREE_SPACE (
      DRIVE           : DRIVE_LETTER := '@'; -- Drive
      CLUSTERS_AVAIL  : out WORD_16;	     -- Available clusters
      CLUST_PER_DRV   : out WORD_16;	     -- Clusters per drive
      BYTES_PER_SEC   : out WORD_16;	     -- Bytes per sector
      SEC_PER_CLUST   : out WORD_16);	     -- Sectors per cluster



--	    ***********************************************************
--	    * Function 38H - Get or Set Country Dependent Information *
--	    ***********************************************************

-- Return country dependent information. This function is available in full
-- full form only in DOS 3.00 or 3.10. If this function is used in earlier
-- versions, then only the date/time format, currency symbol, thousands
-- separator, and decimal separator are defined, all other fields are set to
-- zero values. See TR-6-103 for description of the format of the fields in
-- this record.

   type COUNTRY_INFO_RECORD is
      record
	 DATE_FORMAT	   : WORD_16;	     -- 0=USA, 1=Europe, 2=Japan
	 CURRENCY_SYMBOL   : STRING(1..5);   -- Currency symbol, null term
	 THOUSANDS_SEP	   : STRING(1..2);   -- Thousands separator, null term
	 DECIMAL_SEP	   : STRING(1..2);   -- Decimal separator, null term
	 DATE_SEP	   : STRING(1..2);   -- Date separator, null term
	 TIME_SEP	   : STRING(1..2);   -- Time separator, null term
	 CURSYM_BEFORE_VAL : BYTE_8;	     -- Currency format flags
	 CURRENCY_DIGITS   : BYTE_8;	     -- Currency decimal digits
	 TIME_FORMAT	   : BYTE_8;	     -- 0/1 for 12/24-hour clock
	 CASE_MAP_ADDRESS  : SYSTEM.ADDRESS; -- Case map call address
	 DATA_SEP	   : STRING(1..2);   -- Data list separator, null term
	 RESERVED	   : STRING(1..10);  -- 10 byte area reserved for DOS
      end record;

  for COUNTRY_INFO_RECORD use
     record
	DATE_FORMAT	  at  0 range 0 .. 15;
	CURRENCY_SYMBOL   at  2 range 0 .. 39;
	THOUSANDS_SEP	  at  7 range 0 .. 15;
	DECIMAL_SEP	  at  9 range 0 .. 15;
	DATE_SEP	  at 11 range 0 .. 15;
	TIME_SEP	  at 13 range 0 .. 15;
	CURSYM_BEFORE_VAL at 15 range 0 ..  7;
	CURRENCY_DIGITS   at 16 range 0 ..  7;
	TIME_FORMAT	  at 17 range 0 ..  7;
	CASE_MAP_ADDRESS  at 18 range 0 .. 31;
	DATA_SEP	  at 22 range 0 .. 15;
	RESERVED	  at 24 range 0 .. 79;
     end record;

-- The following procedure obtains information for the specified country. If
-- COUNTRY_CODE is 0 on entry, then information for the currently configured
-- country is returned, and COUNTRY_CODE is reset to the code for this country.
-- In versions of DOS before 3.00, the information is always for the current
-- country, and COUNTRY_CODE is always set to 1 on return.

   procedure GET_COUNTRY_INFO (
      COUNTRY_CODE : in out INTEGER;
      COUNTRY_INFO : out COUNTRY_INFO_RECORD);


-- Set country code. This function is ignored in versions of DOS before 3.00

   procedure SET_COUNTRY_CODE (COUNTRY_CODE : INTEGER);


-- Routine to make use of the CASE_MAP_ADDRESS returned above to fold lower
-- case letters (the definition of a letter depends on the country). Note that
-- contrary to what the documentation on TR-6-104 implies, the routine only
-- affects characters with a code greater than 127 (16#7F#). For characters
-- in the range 0-127 (including normal lower case letters), the character is
-- returned unchanged. If this routine is called with CASE_MAP_ADDRESS zero
-- as it will be if GET_COUNTRY_INFO is used with a DOS version before 3.00,
-- then the character is always returned unchanged.

   function UPPER_CASE (
      CHAR : CHARACTER;
      CASE_MAP_ADDRESS : SYSTEM.ADDRESS)
      return CHARACTER;



--	    **************************************
--	    * Function 39H - Create Subdirectory *
--	    **************************************

-- Creates the specified directory

   procedure MKDIR (DIRECTORY_NAME : STRING);



--	    **************************************
--	    * Function 3AH - Remove Subdirectory *
--	    **************************************

-- Removes the specified directory

   procedure RMDIR (DIRECTORY_NAME : STRING);



--	    *******************************************
--	    * Function 3BH - Change Current Directory *
--	    *******************************************

-- Changes the current directory to the specified directory

   procedure CHDIR (DIRECTORY_NAME : STRING);



--	    ******************************
--	    * Function 3CH - Create File *
--	    ******************************

-- Creates a new file or truncates an old file to zero length.
-- The value returned is the file handle.

   function CREATE_FILE (
      FILE_NAME : STRING;
      ATTRIBUTE : ATTR_TYPE := ATTR_NORMAL)
      return HANDLE_TYPE;



--	    ****************************
--	    * Function 3DH - Open File *
--	    ****************************

-- Opens an existing file. The default mode is read/write exclusive. For
-- other modes, see TR-6-111/121 and definitions at start of this module.
-- The value returned is the file handle. As indicated by the specification,
-- the default is to open the file in read/write compatibility mode. If
-- SHARE is loaded (for example in a network environment), then compatibility
-- mode is treated as a request for exclusive read/write access.

-- In versions of DOS before 3.00, file sharing modes and the inheritance flag
-- are ignored (but do not cause an error to be signalled).

   function OPEN_FILE (
      FILE_NAME : STRING;
      OPEN_MODE : OPEN_TYPE := OPEN_READ_WRITE)
      return HANDLE_TYPE;



--	    *****************************
--	    * Function 3EH - Close File *
--	    *****************************

-- Close specified file handle

   procedure CLOSE_FILE (HANDLE : HANDLE_TYPE);



--	    ****************************
--	    * Function 3FH - Read File *
--	    ****************************

-- Read from specified file. The buffer is given as a STRING. The length of
-- the string indicates the maximum number of characters to be read. The
-- value in COUNT on return is the actual number of characters read.

   procedure READ (
      BUFFER : out STRING;
      COUNT  : out INTEGER;
      HANDLE : HANDLE_TYPE := STANDARD_INPUT);


-- Read from specified file. The buffer is given as a SYSTEM.ADDRESS. The
-- COUNT value on entry is the maximum number of characters to be read, and
-- the caller is responsible for ensuring that it does not exceed the actual
-- length of the buffer (or information can be overwritten without warning).
-- The value in COUNT on return is the actual number of bytes read.

   procedure READ_ADR (
      BUFFER : SYSTEM.ADDRESS;
      COUNT  : in out INTEGER;
      HANDLE : HANDLE_TYPE := STANDARD_INPUT);



--	    *****************************
--	    * Function 40H - Write File *
--	    *****************************

-- Write to specified file. The buffer is given as a STRING, and the length
-- of the string indicates the number of characters to be written. If the
-- number of bytes written does not match the number of bytes specified, then
-- DOS_ERROR is raised and a disk full condition is signalled.

   procedure WRITE (
      BUFFER : STRING;
      HANDLE : HANDLE_TYPE := STANDARD_OUTPUT);


-- Write to specified file. The buffer is given as a SYSTEM.ADDRESS. The
-- COUNT value on entry is the number of characters to be written. If the
-- number of bytes written does not match the number of bytes specified, then
-- DOS_ERROR is raised and a disk full condition is signalled.

   procedure WRITE_ADR (
      BUFFER : SYSTEM.ADDRESS;
      COUNT  : INTEGER;
      HANDLE : HANDLE_TYPE := STANDARD_OUTPUT);



--	    ******************************
--	    * Function 41H - Delete File *
--	    ******************************

-- Deletes an existing file

   procedure DELETE_FILE (FILE_NAME : STRING);



--	    ***********************************************
--	    * Function 42H - Move File Read/Write Pointer *
--	    ***********************************************

-- Moves the read/write pointer according to the method specified. There are
-- two forms, one used only to set the position, the other used to set it and
-- return the resulting position. In the second form, the result is the
-- position of the read/write pointer after it has been moved.	Named constants
-- are provided for the METHOD parameter.

   FROM_START	: constant := WINBASE.FILE_BEGIN;
   FROM_CURRENT : constant := WINBASE.FILE_CURRENT;
   FROM_END	: constant := WINBASE.FILE_END;

   procedure LSEEK (
      HANDLE : HANDLE_TYPE;
      OFFSET : INTEGER;
      METHOD : INTEGER);

   function  LSEEK (
      HANDLE : HANDLE_TYPE;
      OFFSET : INTEGER;
      METHOD : INTEGER)
      return INTEGER;


-- Get size of specified file. This function does an LSEEK to determine the
-- file offset. The current position is saved and restored, so that it is
-- not affected.

   function FILE_SIZE (HANDLE : HANDLE_TYPE) return INTEGER;


-- Return current position of file

   function FILE_POSITION (HANDLE : HANDLE_TYPE) return INTEGER;



--	    ***********************************
--	    * Function 43H - Change File Mode *
--	    ***********************************

-- Get mode of specified file (see definition of ATTR_TYPE values)

   function GET_MODE (FILE_NAME : STRING) return ATTR_TYPE;


-- Set mode of specified value to specified attribute value (see definition
-- of ATTR_TYPE values). Note that attributes 16#08# (volume) and 16#10#
-- (directory) cannot be changed using this function.

   procedure CHMOD (FILE_NAME : STRING; ATTRIBUTE : ATTR_TYPE);



--	    ******************************************
--	    * Function 44H - I/O Control for Devices *
--	    ******************************************


-- Get device information for specified file

   procedure IOCTL_GET_DEVICE_INFO (
      HANDLE	  : HANDLE_TYPE;
      DEVICE_INFO : out DEVICE_INFO_RECORD); -- not fully supported in NT-DOS


-- The remaining I/O control functions are not supported in NT-DOS


-- Set device information for specified file. Note that the setting of bits
-- 8-15 of the device information is ignored, since these bits must be 0.

-- procedure IOCTL_SET_DEVICE_INFO (
--    HANDLE	  : HANDLE_TYPE;
--    DEVICE_INFO : DEVICE_INFO_RECORD);


-- Read bytes from device control channel corresponding to specified handle.
-- The buffer is a string whose length is the maximum number of bytes to be
-- be read. The value in COUNT on exit is the actual number bytes read.

-- procedure IOCTL_READ (
--    HANDLE : HANDLE_TYPE;
--    BUFFER : out STRING;
--    COUNT  : out INTEGER);


-- Read bytes from device control channel corresponding to specified drive.
-- The buffer is a string whose length is the maximum number of bytes to be
-- be read. The value in COUNT on exit is the actual number bytes read.

-- procedure IOCTL_READ_DRIVE (
--    DRIVE  : DRIVE_LETTER := '@';
--    BUFFER : out STRING;
--    COUNT  : out INTEGER);


-- Write bytes to device control channel corresponding to specified handle.
-- The buffer is a string whose length is the maximum number of bytes to be
-- be written. The value in COUNT on exit is the actual number bytes written.

-- procedure IOCTL_WRITE (
--    HANDLE : HANDLE_TYPE;
--    BUFFER : STRING;
--    COUNT  : out INTEGER);


-- Write bytes to device control channel corresponding to specified drive.
-- The buffer is a string whose length is the maximum number of bytes to be
-- be written. The value in COUNT on exit is the actual number bytes written.

-- procedure IOCTL_WRITE_DRIVE (
--    DRIVE  : DRIVE_LETTER := '@';
--    BUFFER : STRING;
--    COUNT  : out INTEGER);


-- Check input status. The channel is specified by a device handle. For a
-- file, TRUE is returned until end of file is reached. For a device, the
-- returned result indicates if the device is ready (function 06).

-- function IOCTL_INPUT_READY  (HANDLE : HANDLE_TYPE) return BOOLEAN;


-- Check output status. The channel is specified by a device handle. For a
-- file, TRUE is returned until end of file is reached. For a device, the
-- returned result indicates if the device is ready (function 07).

-- function IOCTL_OUTPUT_READY (HANDLE : HANDLE_TYPE) return BOOLEAN;


-- Check if block device can support removable media (function 08). This
-- function is only provided in DOS version 3.00 and beyond. If used in
-- DOS version 2, TRUE is returned for drives A and B and FALSE otherwise.

-- function IOCTL_IS_REMOVABLE (DRIVE : DRIVE_LETTER := '@') return BOOLEAN;


-- Test if logical device is associated with a network directory (returns
-- TRUE if so). This is only available in DOS 3.10. If used with earlier
-- versions of DOS, the returned result is always FALSE.

-- function IOCTL_IS_DRIVE_REMOTE (DRIVE : DRIVE_LETTER := '@') return BOOLEAN;


-- Test if handle is for a remote device (returns TRUE if so). This is only
-- available in DOS 3.10. If used with earlier versions of DOS, the result
-- is always FALSE.

-- function IOCTL_IS_FILE_REMOTE (HANDLE : HANDLE_TYPE) return BOOLEAN;


-- Control sharing delay and retry count (see TR-6-138). This is only
-- available in DOS 3.10. The call is simply ignored if used with earlier
-- versions of DOS.

-- procedure IOCTL_SET_SHARE (DDELAY  : INTEGER := 1;
--			      RETRIES : INTEGER := 3);



--	    ****************************************
--	    * Function 45H - Duplicate File Handle *
--	    ****************************************

-- Returns a new file handle for an open file that refers to the same file at
-- the same position.

   function DUP (HANDLE : HANDLE_TYPE) return HANDLE_TYPE;



--	    ************************************************
--	    * Function 46H - Force a Duplicate of a Handle *
--	    ************************************************

-- Forces a specified new file handle to refer to the same file as an existing
-- file handle. If the new file handle is open, it is closed first.

   procedure FORCDUP (
      EXISTING_HANDLE : HANDLE_TYPE;
      NEW_HANDLE      : HANDLE_TYPE);



--	    ****************************************
--	    * Function 47H - Get Current Directory *
--	    ****************************************

-- Returns the full path name of the current directory. The string returned
-- contains the directory name for the specified drive with no null terminator.

   function CURRENT_DIRECTORY (DRIVE : DRIVE_LETTER := '@') return STRING;



--	    **********************************
--	    * Function 48H - Allocate Memory *
--	    **********************************

-- No interface is provided to this function, use Ada allocators instead



--	    ******************************
--	    * Function 49H - Free Memory *
--	    ******************************

-- No interface is provided to this function, use Ada allocators instead



--	    **********************************************
--	    * Function 4AH - Change Size of Memory Block *
--	    **********************************************

-- No interface is provided to this function



--	    *********************************************
--	    * Function 4BH - Load and Execute a Program *
--	    *********************************************

-- Load another program into memory and execute it. The first parameter is
-- the file name, which must include drive, path and extension (COM or EXE,
-- there is no default extension). The second parameter is the command
-- line parameters (no terminating null or count is required). Note that
-- the default FCB's are not set up. No Ada program requires these, and in
-- fact virtually no program requires these, since they are provided for
-- compatibility with CP/M-86, and are not useful for DOS programs).

   procedure EXEC (
      PROGRAM_FILE : STRING;
      COMMAND_LINE : STRING  := "");


-- Load a copy of the DOS COMMAND interpretor using COMSPEC from the
-- environment, and then executes a single DOS command. THe parameter is
-- the complete DOS command (no null terminator is needed).

   procedure EXEC_COMMAND (DOS_COMMAND : STRING);


-- Load a copy of the DOS COMMAND interpretor using COMSPEC from the
-- environment, and read and execute DOS commands from the keyboard until
-- an EXIT command returns control to the EXEC_COMMANDS caller.

   procedure EXEC_COMMANDS;



--	    **************************************
--	    * Function 4CH - Terminate a Process *
--	    **************************************

-- Terminate current process. Note that normal end of job cleanup (closing
-- files, resetting the interrupt vector etc.) is performed before the exit.

   procedure EXIT_PROCESS (RETURN_CODE : WORD_16 := 0);



--	    **************************************************
--	    * Function 4DH - Get Return Code of a Subprocess *
--	    **************************************************

-- Gets the return code specified by another process executed by a call to
-- EXEC, EXEC_COMMAND or EXEC_COMMANDS. Eventually DOS is expected to allow
-- multi-tasking, and it may be possible to do overlapped work between an
-- EXEC call and the corresponding WAIT call. For now, the WAIT call has
-- no synchronization function and will typically immediately follow the
-- EXEC function.

   function WAIT_PROCESS return WORD_16;



--	    **********************************
--	    * Function 4EH - Find First File *
--	    **********************************


-- The file search commands make use of the following type definition for a
-- buffer used to save the context of the currently active search. The fields
-- in this record correspond to those defined for the FIND FIRST function
-- (TR-6-153). The same SEARCH_RECORD must be supplied for a given FIND_FIRST
-- call and all subsequent FIND_NEXT calls for the same search. Using multiple
-- SEARCH_RECORDs, several searches may proceed concurrently.

-- Note that the routines UNPACK_DATE and UNPACK_TIME are available to convert
-- the packed time and date values in this record to TIME and DATE records, and
-- that the routine GET_STRING can be used to convert the ASCIIZ string stored
-- in FILE_NAMX to an unconstrained string with no terminating 00H.

   MAX_FILENAME : constant := 256;		-- Maximum file name length

   type SEARCH_RECORD is
      record
	 RESERVED   : STRING(1..21);		-- reserved for DOS
	 FILE_ATTR  : SHORT_ATTR_TYPE;		-- attribute
	 FILE_TIME  : PACKED_TIME;		-- file creation/update time
	 FILE_DATE  : PACKED_DATE;		-- file creation/update date
	 FILE_SIZE  : INTEGER;			-- size of file
	 FILE_NAMX  : STRING(1..MAX_FILENAME);	-- name/ext of file (ASCIIZ)
      end record;

   for SEARCH_RECORD use
      record
	 RESERVED  at  0 range 0 .. 167;
	 FILE_ATTR at 21 range 0 ..   7;
	 FILE_TIME at 22 range 0 ..  15;
	 FILE_DATE at 24 range 0 ..  15;
	 FILE_SIZE at 26 range 0 ..  31;
	 FILE_NAMX at 30 range 0 .. MAX_FILENAME*8-1;
      end record;


-- FIND_FIRST initiates a search. The supplied SEARCH_RECORD is initialized
-- to control the search. If there is at least one matching file, then the
-- result in FILE_FOUND is TRUE, and the SEARCH_RECORD contains information for
-- the first matching file. If there are no matching files, then FILE_FOUND is
-- false. The ATTRIBUTE parameter controls which types of files match.

   procedure FIND_FIRST (
      FILE_PATTERN : STRING;
      SEARCH_BUF   : out SEARCH_RECORD;
      FILE_FOUND   : out BOOLEAN;
      ATTRIBUTE    : ATTR_TYPE := ATTR_NORMAL);



--	    *********************************
--	    * Function 4FH - Find Next File *
--	    *********************************

-- FIND_NEXT continues a search initiated by a previous call to FIND_FIRST.
-- If there are more matching files, then the result in FILE_FOUND is TRUE
-- and the SEARCH_RECORD contains information for the next matching file. If
-- there are no more matching files, then FILE_FOUND is false.

   procedure FIND_NEXT (
      SEARCH_BUF : in out SEARCH_RECORD;
      FILE_FOUND : out BOOLEAN);



--	    ******************************
--	    * Function 56H - Rename File *
--	    ******************************

-- Renames the specified file. Note that this call can change the directory
-- of a file. See TR-6-156 for details of the permitted arguments.

   procedure RENAME_FILE (OLD_FILE_NAME : STRING; NEW_FILE_NAME : STRING);



--	    *****************************************
--	    * Function 57H - Get/Set File Date/Time *
--	    *****************************************

-- Get the date and time of a file

   procedure GET_FILE_DATE_TIME (
      HANDLE	: HANDLE_TYPE;
      DATE	: out PACKED_DATE;
      TIME	: out PACKED_TIME);


-- Set the date and time of a file

   procedure SET_FILE_DATE_TIME (
      HANDLE	: HANDLE_TYPE;
      DATE	: PACKED_DATE;
      TIME	: PACKED_TIME);



--	    *************************************
--	    * Function 59H - Get Extended Error *
--	    *************************************


-- There is no interface to this function in the DOS package. Programs which
-- need to handle DOS error conditions should use the facilities of the DOSE
-- package which adds a set of definitions and routines for error handling.




--	    *************************************
--	    * Function 5AH - Create Unique File *
--	    *************************************


-- Generate unique filename and creates that file in the specified directory
-- The first parameter is the path to be used, ending with backslash. The
-- name of the file (which is always 8 characters, with no extension), is
-- stored in FILE_NAME on exit. The handle of the resulting file is stored
-- in HANDLE, and may be used in the usual way to read and write the file.

   subtype STRING8 is STRING (1..8);

   procedure CREATE_UNIQUE_FILE (
      PATH	: STRING;
      FILE_NAME : out STRING8;
      HANDLE	: out HANDLE_TYPE;
      ATTRIBUTE : ATTR_TYPE := ATTR_NORMAL);



--	    **********************************
--	    * Function 5BH - Create New File *
--	    **********************************

-- Creates a new file. An error is signalled if there is an exisiting file
-- of the same name (thus differentiating this call from CREATE_FILE). The
-- result returned is the file handle.

   function CREATE_NEW_FILE (
      FILE_NAME : STRING;
      ATTRIBUTE : ATTR_TYPE := ATTR_NORMAL)
      return HANDLE_TYPE;



--	    ******************************************
--	    * Function 5CH - Lock/Unlock File Access *
--	    ******************************************

-- Lock a range of bytes in an opened file. This function is not available
-- in versions of DOS before 3.10. The call is simply ignored, allowing a
-- program to use LOCK and still operate correctly in a non-shared environment
-- using an earlier versions of DOS.

   procedure LOCK (
      HANDLE : HANDLE_TYPE;
      OFFSET : INTEGER;
      LENGTH : INTEGER);


-- Unlock a range of bytes in an opened file. This function is not available
-- in versions of DOS before 3.10. The call is simply ignored, allowing a
-- program to use UNLOCK and still operate correctly in a non-shared
-- environment using an earlier versions of DOS.

   procedure UNLOCK (
      HANDLE : HANDLE_TYPE;
      OFFSET : INTEGER;
      LENGTH : INTEGER);



--	    *************************************
--	    * Function 5E00H - Get Machine Name *
--	    *************************************

-- No interface is provided to this function



--	    **************************************
--	    * Function 5E02H - Set Printer Setup *
--	    **************************************

-- No interface is provided to this function



--	    ***********************************************
--	    * Function 5F02H - Get Redirection List Entry *
--	    ***********************************************

-- No interface is provided to this function



--	    **************************************
--	    * Function 5F03H -	 Redirect Device *
--	    **************************************

-- No interface is provided to this function



--	    *****************************************************
--	    * Function 62H - Get Program Segment Prefix Address *
--	    *****************************************************

-- No interface is provided to this function



--	    *************************************
--	    * (10)  Environment Access Routines *
--	    *************************************

-- Procedure to return string from environment. The returned string
-- does not have a null terminator. If there is no matching string,
-- then a null string is returned. Note that the name argument may
-- be upper or lower case (i.e. case is ignored in the search).

   function GET_ENV_STRING (NAMESTR : STRING) return STRING;


-- Procedure to return the name of the current program. The returned
-- string is not null terminated, and contains the complete drive, path
-- and extension of the program file. Note however that in the released
-- version of DOS 3.10, when DEBUG loads a program it does not completely
-- duplicate this function, and sets a copy of the name as given, rather
-- than filling out the drive, path and extension. This function is not
-- available in versions of DOS before 3.00, and a null string will be
-- returned if GET_PROGRAM_NAME is used with earlier versions of DOS.

   function GET_PROGRAM_NAME return STRING;


-- Procedure to obtain the program parameters. The returned string does not
-- include a count byte or a terminating carriage return. Leading blanks
-- are removed.

   function GET_PARMS return STRING;



--	    ***************************************
--	    * (11)  Absolute Disk Access Routines *
--	    ***************************************

-- The absolute disk access routines are not supported in NT-DOS

-- These procedures allow access to disk drives at the sector level. Needless
-- to say they are extremely dangerous. Using WRITE_ABS it is quite easy to
-- totally destroy the data on a drive. The buffer may be given either as
-- a string, or as a system address. A returned result of zero means that
-- the operation was completed successfully. Note that the caller must always
-- check the status when using these calls, DOS_ERROR is never signalled.

-- The STATUS return is zero if there is no error. If there is an error,
-- then the non-zero error code is 256*E1+E2 where E1 is one of the codes
-- described in TR-6-15, and E2 is one of the codes described in TR-6-23.
-- See definitions in DOSE for further details of these error codes.


-- Read absolute sectors, buffer given as STRING

-- procedure READ_ABS (
--    DRIVE        : DRIVE_LETTER := '@';
--    NUM_SECTORS  : WORD_16;
--    START_SECTOR : INTEGER_32;
--    BUFFER	   : out STRING;
--    STATUS	   : out WORD_16);


-- Read absolute sectors, buffer given as SYSTEM.ADDRESS

-- procedure READ_ABS_ADR (
--    DRIVE        : DRIVE_LETTER := '@';
--    NUM_SECTORS  : WORD_16;
--    START_SECTOR : INTEGER_32;
--    BUFFER	   : SYSTEM.ADDRESS;
--    STATUS	   : out WORD_16);


-- Write absolute sectors, buffer given as STRING

-- procedure WRITE_ABS (
--    DRIVE        : DRIVE_LETTER := '@';
--    NUM_SECTORS  : WORD_16;
--    START_SECTOR : INTEGER_32;
--    BUFFER	   : STRING;
--    STATUS	   : out WORD_16);


-- Write absolute sectors, buffer given as SYSTEM.ADDRESS

-- procedure WRITE_ABS_ADR (
--    DRIVE        : DRIVE_LETTER := '@';
--    NUM_SECTORS  : WORD_16;
--    START_SECTOR : INTEGER_32;
--    BUFFER	   : SYSTEM.ADDRESS;
--    STATUS	   : out WORD_16);



--	    *********************************
--	    * (12)  Control/Break Intercept *
--	    *********************************

-- The control/break intercept routines are not supported in NT-DOS

-- Normally if DOS intercepts a Ctrl-Break sequence it terminates execution.
-- This interface function allows an Ada program to install an INT 23H handler
-- to allow control to be retained on a Ctrl-Break. If a Ctrl-Break occurs
-- after this handler has been installed, execution continues, and the program
-- may use the status calls to determine if a Ctrl-Break has occured. Finally
-- there is a routine to cancel the intercept.

-- Install Ctrl/Break handler

-- procedure CTRL_BREAK_INSTALL;


-- Test if Ctrl/Break occured since previous call. A result of TRUE is
-- returned if a Ctrl/Break has occured since the most recent call to
-- CTRL_BREAK_INSTALL or CTRL_BREAK_TEST.

-- function CTRL_BREAK_TEST return BOOLEAN;


-- Cancel Ctrl-Break handling. Following this call a Ctrl-Break will
-- terminate execution in the normal manner.

-- procedure CTRL_BREAK_CANCEL;



--	    *****************************
--	    * (13)  Conversion Routines *
--	    *****************************

-- These routines are provided to allow convenient handling of the packed
-- date and time values used in file directory entries, and returned directly
-- during a file search (FIND_FIRST and FIND_NEXT).

-- Convert date from packed form to unpacked form

   function UNPACK_DATE (DATEP : PACKED_DATE) return DATE_RECORD;


-- Convert date from unpacked form to packed form

   function PACK_DATE (DATEU : DATE_RECORD) return PACKED_DATE;


-- Convert time from packed form to unpacked form

   function UNPACK_TIME (TIMEP : PACKED_TIME) return TIME_RECORD;


-- Convert time from unpacked form to packed form

   function PACK_TIME (TIMEU : TIME_RECORD) return PACKED_TIME;


-- The following function takes a string as an input parameter and returns the
-- corresponding unconstrained string as result. As usual the input string may
-- be terminated with 00H, in which case the returned string includes only the
-- characters up to but not including the terminating 00H. One use of this
-- function is to convert the FILE_NAMX field of a SEARCH_RECORD following
-- a call to FIND_FIRST or FIND_NEXT.

   function GET_STRING (STR : STRING) return STRING;


end DOS;
-- VAX/DEC CMS REPLACEMENT HISTORY, Element DOS.ADS
-- *4     9-FEB-1995 15:31:02 JLO "Support long file name"
-- *3     8-SEP-1994 00:24:21 MORSE "MANY MORE FUNCTIONS IMPLEMENTED, FORMAT CLEANED UP"
-- *2    25-AUG-1994 21:08:45 MORSE "ADDED DELETE_FILE AND RENAME_FILE IN ORDER TO SUPPORT COUNT AND SPLIT"
-- *1    23-JUN-1994 21:10:08 MORSE "DOS PACKAGE FOR NT"
-- VAX/DEC CMS REPLACEMENT HISTORY, Element DOS.ADS
