-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ***************************************************************************
--
-- PACKAGE SPECIFICATION System.RTS.TGT.Exceptions.TYPES
-- ------------------------------
--
-- OUTLINE
-- -------
-- This module contains the type definitions for the exception manager.
-- It also contains the global variable which contains the root of the
-- tree of tables used by the exception manager.
--
-- NOTE
-- ----
-- This module contains target-dependent representation clauses.
-- ***************************************************************************

with System.RTS.TGT.Exceptions.TDEP;
with System.Rts.Types; use System.Rts.Types;

package System.RTS.TGT.Exceptions.TYPES is

   pragma SUPPRESS (ELABORATION_CHECK);

   --       *********************
   --       * TABLE OF CONTENTS *
   --       *********************

   --         1. General Types and Constants
   --         2. Ada Code Table
   --         3. Entry Table
   --         4. Procedure Record
   --         5. Scope Table
   --         6. Handler Table
   --         7. Dump Routines
   --         8. Revision History

   -- ------------------------------------------
   -- Package renamings to remove prefix:
   -- ------------------------------------------
   package EM_TDEP renames System.RTS.TGT.Exceptions.TDEP;
   package RT_TYPES renames System.Rts.Types;
   -- ------------------------------------------


   --       **********************************
   --       * 1. GENERAL TYPES AND CONSTANTS *
   --       **********************************

   NUMBER_OF_ELEMENT : constant := 65537;
   --%%%% NOTE : This is an artificial upper bound of any array declared in
   --            this package.
   --            This value is in fact only used when checks are on, and so
   --            only for debugging purposes.
   --            The value has been chosen to allow full indexing (on 32
   --            bits), and is not 2**31-1 which has the effect of crashing
   --            the compiler.

   -- Note: in the following comments, the word "byte" is used in describing
   -- addresses, since it leads to the most understandable commentary in the
   -- normal case of byte addressed machines. However, there is nothing in
   -- the coding of the target independent sections of the exception manager
   -- that depends on bytes being 8-bits, so on a machine with a different
   -- sized storage unit, the comments may be a little inappropriate, but
   -- the code will function correctly.



   --       *********************
   --       * 2. ADA CODE TABLE *
   --       *********************

   -- An Ada code table record is constructed for each contiguous block of Ada
   -- code in an application, including any shared modules that are referenced.
   -- The code table records are linked together in a linked list, with NULL
   -- marking the end of the list, and CODE_TABLE_HEAD points to the list head.

   type ENTRY_TABLE;
   type REF_ENTRY_TABLE is access all ENTRY_TABLE;

   type SH_RECORD;
   type REF_SH_RECORD is access all SH_RECORD;

   type SCOPE_TABLE;
   type REF_SCOPE_TABLE is access all SCOPE_TABLE;

   type HANDLER_TABLE;
   type REF_HANDLER_TABLE is access all HANDLER_TABLE;

   type ADA_CODE_TABLE_RECORD;
   type REF_ADA_CODE_TABLE is access all ADA_CODE_TABLE_RECORD;

   CODE_TABLE_HEAD : REF_ADA_CODE_TABLE;


   type ADA_CODE_TABLE_RECORD is record

      NEXT                     : REF_ADA_CODE_TABLE;
      -- Pointer to next entry in chain of linked records

      CODE_START               : CODE_ADDRESS;
      -- Absolute address of start of code (address of first byte)

      CODE_END                 : CODE_ADDRESS;
      -- Absolute address of byte just past last byte of code

      ENTRY_TABLE_PTR          : REF_ENTRY_TABLE;
      -- Pointer to entry table for this code segment

      ENTRY_TABLE_LAST         : ENTRY_TABLE_SUBSCRIPT;
      -- Subscript of last entry (sentinel entry) in entry table

      HISTORY_TABLE_PTR        : EM_TDEP.REF_HISTORY_TABLE;
      -- Pointer to history table (containing stack trace back information).
      -- Not all targets use this field. For targets which do not use this
      -- field, a dummy history module is provided, and this pointer is either
      -- omitted (it is at the end of the record), or set to null and ignored.

   end record;

   --       ******************
   --       * 3. ENTRY TABLE *
   --       ******************

   -- Note: the ENTRY TABLE will be renamed the PROCEDURE ADDRESS TABLE
   -- at some time in the future. This is a cosmetic change only!

   -- There is one entry point table for each contiguous code section (i.e.
   -- one entry point table for each entry in the Ada code table list). It
   -- consists of a sorted list of entry points of procedures in the section.
   -- There is a sentinel at the end which is the code offset of the code unit
   -- immediately following the end of the last procedure in the section.

   -- The bounds of this table are 0 .. ENTRY_TABLE_LAST. The declaration here
   -- uses static bounds to avoid the need to construct descriptors in the
   -- exception tables (in any case EM must be compiled with checks off!)

   type ENTRY_TABLE_RECORD is record

      ENTRY_POINT : CODE_ADDRESS;
      -- Entry point of the current procedure.

      SH_RECORD : REF_SH_RECORD;
      -- Reference to the record defining the procedure information, scope table,
      -- and handler table for the current procedure.

      PROCEDURE_INFORMATION : EM_TDEP.PROCEDURE_INFORMATION_RECORD;
      -- This is the information describing the current procedure (so that
      -- the exception manager can reconstruct the calling context if the
      -- exception is propagated. The format of this record is target
      -- dependent, and the type is defined in EM_TDEP.

   end record;

   type ENTRY_TABLE is
      array (ENTRY_TABLE_SUBSCRIPT range 0..NUMBER_OF_ELEMENT) of ENTRY_TABLE_RECORD;


   --       **********************
   --       * 4. SH RECORD      *
   --       **********************

   -- The complete SH record is not directly representable in Ada.  Instead,
   -- subprograms are provided to access the components of the record.
   --
   -- The SH record is arranged in memory as follows:
   --   Number of entries in scope table (32 bits)
   --   Scope table
   --   Handler table

   type SH_RECORD is record
      SCOPE_TABLE_LENGTH : SCOPE_TABLE_SUBSCRIPT;
   end record;


   function GET_SCOPE_TABLE_LENGTH (SH_RECORD : in REF_SH_RECORD) 
            return SCOPE_TABLE_SUBSCRIPT;
   --------------------------------------------------------------------------------
   -- Return the length of the scope table for this SH record.
   --------------------------------------------------------------------------------

   function GET_SCOPE_TABLE (SH_RECORD : in REF_SH_RECORD) return REF_SCOPE_TABLE;
   --------------------------------------------------------------------------------
   -- Return a pointer to the scope table for this SH record.
   --------------------------------------------------------------------------------

   function GET_HANDLER_TABLE (SH_RECORD : in REF_SH_RECORD) return REF_HANDLER_TABLE;
   --------------------------------------------------------------------------------
   -- Return a pointer to the scope table for this SH record.
   --------------------------------------------------------------------------------



   --       ******************
   --       * 5. SCOPE TABLE *
   --       ******************

   -- There is one scope table for each contiguous code section (i.e.
   -- one scope table for each entry in the Ada code table list). This
   -- table consists of a series of sections, one for each procedure
   -- which has at least one exception scope. The procedure table entry
   -- contains the starting subscript for the scope table entries which
   -- correspond to that procedure.

   type SCOPE_TABLE_RECORD is record
 
      STARTING_PROC_OFFSET : PROC_OFFSET;
      -- Offset from start of procedure of first byte of code in the
      -- scope to which this entry applies.

      ENDING_PROC_OFFSET   : PROC_OFFSET;
      -- Offset from start of procedure of last byte of code in the scope
      -- to which this entry applies. Note that this is NOT the offset of
      -- the byte just past the last instruction in the scope, it is one
      -- less than this value.

      HANDLER_TABLE_ENTRY : HANDLER_TABLE_SUBSCRIPT;
      -- Subscript of first entry in the handler table which applies
      -- to this scope.

   end record;

   -- The bounds of this table are 0 .. SCOPE_TABLE_LENGTH-1. The declaration here
   -- uses static bounds to avoid the need to construct descriptors in the
   -- exception tables (in any case EM must be compiled with checks off!)

   type SCOPE_TABLE is array (SCOPE_TABLE_SUBSCRIPT
      range 0 .. NUMBER_OF_ELEMENT) of SCOPE_TABLE_RECORD;


   --       ********************
   --       * 6. HANDLER TABLE *
   --       ********************

   -- There is one handler table for each contiguous code section (i.e.
   -- one handler table for each entry in the Ada code table list). This
   -- table consists of a series of sections, one for each exception scope.
   -- The scope table entry contains the starting subscript for the scope
   -- table entries which correspond to that scope.

   HTE_OTHERS                  : constant := 0;
   HTE_END_OF_SEARCH           : constant := 1;
   HTE_GOTO_ENCLOSING_SCOPE    : constant := 2;
   HTE_SET_MARKER              : constant := 3;

   type HANDLER_TABLE_RECORD (ECODE : EXCEPTION_CODE := 0) is record

      case ECODE is

         -- The HTE_OTHERS entry is used to represent an others handler

         when HTE_OTHERS =>

            OTHERS_HANDLER     : PROC_OFFSET;
            -- Offset in procedure to first byte of handler code


         -- The HTE_END_OF_SEARCH entry is used to terminate the search
         -- for the current procedure, indicating that the exception is
         -- not handled in the current procedure, and that therefore the
         -- exception must be propagated to the calling procedure.

         when HTE_END_OF_SEARCH =>
            null;


         -- The HTE_GOTO_ENCLOSING_SCOPE entry is used to terminate the
         -- search for the current scope, and indicate a location in the
         -- handler table where the search for an applicable scope is to
         -- be continued (this will correspond to a scope that encloses
         -- the current scope, but is in the same procedure).

         when HTE_GOTO_ENCLOSING_SCOPE =>

            NEXT_ENTRY         : HANDLER_TABLE_SUBSCRIPT;
            -- Relative subscript of handler entry (in this handler table)
            -- where the search is to be resumed. The entry is determined
            -- by adding the NEXT_ENTRY value to the current subscript.


         -- The HTE_SET_MARKER entry is used to set the current marker value
         -- to an indicated location in the current stack frame. This is used
         -- to deal with resetting the stack pointer when an exception is
         -- propagated outside a block.

         when HTE_SET_MARKER =>

            MARKER_OFFSET      : FRAME_OFFSET;
            -- Offset in frame units from the frame pointer. The marker value
            -- is set to the current frame pointer value plus this value.


         -- Finally, all other cases correspond to normal exception handlers,
         -- where the ECODE value is the exception code for the handler.

         when others =>

            HANDLER            : PROC_OFFSET;
            -- Offset in procedure to first code unit of handler code

         end case;

   end record;

   -- The bounds of this table are 0 to the entry which has an ECODE of 
   -- HTE_END_OF_SEARCH. The declaration here uses static bounds to avoid the 
   -- need to construct descriptors in the exception tables (in any case EM must 
   -- be compiled with checks off!)

   type HANDLER_TABLE is array (HANDLER_TABLE_SUBSCRIPT
      range 0 .. NUMBER_OF_ELEMENT) of HANDLER_TABLE_RECORD;


   --       ********************
   --       * 7. DUMP ROUTINES *
   --       ********************

   procedure DUMP_TABLES;
   -- Routine to dump exception tables using ERROR_IO

   procedure INIT_EM_TYPES;

private

   pragma import (C, CODE_TABLE_HEAD, "ada_code_table_head");

   -- Representation specifications to ensure compatibility with compiler
   -- and binder-generated tables.

   for ADA_CODE_TABLE_RECORD use
      record
         NEXT                at  0 range 0 .. 31;
         CODE_START          at  4 range 0 .. 31;
         CODE_END            at  8 range 0 .. 31;
         ENTRY_TABLE_PTR     at 12 range 0 .. 31;
         ENTRY_TABLE_LAST    at 16 range 0 .. 31;
         HISTORY_TABLE_PTR   at 20 range 0 .. 31;
      end record;

   for ADA_CODE_TABLE_RECORD'SIZE use 24*8;


   for ENTRY_TABLE_RECORD use
      record
         ENTRY_POINT           at 0 range 0 .. 31;
         SH_RECORD             at 4 range 0 .. 31;
         PROCEDURE_INFORMATION at 8 range 0 .. EM_TDEP.PROCEDURE_INFO_SIZE-1;
   end record;
   
   for ENTRY_TABLE_RECORD'SIZE use 8*8 + EM_TDEP.PROCEDURE_INFO_SIZE;
   

   for SH_RECORD use
      record 
         SCOPE_TABLE_LENGTH at 0 range 0 .. 31;
      end record;

   for SH_RECORD'SIZE use 32;


   for SCOPE_TABLE_RECORD use
      record
         STARTING_PROC_OFFSET at 0 range 0 .. 31;
         ENDING_PROC_OFFSET   at 4 range 0 .. 31;
         HANDLER_TABLE_ENTRY  at 8 range 0 .. 31;
      end record;

   for SCOPE_TABLE_RECORD'SIZE use 12*8;


   for HANDLER_TABLE_RECORD use
      record
         ECODE          at 0 range 0 .. 31;
         OTHERS_HANDLER at 4 range 0 .. 31;
         NEXT_ENTRY     at 4 range 0 .. 31;
         MARKER_OFFSET  at 4 range 0 .. 31;
         HANDLER        at 4 range 0 .. 31;
      end record;

   for HANDLER_TABLE_RECORD'SIZE use 8*8;


   pragma Inline (GET_SCOPE_TABLE_LENGTH);
   pragma Inline (GET_SCOPE_TABLE);
   pragma Inline (GET_HANDLER_TABLE);

   
end System.RTS.TGT.Exceptions.TYPES;

--          ***********************
--          * 8. REVISION HISTORY *
--          ***********************

--  0.01  15 May 1990, 21:35 (RBKD)
--        Initial version

--  0.02  28 Sep 1990, 13:41 (PC+RBKD)
--        Change to avoid use of unconstrained records
--        Note that the exception manager must be compiled with checks off
--        Add DUMP_TABLES routine
--        Create separate body

--  0.03  28 Sep 1990, 16:18 (PC)
--        HANDLER_TABLE_LAST is of type HANDLER_TABLE_SUBSCRIPT

--  0.04  03 Oct 1990, 16:00 (PC)
--        Add representation clauses on records generated by the
--         binder or seen by C or ASM

--  0.05  04 Oct 1990, 13:51 (PC)
--        Add pragma PACK on arrays and size clauses on collections

--  0.06  14 Oct 1990, 16:50 (RBKD)
--        Add declaration of HISTORY_OPTION
--        Adjust documentation for presence of sentinel in procedure table
--        Remove DUMP_TABLE_SWITCH (now moved to EM_DEBUG)
--        Reduce dependence on use clauses

--  0.07  20 Dec 1990, 01:01 (RBKD)
--        Initial version for common runtime source checkin
--        Uses new version of RT_TYPES
--        Expand handler and scope tables to 32 bits
--        Procedure offsets are now 32 bits, and no longer in code units
--        The procedure table does not have a sentinel entry
--        GOTO entry is now a relative instead of absolute subscript

--  6.01  20 Dec 1990 (BJD)
--        Initial introduction to CMS v6.
--        Comment out zero-length collection sizes (awaiting Root support).
--        Comment out highly target-dependent alignment constraint for
--        HANDLER_TABLE_RECORD, and record representation and size clauses
--        for PROCEDURE_TABLE_RECORD (contains a field from EM_TDEP).

--  6.02  23rd January 1991 (BJD)
--        Move rep specs to private part.
--        Remove type decls using INTEGER_32 (moved into RT_TYPES).
--        Use targat-dep size for PROCEDURE_INFORMATION_RECORD instead of 32.

--  6.03  21st May 1991 (BJD)
--        Remove HISTORY_OPTION (now in CONFIGURATION).
--        Introduce NUMBER_OF_ELEMENT as artificial upper bound to allow
--        debugging with checks on.
--        Get REF_HISTORY_TABLE from EM_TDEP now, since HISTORY_TABLE type is
--        target-dependent.

--  6.04  25th October 1991 (CEB)
--        Enabled STORAGE_SIZE = 0 option.

-- 6.??   3rd November 1991 (BJD)
--        Prefix all CRTS library unit names with ALSYS_
