-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- This unpublished work is protected both as a proprietary work and under
-- the Universal Copyright Convention and the US Copyright Act of 1976.  Its
-- distribution and access are limited only to authorized persons.  Copyright
-- (C) Alsys.  Created 1990, initially licenced 1990.  All rights reserved.
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction or distribution may violate national criminal law.
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-- ************************************************************************
-- PACKAGE SPECIFICATION SYSTEM.RTS.ERROR_IO
-- ------------------------------
--
-- OUTLINE :
-- -------
--    This is a minimal package to perform output on STANDARD_ERROR.
-- This output consists of error and informational messages emitted
-- by the runtime system for the user. It is also used to output
-- debug trace text for internal use.
--
--    It should be one of the first runtime packages to be elaborated so
-- that it can be used to output any error messages resulting from
-- the elaboration of the remainder of the runtime.
--
--    In order to support foreign languages in user error messages, the
-- text of such messages is hidden behind an interface. This results
-- in callers generating an error code which is translated into text
-- in the body.
--
-- ************************************************************************

--IIwith ALSYS_TCB_PACKAGE;        -- For TASK_ID
with System.Rts.Types;           -- For LONGEST_INTEGER
with SYSTEM;                   -- For ADDRESS
--II with ALSYS_RTS_EXTENDED_ASCII; -- For EXTENDED_STRING

package SYSTEM.RTS.ERROR_IO is

   pragma SUPPRESS (ELABORATION_CHECK);

   -- ============================================
   -- Table of Contents
   -- -----------------
   -- 1. Support for Language-Independent Messages
   -- 2. Routines for Formatted Output
   -- 3. Routines for User Error Message Output
   -- 4. Routines to prevent Interleaving Problems
   -- ============================================

   -- ------------------------------------------
   -- Package renamings to remove ALSYS_ prefix:
   -- ------------------------------------------
--II   package TCB_PACKAGE renames ALSYS_TCB_PACKAGE;
   package RT_TYPES renames System.Rts.Types;
--II   package RTS_EXTENDED_ASCII renames ALSYS_RTS_EXTENDED_ASCII;
   -- ------------------------------------------

   -- ********************************************
   -- 1. SUPPORT FOR LANGUAGE-INDEPENDENT MESSAGES
   -- ********************************************

   type MESSAGE is new NATURAL;

   -- ---------------------------
   -- Target-independent messages
   -- ---------------------------
   GLOBAL_BLOCKING : constant MESSAGE := 0;
   TASKS_SUSPENDED : constant MESSAGE := 1;
   PROGRAM_ABORTED : constant MESSAGE := 2;
--II   SUBSTATUS_MESSAGE : constant array (TCB_PACKAGE.TASK_SUBSTATUS) of
--II      MESSAGE := (3, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12);
   -- Traceback output messages:
   -- Three possible headers and footers: dead program, dead task, handled
   TRACE_TITLE     : constant array (0..2) of MESSAGE := (13, 14, 15);
   -- Note: we use the same message for ending a dead program's propagation
   -- and a dead task's propagation, so they get the same message number.
   TRACE_FOOTER    : constant array (0..2) of MESSAGE := (16, 16, 17);
   -- Message specifying name of unhandled exception
   EXCEPTION_IS    : constant MESSAGE := 18;
   NO_EXCEPTION_ID : constant MESSAGE := 19;
   -- We allow two lines for traceback column headings
   TRACE_HEADER_1  : constant MESSAGE := 20;
   TRACE_HEADER_2  : constant MESSAGE := 21;
   -- Text in traceback indicating address of handler
   OTHERS_HANDLED  : constant MESSAGE := 22;
   HANDLED         : constant MESSAGE := 23;
   IMPROPER_CONTEXT: constant MESSAGE := 24;

   -- --------------------------------------------------------------------
   -- Target-dependent messages start at 100, see package TARGET_MESSAGES.
   -- --------------------------------------------------------------------
   FIRST_TARGET_MESSAGE : constant := 100;

   -- The length of the longest message string:
   MAX_MESSAGE_LENGTH : constant := 255;

   -- We need to declare both constrained and unconstrained types to use for
   -- buffers to avoid heap calls. We assume that MAX_MESSAGE_LENGTH arrays
   -- of CHARACTERS and EXTENDED_CHARACTERS are allocated on the stack.
   subtype MESSAGE_LENGTH is POSITIVE range 1 .. MAX_MESSAGE_LENGTH;
   subtype SHORT_EXTENDED is STRING;
   
   --   array(MESSAGE_LENGTH range <>) of CHARACTER;
--II      array(MESSAGE_LENGTH range <>) of RTS_EXTENDED_ASCII.EXTENDED_CHARACTER;
--II   pragma PACK (SHORT_EXTENDED); -- Ensure same rep as EXTENDED_STRINGs.
--II   subtype MESSAGE_STRING is SHORT_EXTENDED(MESSAGE_LENGTH);
   subtype MESSAGE_STRING is STRING(MESSAGE_LENGTH);
   
   -- ********************************
   -- 2. ROUTINES FOR FORMATTED OUTPUT
   -- ********************************

   procedure PUT (INT   : in RT_TYPES.LONGEST_INTEGER;
                  WIDTH : in NATURAL := 0);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Used to output any integer value in formatted output. The value is
   --    output as a signed decimal number right-justified in a field width
   --    of WIDTH with leading blanks. The sign is generated only for
   --    negative numbers. If WIDTH is insufficient to contain the signed
   --    number, it is output in its entirety with no leading blanks.
   -- ---------------------------------------------------------------------


   procedure PUT (ADD   : in SYSTEM.ADDRESS;
                  WIDTH : in NATURAL := 0);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Used to output any address in formatted output. The address is
   --    output in the same format as System.Image (System.Address) right-
   --    justified in a field width of WIDTH with leading blanks. If WIDTH
   --    is insufficient to contain the address, it is output in its
   --    entirety with no leading blanks.
   -- ---------------------------------------------------------------------


   procedure PUT_HEX (INT    : in RT_TYPES.LONGEST_INTEGER;
                      DIGS   : in NATURAL := RT_TYPES.LONGEST_INTEGER'SIZE/4;
                      WIDTH  : in NATURAL := 0);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Used to output a value in hex in formatted output. The value is
   --    output as a full unsigned hexadecimal number right-justified in a
   --    field width of WIDTH with leading blanks. The format of the number
   --    itself is as an unadorned string of hexadecimal digits. The number
   --    of digits is specified by the DIGS parameter, with sign-extension
   --    if necessary. If DIGS is insufficient to hold the magnitude of 
   --    INT then it is increased to accommodate all significant digits.
   --    If DIGS is larger than the number of hex digits in 
   --    LONGEST_INTEGER then it is decreased to that number of hex digits.
   --    If WIDTH is insufficient to contain these digits, the number is
   --    output in its entirety with no leading blanks.
   -- ---------------------------------------------------------------------

   procedure PUT (S : in STRING);
   procedure PUT_LINE (S : in STRING);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Used to output an ASCII string in formatted output. These routines
   --    should NOT be used for outputting user messages; the PUT_MESSAGE
   --    form should be used for this purpose. These routines are intended
   --    for outputting Ada identifiers and for internal debug trace output
   --    and for any other output which is guaranteed to be of type STRING.
   --    The effect of PUT_LINE(S) is equivalent to PUT(S); NEW_LINE;
   -- ---------------------------------------------------------------------

   procedure NEW_LINE;
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Used to force a line-break in formatted output.
   -- ---------------------------------------------------------------------

--II   procedure PUT_TASK_NAME ( T : in TCB_PACKAGE.TASK_ID; S : in STRING );
   -- -------------------------------------------------------------------
   -- OUTLINE:
   --    Outputs the name of the current task followed by a string. This
   --    routine is intended for internal error messages only.
   -- -------------------------------------------------------------------

   -- *****************************************
   -- 3. ROUTINES FOR USER ERROR MESSAGE OUTPUT
   -- *****************************************


   procedure PUT_MESSAGE (M : in MESSAGE);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Converts error code M into the corresponding user error message
   --    and outputs the text followed by a new line character.
   -- ---------------------------------------------------------------------

   procedure PUT_MESSAGE (M   : in MESSAGE;
                          INT : in RT_TYPES.LONGEST_INTEGER);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Converts error code M into the corresponding user error message
   --    which should be one which expects a single parameter. If so,
   --    converts INT into its Image in signed decimal form and inserts
   --    the resulting text into the correct place in the error message
   --    text. If no parameter is expected in the message, INT is ignored.
   --    The final text is output followed by a new line character.
   -- ---------------------------------------------------------------------


   procedure PUT_MESSAGE (M   : in MESSAGE;
                          ADD : in SYSTEM.ADDRESS);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Converts error code M into the corresponding user error message
   --    which should be one which expects a single parameter. If so,
   --    converts ADD to System.Image (System.Address) form and inserts
   --    the resulting text into the correct place in the error message
   --    text. If no parameter is expected in the message, ADD is ignored.
   --    The final text is output followed by a new line character.
   -- ---------------------------------------------------------------------


--II   procedure PUT_MESSAGE (M : in MESSAGE;
--II                          T : in TCB_PACKAGE.TASK_ID);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Converts error code M into the corresponding user error message
   --    which should be one which expects a single parameter. If so,
   --    converts the task identifier T into the task name and inserts
   --    the resulting text into the correct place in the error message
   --    text. If no parameter is expected in the message, T is ignored.
   --    The final text is output followed by a new line character.
   -- ---------------------------------------------------------------------


   procedure PUT_MESSAGE (M : in MESSAGE;
                          S : in STRING);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Converts error code M into the corresponding user error message
   --    which should be one which expects a single parameter. If so,
   --    converts S if necessary to extended character set and inserts
   --    the resulting text into the correct place in the error message
   --    text. If no parameter is expected in the message, S is ignored.
   --    The final text is output followed by a new line character.
   -- ---------------------------------------------------------------------


   procedure PUT_MESSAGE_WITH_HEX (M      : in MESSAGE;
                                   INT    : in RT_TYPES.LONGEST_INTEGER;
                                   DIGS   : in NATURAL := 
                                              RT_TYPES.LONGEST_INTEGER'SIZE/4);
   -- ---------------------------------------------------------------------
   -- OUTLINE:
   --    Converts error code M into the corresponding user error message
   --    which should be one which expects a single parameter. If so,
   --    converts INT into full hexadecimal form as for PUT_HEX and inserts
   --    the resulting text into the correct place in the error message
   --    text. If no parameter is expected in the message, INT is ignored.
   --    The final text is output followed by a new line character.
   -- ---------------------------------------------------------------------


   -- *********************************************
   -- 4. ROUTINES TO PREVENT INTERLEAVING PROBLEMS
   -- *********************************************

   procedure BEGIN_OUTPUT;
   -- -------------------------------------------------------------------
   -- OUTLINE:
   --   A semaphore is used to prevent interleaving. When the user wishes
   -- to perform a group of PUT statements which should not appear
   -- interleaved then the group begins BEGIN_OUTPUT and ends END_OUTPUT.
   -- --------------------------------------------------------------------


   procedure END_OUTPUT;
   -- -------------------------------------------------------------------
   -- OUTLINE:
   --   A semaphore is used to prevent interleaving. When the user wishes
   -- to perform a group of PUT statements which should not appear
   -- interleaved then the group begins BEGIN_OUTPUT and ends END_OUTPUT.
   -- --------------------------------------------------------------------

end SYSTEM.RTS.ERROR_IO;

-- Modification History:
--
-- 0.00 3rd September 1990 (BJD)
--      Initial version

-- 6.01 12th October 1990 (BJD)
--      Initial introduction to CMS

-- 6.02 29th November 1990 (APC)
--      Addition of BEGIN_OUTPUT, END_OUTPUT, and the version of PUT_MESSAGE
--      which takes a task as a parameter. Some initial messages have been
--      given.

-- 6.03 17th December 1990 (BJD)
--      Update for new name of EXTENDED_ASCII package (RTS_EXTENDED_ASCII)

-- 6.04 21st December 1990 (BJD)
--      Add DIGS parameter to HEX routines and change parameter type to
--      LONGEST_INTEGER to allow more general output of hex values.

-- 6.05 20th Feb 1991 (BJD)
--      Add SHORT_EXTENDED to avoid heap calls resulting from use of
--      unconstrained EXTENDED_STRINGs

-- 6.06 21st May 1991 (BJD)
--      Add MESSAGES for traceback generation.
--      Amend PUT_MESSAGE param from EXTENDED_STRING to STRING, since the
--      parameter is an identifier (and so cannot be extended characters).

-- 6.07 6th August 1991 (BJD)
--      Add pragma PACK for SHORT_EXTENDED to ensure same representation as
--      RTS_EXTENDED_ASCII.EXTENDED_STRINGs, which are packed.

-- 6.08 3rd November 1991 (BJD)
--      Prefix all CRTS library unit names with ALSYS_

-- 6.09 19th November 1991 (BJD)
--      Add message entry for new substatus WAITING_FOR_RESOURCE.
