-- $Source: /home/dsd/d/aa/monoBANK/AA/TSP_system.rts.exception_manager.spc,v $ 
-- $Revision: 1.7 $ $Date: 96/12/31 14:40:07 $ $Author: vdelvecc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Exception_Types;
with System.RTS.TGT.Frames;
with Ada.Exceptions;

package System.RTS.Exception_Manager is

    pragma Suppress( Elaboration_Check );

    package ET renames System.RTS.Exception_Types;
    package AE renames Ada.Exceptions;

--* OVERVIEW
    
-- The exception manager requires the following parameters: 
--
--   - exception_val, which is the value of the exception
--     (as defined in the compiler front end, a pointer to a string)
-- 
--  - exception_loc, which is the code location where the 
--    exception was raised,
-- 
-- 
-- Given exception_loc, RTS_Raise consults the exception table
-- for the subprogram in which the exception was raised.
-- If a corresponding handler is found, then the control is given
-- back to the handler.  If no handler is found in the current subprogram,
-- the stack frame is walked back by executing the epilogue of the
-- current subprogram and the exception is propagated to the caller.  
-- This is continued until a handler is found or until the top
-- level is reached.  If no handler is found during this walk
-- back, a return from the top level subprogram is executed.
-- This gives control to a Run Time Library routine which 
-- terminates the current task or program.
-- 
-- Note that the epilogue executed corresponds to the Back End-
-- generated epilogue and can only consist of register and
-- display restores (memory and task cleanups are done in 
-- compiler-generated handlers). The epilogue is only allowed 
-- to modify a limited number of registers (as defined in the 
-- Run Time Model) so that the state of the exception manager 
-- is not modified during the unwinding of the frame stack.
-- 	
-- The exception manager finds the addresses needed by 
-- consulting the subprogram table which is built by the
-- linker which gives the address of the entry point.
-- From this, the other data structures can be located.
-- 				Handler_Table
-- 				...
-- 				PC_Table
-- 				....
--				epilogue_address
--				PC_Table_Size

    -- subtype Exception_Ptr is Integer;

    ---------------------------------------------------------------------
    -- Runtime names ...
    ---------------------------------------------------------------------

    procedure RTS_Raise_Program_Error;
    pragma Import(Intrinsic,
                  RTS_Raise_Program_Error,
                  External_Name => "rts_raise_program_error");
    --* OVERVIEW
    -- Calls to this subprogram are inserted by the compiler
    -- when PROGRAM_ERROR is detected.
    -- This subprogram must be written using machine code inserts
    -- because it must have access to specific registers.
    --* EFFECTS
    -- RTS_Raise_Program_Error calls RTS_Raise passing the exception
    -- name for PROGRAM_ERROR.

    procedure RTS_Raise_Constraint_Error;
    pragma Import(Intrinsic,
                  RTS_Raise_Constraint_Error,
                  External_Name => "rts_raise_constraint_error");
    --* OVERVIEW
    -- Calls to this subprogram are inserted by the compiler
    -- when CONSTRAINT_ERROR is detected.
    -- This subprogram must be written using machine code inserts
    -- because it must have access to specific registers.
    --* EFFECTS
    -- RTS_Raise_Constraint_Error calls RTS_Raise passing the exception
    -- name for CONSTRAINT_ERROR.

    procedure RTS_Raise_Storage_Error;
    pragma Import(Intrinsic,
                  RTS_Raise_Storage_Error,
                  External_Name => "rts_raise_storage_error");

    --* OVERVIEW
    -- Calls to this subprogram are inserted by the compiler
    -- when STORAGE_ERROR is detected.
    -- This subprogram must be written using machine code inserts
    -- because it must have access to specific registers.
    --* EFFECTS
    -- RTS_Raise_Storage_Error calls RTS_Raise passing the exception
    -- name for STORAGE_ERROR.

    procedure RTS_Raise_Tasking_Error;
    pragma Import(Intrinsic,
                  RTS_Raise_Tasking_Error,
                  External_Name => "rts_raise_tasking_error");

    --* OVERVIEW
    -- Calls to this subprogram are inserted by the compiler
    -- when TASKING_ERROR is detected.
    -- This subprogram must be written using machine code inserts
    -- because it must have access to specific registers.
    --* EFFECTS
    -- RTS_Raise_Tasking_Error calls RTS_Raise passing the exception
    -- name for TASKING_ERROR.

    -- OBSOLETE procedure Overflow;
    -- OBSOLETE pragma Export (ada, Overflow, External_Name => "overflow" );
    --* OVERVIEW
    -- Calls to this subprogram are inserted by the compiler
    -- when overflow is detected.
    -- This subprogram must be written using machine code inserts
    -- because it must have access to specific registers.
    --* EFFECTS
    -- Overflow calls RTS_Raise passing the exception name for
    -- CONSTRAINT_ERROR.

    procedure RTS_Raise( Name : in AE.Exception_ID );
    pragma Import(Intrinsic, RTS_Raise, External_Name => "rts_raise");

    --* OVERVIEW
    -- Calls to this subprogram are inserted by the compiler and
    -- and correspond to an Ada "raise" statement.  This subprogram
    -- is also called from RTS_Raise_<predefined-exception>.
    -- It must be written using machine code inserts because
    -- it needs to access specific registers.  Also, it must be able
    -- to function even when there is no stack space available.
    --* EFFECTS
    -- Fills in the Exception_Occurrence in the TCB, and then calls
    -- RTS_Raise_Exception_Occurrence.

    procedure RTS_Raise_Occurrence( Exc : ET.Exception_Occurrence );
    pragma Import(Intrinsic, RTS_Raise_Occurrence,
		  External_Name => "rts_raise_occurrence");

    --* OVERVIEW
    -- This subprogram is called from Ada.Exceptions.Raise_Exception
    -- and from RTS_Raise; it is also called from Ada.Exceptions.
    -- Reraise_Occurrence and calls are generated by the compiler
    -- when a "raise;" statement is seen.  This routine does the
    -- bulk of the work of raising an exception.
    --
    -- RTS_Raise_Occurrence must be written using machine code
    -- inserts because:
    -- (i) it cannot have a frame (it destroys the stack), and
    -- (ii) it needs to access specific registers.
    -- Note that some subtasks (like Locate_Handlers) are  
    -- represented as subprogram calls to make the 
    -- description of the algorithm easier, but will 
    -- probably be inlined in the code.
    --
    --* EFFECTS
    -- Calls Locate_Handler to get the handler address if it
    -- exists.  If there is a handler then branch there and
    -- execute it.  If no handler is found in this frame then 
    -- propagate the exception by executing the subprogram's
    -- epilogue thus unwinding the stack.  The return address in
    -- the stack is altered before executing the epilogue to return
    -- to the proper location within the exception manager to repeat
    -- the process until either a handler is found or the top level
    -- is reached.
    -- This subprogram needs to keep all of its locals in registers.
    -- Scratch registers are available for that purpose.

    function RTS_SS_Copy_Exception_And_Rlse(
		 Name : in ET.Exception_Occurrence_Access;
		 Mark : in Address;
	         Master : in Address )
			return ET.Exception_Occurrence_Access;
    pragma Export(Ada,
		  RTS_SS_Copy_Exception_And_Rlse,
		  "rts_ss_copy_exception_and_rlse");

    function RTS_New_Occurrence( Name: AE.Exception_Id )
		return ET.Exception_Occurrence_Access;
    pragma Export(Ada,
		  RTS_New_Occurrence,
		  "rts_new_occurrence");
    --* OVERVIEW
    -- Returns a pointer to the exception occurrence which is in the
    -- current TCB, initialized to represent a new occurrence of the
    -- given Exception_Id.  The value returned is suitable for passing
    -- to RTS_Raise_Occurrence.

    type Finalize_Before_Handler_Type is access procedure(
	Master   : Address;
	Frame_ID : TGT.Frames.Frame_ID;
	Exception_Address : out Address);

    Finalize_Before_Handler : Finalize_Before_Handler_Type := null;
    -- This procedure pointer is initialized as a side-effect of
    -- elaborating System.RTS.Master_Pkg, if that is included in the
    -- program by the builder.

end System.RTS.Exception_Manager;
