-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.finalization_pkg.spc,v $ 
-- $Revision: 1.9 $ $Date: 96/05/03 13:09:59 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Exception_Types;
with System.RTS.Tasking_Types_Pkg;
with System.RTS.TGT.Sec_Stack_Pkg;	-- For Mark_Ptr
with System.Storage_Elements;

package System.RTS.Finalization_Pkg is

    pragma Suppress( Elaboration_Check );

    package ET  renames System.RTS.Exception_Types;
    package RTT renames System.RTS.Tasking_Types_Pkg;

    subtype Secondary_Mark_Ptr is System.RTS.TGT.Sec_Stack_Pkg.Mark_Ptr;

--* OVERVIEW
-- This package implements the abstract data type of finalization
-- list.  This package provides external interfaces for building a
-- finalization list and walking finalization list upon leaving a
-- master.  The portion of the finalization list walked depends on
-- how the currently executing master is left.
-- 
-- The overall set of finalization actions is organized into
-- a series of finalization lists, one per master record.

-- Some masters (block statements, subprograms and task bodies) 
-- may have dependent tasks.  In this case, finalization 
-- is performed after the dependent tasks have terminated.  
-- The master record includes a suspension object, a list
-- of dependent tasks, and a list of cleanup actions.
-- The list of dependent tasks must be protected
-- because multiple tasks may attempt to add dependents simultaneously.
--
-- Operations for managing task dependencies are described in the
-- package System.RTS.Task_Dependency.
--
-- The finalization list consists of cleanup actions.  The compiler
-- creates cleanup actions and calls various routines to add
-- them to the decls master or to the temps master.
-- At any given time, the temps master is the innermost master,
-- with the decls master immediately above it.
-- NOTE: When we add a new temps master, we need to check
--   whether it is already there (this can happen if the
--   temps master is first added inside an "if" statement; after
--   the "if" statement we might add it again since we aren't
--   sure whether it has been added).

-- There are various kinds of cleanup actions:
-- (1) a pointer to an object and a cleanup procedure to be applied
--     to it
-- (2) a pointer to an array, a number of elements, SUs per element,
--     an offset into each element, and a cleanup procedure
-- (3) a head of a doubly-linked list of allocated objects, 
--     plus a cleanup procedure and an offset within each object 
--     for the "interesting" component; such a list is associated 
--     with the "ultimate ancestor" access type used in the allocation.  
--     Its position in overall list of cleanup actions corresponds to 
--     its freezing point.
-- (4) a head of a doubly-linked list of allocated array objects, 
--     each of which has its own count.  The head also includes
--     the offset to the first "interesting" (sub)component
--     (this skips over the dope as well if unconstrained),
--     the number of SUs per component, and a cleanup procedure
--
-- Protected objects are handled as simply objects with a special
-- cleanup action, namely "finalize_protected_object."
-- If an "interesting" object contains interesting subcomponents,
-- then the compiler will generate a procedure to handle that.
--
-- There is a parallel set of kinds of cleanup actions (perhaps
-- we will merge these) for cases when the cleanup routine needs
-- a static link.

-- All cleanup actions are extensions of a root abstract type
-- Cleanup_Record defined in RTT, with one operation Cleanup
-- which "does the right thing" for the given cleanup action.

-- Normally a cleanup action is registered as the last step
-- of initializing or adjusting an "interesting" object.
-- Abort is normally deferred during the initialization/adjustment
-- to ensure that the cleanup action gets properly registered.

-- Extension aggregates pose a unique problem for out-of-line
-- initialization since they may require a miscellaneous
-- set of things to be done.  We may have to break down and
-- call defer-abort from the generated code, with an exception
-- handler and the whole nine yards in the generated code.
-- Alternatively, we could generate a local subprogram which
-- did the write thing and then pass that to initialize_and_register.


    -------------------------------------------------------
    --     S I M P L E     C L E A N U P                 --
    -------------------------------------------------------
    type Operation_Ptr is access procedure(Obj : in System.Address);

    type Simple_Cleanup is new RTT.Cleanup_Record with
      record
	Obj : System.Address;  -- Object to be cleaned up
	Cleanup_Op : Operation_Ptr;  -- Operation to do it
      end record;
    procedure Cleanup(Rec : Simple_Cleanup);
    type Simple_Cleanup_Ptr is access all Simple_Cleanup;


    procedure Default_Init_And_Register(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Cleanup_Op : in Operation_Ptr;  -- TBD: Static link
	Init_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access);
    pragma Global(Default_Init_And_Register);
    pragma Export(Ada, Default_Init_And_Register, 
      External_Name => "rts_default_init_and_register");

    -- Overview:
    --   This defers abort, and then calls Init_Op on Obj.
    --   If Init_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup operation.
                  
    procedure Adjust_And_Register(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access);
    pragma Global(Adjust_And_Register);
    pragma Export(Ada, Adjust_And_Register,
      External_Name => "rts_adjust_and_register");

    -- Overview:
    --   This defers abort, and then calls Adjust_Op on Obj.
    --   If Adjust_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup op.
    --   Program_Error is propagated if Adjust_Op propagates
    --   an exception.

    procedure Register_For_Cleanup(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access);
    pragma Global(Register_For_Cleanup);
    pragma Export(Ada, Register_For_Cleanup,
      External_Name => "rts_register_for_cleanup");

    -- Overview:
    --   This registers the cleanup record with the given
    --   obj and cleanup op.

    procedure Finalize_Copy_And_Adjust(
	Target_Obj : in System.Address;
	Source_Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Controlled_Part : in System.Address;
	Cleanup_Op : in Operation_Ptr;		-- TBD: Static Link
	Adjust_Op : in Operation_Ptr);		-- TBD: Static link
    pragma Global(Finalize_Copy_And_Adjust);
    pragma Export(Ada, Finalize_Copy_And_Adjust,
      External_Name => "rts_finalize_copy_and_adjust");

    -- Overview:
    --    This defers abort, finalizes Controlled_Part using Cleanup_Op,
    --    and if Cleanup_Op returns normally, copies Source_Obj to
    --    Target_Obj (Storage_Size gives number of storage elements),
    --    and then calls Adjust_Op on Controlled_Part.
    --    If either Cleanup_Op or Adjust_Op propagates an exception,
    --    Program_Error is raised.
    --    Note that abort is allowed to occur between finalize and copy, 
    --    but not between copy and adjust.

    procedure Preregister_Func_Result(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address);
    pragma Global(Preregister_Func_Result);
    pragma Export(Ada, Preregister_Func_Result,
      External_Name => "rts_preregister_func_result");

    -- Overview:
    --    This registers a cleanup record for the result of a
    --    function call.  It must be called immediately prior
    --    to calling any function returning an object requiring
    --    cleanup.  The cleanup record is only partially filled
    --    in at this point.

    procedure Reregister_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Obj_Cleanup_Rec : in Simple_Cleanup_Ptr);
    pragma Global(Reregister_Func_Result);
    pragma Export(Ada, Reregister_Func_Result,
      External_Name => "rts_reregister_func_result");

    -- Overview:
    --    This is to be called before returning a local object
    --    from a function into caller-allocated space.  It cancels
    --    the finalization for the object in the local scope
    --    (the finalization associated with Obj_Cleanup_Rec).
    --    Master is finalized and popped, and Obj is registered
    --    on a master in the caller of the function, using a
    --    cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

    procedure Adjust_And_Reregister_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr);	-- TBD: Static link
    pragma Global(Adjust_And_Reregister_Func_Result);
    pragma Export(Ada, Adjust_And_Reregister_Func_Result,
      External_Name => "rts_adjust_and_reregister_func_result");

    -- Overview:
    --    This is to be called before returning a value from
    --    a function into caller-allocated space.  Obj is adjusted
    --    and registered on a master in the caller of the function,
    --    using a cleanup record which was specified using
    --    Register_Func_Result_Before_Call.  Master is finalized
    --    and popped.

    function Reregister_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Simple_Cleanup_Ptr;
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address;
    pragma Global(Reregister_Func_Result_Copy_On_SS);
    pragma Export(Ada, Reregister_Func_Result_Copy_On_SS,
      External_Name => "rts_reregister_func_result_copy_on_ss");

    -- Overview:
    --    This is to be called before returning a local object from
    --    a function.  It cancels the finalization for the object in
    --    the local scope (the finalization associated with
    --    Obj_Cleanup_Rec).  Master is finalized and popped.  Obj
    --    is copied to Target_Mark in the secondary stack, which is
    --    released to the end of the copy of Obj.  The interesting
    --    part of the copy of Obj (as determined from Obj_Cleanup_Rec)
    --    is registered on a master in the caller of the function,
    --    using a cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

    function Adjust_And_Reregister_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Simple_Cleanup_Ptr;
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address;
    pragma Global(Adjust_And_Reregister_Func_Result_Copy_On_SS);
    pragma Export(Ada, Adjust_And_Reregister_Func_Result_Copy_On_SS,
      External_Name => "rts_adjust_and_reregister_func_result_copy_on_ss");

    -- Overview:
    --    This is to be called before returning a local object with
    --    aliased components from a function.  It proceeds as
    --    Reregister_Func_Result_Copy_On_SS, except that after
    --    copying Obj to the secondary stack, it adjusts the copy
    --    and finalizes the original.


    -------------------------------------------------------
    --     A R R A Y   C L E A N U P                     --
    -------------------------------------------------------
    type Array_Cleanup is new Simple_Cleanup with
      record
	Component_Size : System.Storage_Elements.Storage_Count;
	Num_Components : Natural;
      end record;

    procedure Cleanup(Rec : Array_Cleanup);
    type Array_Cleanup_Ptr is access all Array_Cleanup;

    procedure Default_Init_And_Register_Array(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Component_Size : in System.Storage_Elements.Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;  -- TBD: Static link
	Init_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access);
    pragma Global(Default_Init_And_Register_Array);
    pragma Export(Ada, Default_Init_And_Register_Array, 
      External_Name => "rts_default_init_and_register_array");

    -- Overview:
    --   This defers abort, and then calls Init_Op on components of Obj.
    --   If Init_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup operation
                  
    procedure Adjust_And_Register_Array(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Component_Size : in System.Storage_Elements.Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access);
    pragma Global(Adjust_And_Register_Array);
    pragma Export(Ada, Adjust_And_Register_Array,
      External_Name => "rts_adjust_and_register_array");

    -- Overview:
    --   This defers abort, and then calls Adjust_Op on comps of Obj.
    --   If Adjust_Op returns normally, the cleanup record
    --   is registered with the given obj and cleanup op.
    --   Program_Error is propagated if Adjust_Op propagates
    --   an exception.

    procedure Register_Array_For_Cleanup(
	Master: in RTT.Master_Access;
	Cleanup_Rec : in System.Address;
	Obj : in System.Address;
	Component_Size : in System.Storage_Elements.Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Flush : in RTT.Master_Access);
    pragma Global(Register_Array_For_Cleanup);
    pragma Export(Ada, Register_Array_For_Cleanup,
      External_Name => "rts_register_array_for_cleanup");

    -- Overview:
    --   This registers the cleanup record with the given
    --   obj and cleanup op.

    procedure Bump_Array_Register(
	Cleanup_Rec : in out Array_Cleanup;
	New_Components : in Natural;
	Flush : in RTT.Master_Access);
    pragma Global(Bump_Array_Register);
    pragma Export(Ada, Bump_Array_Register,
      External_Name => "rts_bump_array_register");

    -- Overview:
    --   This incrementally registers new elements of a previously
    --   registered array.

    procedure Finalize_Copy_And_Adjust_Array(
	Target_Obj : in System.Address;
	Source_Obj : in System.Address;
	Component_Size : in System.Storage_Elements.Storage_Count;
	Num_Components : in Natural;
	Controlled_Part : in System.Address;
	Cleanup_Op : in Operation_Ptr;		-- TBD: Static Link
	Adjust_Op : in Operation_Ptr);		-- TBD: Static link
    pragma Global(Finalize_Copy_And_Adjust_Array);
    pragma Export(Ada, Finalize_Copy_And_Adjust_Array,
      External_Name => "rts_finalize_copy_and_adjust_array");

    -- Overview:
    --    This defers abort, and then one component at a time,
    --    finalizes component of Controlled_Part using Cleanup_Op,
    --    and if Cleanup_Op returns normally, copies component of 
    --    Source_Obj to component of Target_Obj (Component_Size
    --    determines number of SUs to copy)
    --    and then calls Adjust_Op on component of Controlled_Part.
    --    If either Cleanup_Op or Adjust_Op propagates an exception,
    --    Program_Error is raised.
    --    Note that abort is allowed to occur between finalize and copy, 
    --    but not between copy and adjust of a given component.
    --    Operation is performed left-to-right or right-to-left
    --    as necessary, depending on addresses.  If addresses
    --    are the same, nothing is done.


    procedure Preregister_Array_Func_Result(
	Master : in RTT.Master_Access;
	Cleanup_Rec : in System.Address);
    pragma Global(Preregister_Array_Func_Result);
    pragma Export(Ada, Preregister_Array_Func_Result,
      External_Name => "rts_preregister_array_func_result");

    -- Overview:
    --    This registers a cleanup record for the array result of
    --    a function call.  It must be called immediately prior
    --    to calling any function returning an array requiring
    --    cleanup.  The cleanup record is only partially filled
    --    in at this point.

    procedure Reregister_Array_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Obj_Cleanup_Rec : in Array_Cleanup_Ptr);
    pragma Global(Reregister_Array_Func_Result);
    pragma Export(Ada, Reregister_Array_Func_Result,
      External_Name => "rts_reregister_array_func_result");

    -- Overview:
    --    This is to be called before returning a local array object
    --    from a function into caller-allocated space.  It cancels
    --    the finalization for the object in the local scope
    --    (the finalization associated with Obj_Cleanup_Rec).
    --    Master is finalized and popped, and Obj is registered
    --    on a master in the caller of the function, using a
    --    cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

    procedure Adjust_And_Reregister_Array_Func_Result(
	Master : in RTT.Master_Access;
	Obj : in System.Address;			-- interesting part
	Component_Size : in System.Storage_Elements.Storage_Count;
	Num_Components : in Natural;
	Cleanup_Op : in Operation_Ptr;	-- TBD: Static link
	Adjust_Op : in Operation_Ptr);	-- TBD: Static link
    pragma Global(Adjust_And_Reregister_Array_Func_Result);
    pragma Export(Ada, Adjust_And_Reregister_Array_Func_Result,
      External_Name => "rts_adjust_and_reregister_array_func_result");

    -- Overview:
    --    This is to be called before returning an array value from
    --    a function into caller-allocated space.  The components of
    --    Obj are adjusted and registered on a master in the caller
    --    of the function, using a cleanup record which was specified
    --    using Register_Func_Result_Before_Call.  Master is finalized
    --    and popped.

    function Reregister_Array_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Array_Cleanup_Ptr;
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address;
    pragma Global(Reregister_Array_Func_Result_Copy_On_SS);
    pragma Export(Ada, Reregister_Array_Func_Result_Copy_On_SS,
      External_Name => "rts_reregister_array_func_result_copy_on_ss");

    -- Overview:
    --    This is to be called before returning a local array object
    --    from a function.  It cancels the finalization for the
    --    object in the local scope (the finalization associated with
    --    Obj_Cleanup_Rec).  Master is finalized and popped.  Obj
    --    is copied to Target_Mark in the secondary stack, which is
    --    released to the end of the copy of Obj.  The interesting
    --    part of the copy of Obj (as determined from Obj_Cleanup_Rec)
    --    is registered on a master in the caller of the function,
    --    using a cleanup record which was specified using
    --    Register_Func_Result_Before_Call.

    function Adjust_And_Reregister_Array_Func_Result_Copy_On_SS(
	Master : in RTT.Master_Access;
	Obj : in System.Address;
	Storage_Size : in System.Storage_Elements.Storage_Count;
	Obj_Cleanup_Rec : in Array_Cleanup_Ptr;
	Adjust_Op : in Operation_Ptr;	-- TBD: Static link
	Target_Mark : Secondary_Mark_Ptr
    ) return System.Address;
    pragma Global(Adjust_And_Reregister_Array_Func_Result_Copy_On_SS);
    pragma Export(Ada, Adjust_And_Reregister_Array_Func_Result_Copy_On_SS,
      External_Name => "rts_adjust_and_reregister_array_func_result_copy_on_ss");

    -- Overview:
    --    This is to be called before returning a local array object
    --    with aliased components from a function.  It proceeds as
    --    Reregister_Array_Func_Result_Copy_On_SS, except that after
    --    copying Obj to the secondary stack, it adjusts the copy
    --    and finalizes the original.



    -------------------------------------------------------
    --     A C C E S S    C O L L E C T I O N S          --
    -------------------------------------------------------
    type Collection_Element;
    type Collection_Element_Ptr is access all Collection_Element;
    type Collection_Data is null record;

    type Collection_Element is 
	-- Object within a collection needing cleanup
	-- is doubly-linked into collection-cleanup
      record
        Next, Prev : Collection_Element_Ptr;
        Data : aliased Collection_Data;
      end record;

    type Collection_Cleanup is new RTT.Cleanup_Record with
      record
	List : Collection_Element_Ptr;  -- Doubly linked list
	Cleanup_Op : Operation_Ptr;	-- Operation to cleanup one element
	Offset : System.Storage_Elements.Storage_Offset;
					-- Offset to "interesting" part
      end record;
    procedure Cleanup(Rec : Collection_Cleanup);
    type Collection_Cleanup_Ptr is access all Collection_Cleanup;

    procedure Register_Collection_For_Cleanup(
	Master: in RTT.Master_Access;
	Collection : in System.Address;
	Offset : in System.Storage_Elements.Storage_Offset;
	Cleanup_Op : in Operation_Ptr);	-- TBD: Static link
    pragma Global(Register_Collection_For_Cleanup);
    pragma Export(Ada, Register_Collection_For_Cleanup,
      External_Name => "rts_register_collection_for_cleanup");

    -- Overview:
    --   Adds collection cleanup record to given master
    --   (or to default master, if none).
    --   The collection is cleaned up as a whole when the
    --   corresponding scope is exited.

    procedure Link_Collection_Element(
	Collection : in Collection_Cleanup_Ptr;
	Element : in System.Address;
	Flush : in RTT.Master_Access);
    pragma Global(Link_Collection_Element);
    pragma Export(Ada, Link_Collection_Element,
      External_Name => "rts_link_collection_element");

    -- Overview:
    --   This links the element into the collection list,
    --   so that it eventually gets cleaned up.

    procedure Finalize_And_Unlink_Collection_Element(
	Collection : in Collection_Cleanup_Ptr;
	Element : System.Address);
    pragma Global(Finalize_And_Unlink_Collection_Element);
    pragma Export(Ada, Finalize_And_Unlink_Collection_Element,
      External_Name => "rts_finalize_and_unlink_collection_element");
    -- Overview:
    --   This defers abort, calls the Cleanup_Op on Element.Data+Offset,
    --   and then unlinks the element from the collection list,
    --   so that it doesn't get cleaned up twice.
    --   If Cleanup_Op propagates an exception, Program_Error is raised,
    --   but the element is unlinked anyway.
    --   This is intended to be called by Unchecked_Deallocation.

    -------------------------------------------------------
    --     C L A S S W I D E    C L E A N U P            --
    -------------------------------------------------------
    procedure Null_Operation(Obj : in System.Address);
    pragma Global(Null_Operation);
    pragma Export(Ada, Null_Operation, External_Name => "rts_null_operation");
    -- Overview:
    --   This operation just returns.  It is put into
    --   Adjust_Whole_Object and Finalize_Whole_Object slots
    --   for tagged types that have no controlled parts.

    -- See also classwide finalization operations in the body of Ada.Tags.


end System.RTS.Finalization_Pkg;
