-- **********************************************************************
--
--  COPYRIGHT NOTICE:
-- "This unpublished work is protected both as a proprietary work and
-- under the Universal Copyright Convention and the US Copyright Act of
-- 1976. Its distribution and access are limited only to authorized
-- persons. Copr. (c) Alsys. Created 1990, initially licensed 1990.
-- All rights reserved.
--
-- Unauthorized use (including use to prepare other works), disclosure,
-- reproduction, or distribution may violate national criminal law."
--
-- **********************************************************************

------------------------------------------------------------------------------
-- This module defines the interface for the Intrinsic routines used to     --
-- generate machine language in the context of 386 Ada source programs.     --
------------------------------------------------------------------------------

with SYSTEM;
package MACHINE_OPERATIONS_386 is

   --	    *********************
   --	    * TABLE OF CONTENTS *
   --	    *********************

   --	      1. Register References
   --	      2. Immediate Operands
   --	      3. CALL/JUMP Targets
   --	      4. Memory References
   --	      5. Offset Operations
   --	      6. Intrinsic Operations
   --	      7. Floating-Point Operations
   --	      8. Interspersed Ada Statements
   --	      9. Flag Test Operations
   --	      10. Suppressing Subprogram Prolog/Epilog Code
   --	      11. Private Declarations of Operand Types


   --	    **************************
   --	    * 1. REGISTER REFERENCES *
   --	    **************************

   -- Define the types used for register values. The only allowed use of these
   -- types is to pass explicit constants as register parameters (it is not
   -- permitted to define and use register variables for this purpose).

   -- In many of the procedures where REG is used, the semantics of the
   -- instructions impose additional constraints. For example in a MOV
   -- call, both registers must be of the same type. These semantic
   -- restrictions follow exactly those of the assembly language.

   type REG is (
      AH,  AL,	BH,  BL,  CH,  CL,  DH,  DL,	-- General registers
      AX,  BX,	CX,  DX,  SI,  DI,  BP,  SP,
      EAX, EBX, ECX, EDX, ESI, EDI, EBP, ESP);

   subtype REG8  is REG range AH  .. DL;	-- Byte registers (8 bits)
   subtype REG16 is REG range AX  .. SP;	-- Word registers (16 bits)
   subtype REG32 is REG range EAX .. ESP;	-- Dword registers (32 bits)

   subtype XREG is REG range EAX .. EBP;	-- Index register

   subtype REG_DX is REG range DX .. DX;	-- Port in I/O instruction
   subtype REG_CL is REG range CL .. CL;	-- Count in shift instruction
   subtype REG_AX is REG range AX .. AX;	-- Destination for FSTSW

   type SREG is (DS, ES, SS, CS, FS, GS);	-- Segment registers

   type CREG is (CR0, CR2, CR3);		-- Control registers

   type DREG is (DR0, DR1, DR2, DR3, DR6, DR7); -- Debug registers

   type TREG is (TR6, TR7);			-- Test registers

   type FREG is (ST0, ST1, ST2, ST3,		-- Floating-point registers
		 ST4 ,ST5, ST6, ST7);		-- STn is n from stack top
   subtype FREG_ST is FREG range ST0 .. ST0;    -- Src for stack instructions

   subtype ST_07 is INTEGER range 0 .. 7;	-- Allowable FPT REG numbers
   function ST (STRG : ST_07 := 0) return FREG;
   -- Allows more familiar ST(4) notation
   -- with default use of ST to mean ST(0)


   --	    *************************
   --	    * 2. IMMEDIATE OPERANDS *
   --	    *************************

   subtype IMM is INTEGER;

   -- This type is used for immediate values in instructions. The permitted
   -- range depends on the context:

   --	    Byte:   -128   .. +255;
   --	    Word:   -32768 .. +65535;
   --	    Dword:  full integer range

   -- The range permitted in a given situation is context dependent. For
   -- example, in a MOV immediate instruction, it depends on the type of
   -- the target register for the move.

   subtype IMM8 is INTEGER range -128 .. +255;
   subtype IMM16 is INTEGER range -32768 .. +65535;
   -- These types are used where the range is restricted as indicated

   -- Note: in the case of Dword, the range does not include unsigned values
   -- greater than 2**31-1. Such values can be conveniently expressed using
   -- the following constant:

   U : constant := 2 ** 32;

   -- For example, the replacement for the assembly language instruction:

   --	  and	   eax,0F0F0F0F0H

   -- can be written using the Intrinsic call

   --	  AND	 (EAX, 16#F0F0F0F0# - U);

   subtype CTR is INTEGER range 0 .. 31;
   -- used for shift counts


   --	    ************************
   --	    * 3. CALL/JUMP TARGETS *
   --	    ************************

   subtype LBL is SYSTEM.ADDRESS;

   -- The LBL operand may be used for the target of jump and call instructions.
   -- All such jumps and calls are NEAR jumps (i.e. within the current segment,
   -- following the basic assumption that all code is in a single code segment).
   -- Other specialized forms of jump and call instructions (not using operands
   -- of type LBL) are provided for intersegment calls and jumps.

   -- The only permissible form of operands of type LBL are explicit uses of
   -- the 'ADDRESS attribute, applied to a visible label.


   --	    ************************
   --	    * 4. MEMORY REFERENCES *
   --	    ************************

   -- All memory references use the type MEM. MEM values can be obtained
   -- only by explicit calls to one of the PTR routines described below.
   -- It is not permissible to declare variables of this type, or to rename
   -- MEM values, or to define additional subprograms using this type.

   type MEM is limited private;

   -- Conceptually, a MEM value has five components (offset, index register,
   -- base register, segment override, and length in bytes). These values
   -- may be set explicitly, or implicitly by reference to Ada variables.

   -- The general name PTR is associated with the routines that return
   -- MEM values, to echo the use of this keyword in the standard assembly
   -- language. Of course other names may be chosen as considered convenient
   -- by renaming.

   -- There are several versions of the PTR routine, depending on which set
   -- of parameters is required. Note that the reason that Ada parameter
   -- defaults are not used for this purpose is to simplify the use of
   -- renaming declarations, as described in a later section).

   function L_ZZOZ (OFFSET : IMM;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (OFFSET : IMM;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_ZZOZ;
   pragma Import (Intrinsic, L_ZZOZ, Link_Name => "L_ZZOZ");

   function L_SZOZ (SEG	: SREG;
		 OFFSET : IMM;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (SEG	: SREG;
		 OFFSET : IMM;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_SZOZ;
   pragma Import (Intrinsic, L_SZOZ, Link_Name => "L_SZOZ");

   function L_ZBOZ (BASE: REG;
		 OFFSET : IMM := 0;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (BASE	: REG;
		 OFFSET : IMM := 0;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_ZBOZ;
   pragma Import (Intrinsic, L_ZBOZ, Link_Name => "L_ZBOZ");

   function L_SBOZ (SEG	: SREG;
		 BASE	: REG;
		 OFFSET : IMM := 0;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (SEG	: SREG;
		 BASE	: REG;
		 OFFSET : IMM := 0;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_SBOZ;
   pragma Import (Intrinsic, L_SBOZ, Link_Name => "L_SBOZ");

   function L_ZZOX (OFFSET : IMM;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (OFFSET : IMM;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_ZZOX;
   pragma Import (Intrinsic, L_ZZOX, Link_Name => "L_ZZOX");

   function L_SZOX (SEG	: SREG;
		 OFFSET : IMM;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (SEG	: SREG;
		 OFFSET : IMM;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_SZOX;
   pragma Import (Intrinsic, L_SZOX, Link_Name => "L_SZOX");

   function L_ZBOX (BASE: REG;
		 OFFSET : IMM := 0;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (BASE	: REG;
		 OFFSET : IMM := 0;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_ZBOX;
   pragma Import (Intrinsic, L_ZBOX, Link_Name => "L_ZBOX");

   function L_SBOX (SEG	: SREG;
		 BASE	: REG;
		 OFFSET : IMM := 0;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM;
   function PTR (SEG	: SREG;
		 BASE	: REG;
		 OFFSET : IMM := 0;
		 INDEX	: XREG;
		 LENGTH : NATURAL := 0)
		 return MEM renames L_SBOX;
   pragma Import (Intrinsic, L_SBOX, Link_Name => "L_SBOX");

   -- All actual parameters must be static expressions of the indicated type.
   -- The one exception to this rule occurs with XREG, where either a simple
   -- index register reference may be used, or an expression of the form
   -- index*scale, where index is an index register, and scale is one of the
   -- possible constant values 1,2,4 or 8.

   function "*" (INDEX : XREG; SCALE : IMM) return XREG;
   -- Dummy function to allow index scaling with assembler notation

   -- If the length is left at the default value of 0, then the resulting
   -- memory reference has no length. Such memory references are permitted
   -- whereever untyped references would be permitted in the assembly
   -- language (e.g. as one operand of a move, where the other operand,
   -- a register, supplies the required type).

   -- The following variants on these PTR routines provide a default length
   -- corresponding to the name.

   -- BYTE_PTR routines provide a default length of 1

   function B_ZZOZ (OFFSET : IMM)
		      return MEM;
   function BYTE_PTR (OFFSET : IMM)
		      return MEM renames B_ZZOZ;
   pragma Import (Intrinsic, B_ZZOZ, Link_Name => "B_ZZOZ");

   function B_SZOZ    (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM;
   function BYTE_PTR (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM renames B_SZOZ;
   pragma Import (Intrinsic, B_SZOZ, Link_Name => "B_SZOZ");

   function B_ZBOZ (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function BYTE_PTR (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames B_ZBOZ;
   pragma Import (Intrinsic, B_ZBOZ, Link_Name => "B_ZBOZ");

   function B_SBOZ (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function BYTE_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames B_SBOZ;
   pragma Import (Intrinsic, B_SBOZ, Link_Name => "B_SBOZ");

   function B_ZZOX (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM;
   function BYTE_PTR (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM renames B_ZZOX;
   pragma Import (Intrinsic, B_ZZOX, Link_Name => "B_ZZOX");

   function B_SZOX (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM;
   function BYTE_PTR (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM renames B_SZOX;
   pragma Import (Intrinsic, B_SZOX, Link_Name => "B_SZOX");

   function B_ZBOX (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function BYTE_PTR (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames B_ZBOX;
   pragma Import (Intrinsic, B_ZBOX, Link_Name => "B_ZBOX");

   function B_SBOX (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function BYTE_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames B_SBOX;
   pragma Import (Intrinsic, B_SBOX, Link_Name => "B_SBOX");

   -- WORD_PTR routines provide a default length of 2

   function W_ZZOZ (OFFSET : IMM)
		      return MEM;
   function WORD_PTR (OFFSET : IMM)
		      return MEM renames W_ZZOZ;
   pragma Import (Intrinsic, W_ZZOZ, Link_Name => "W_ZZOZ");

   function W_SZOZ    (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM;
   function WORD_PTR (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM renames W_SZOZ;
   pragma Import (Intrinsic, W_SZOZ, Link_Name => "W_SZOZ");

   function W_ZBOZ (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function WORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames W_ZBOZ;
   pragma Import (Intrinsic, W_ZBOZ, Link_Name => "W_ZBOZ");

   function W_SBOZ (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function WORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames W_SBOZ;
   pragma Import (Intrinsic, W_SBOZ, Link_Name => "W_SBOZ");

   function W_ZZOX (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM;
   function WORD_PTR (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM renames W_ZZOX;
   pragma Import (Intrinsic, W_ZZOX, Link_Name => "W_ZZOX");

   function W_SZOX (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM;
   function WORD_PTR (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM renames W_SZOX;
   pragma Import (Intrinsic, W_SZOX, Link_Name => "W_SZOX");

   function W_ZBOX (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function WORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames W_ZBOX;
   pragma Import (Intrinsic, W_ZBOX, Link_Name => "W_ZBOX");

   function W_SBOX (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function WORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames W_SBOX;
   pragma Import (Intrinsic, W_SBOX, Link_Name => "W_SBOX");

   -- DWORD_PTR routines provide a default length of 4

   function D_ZZOZ (OFFSET : IMM)
		      return MEM;
   function DWORD_PTR (OFFSET : IMM)
		      return MEM renames D_ZZOZ;
   pragma Import (Intrinsic, D_ZZOZ, Link_Name => "D_ZZOZ");

   function D_SZOZ    (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM;
   function DWORD_PTR (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM renames D_SZOZ;
   pragma Import (Intrinsic, D_SZOZ, Link_Name => "D_SZOZ");

   function D_ZBOZ (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function DWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames D_ZBOZ;
   pragma Import (Intrinsic, D_ZBOZ, Link_Name => "D_ZBOZ");

   function D_SBOZ (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function DWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames D_SBOZ;
   pragma Import (Intrinsic, D_SBOZ, Link_Name => "D_SBOZ");

   function D_ZZOX (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM;
   function DWORD_PTR (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM renames D_ZZOX;
   pragma Import (Intrinsic, D_ZZOX, Link_Name => "D_ZZOX");

   function D_SZOX (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM;
   function DWORD_PTR (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM renames D_SZOX;
   pragma Import (Intrinsic, D_SZOX, Link_Name => "D_SZOX");

   function D_ZBOX (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function DWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames D_ZBOX;
   pragma Import (Intrinsic, D_ZBOX, Link_Name => "D_ZBOX");

   function D_SBOX (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function DWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames D_SBOX;
   pragma Import (Intrinsic, D_SBOX, Link_Name => "D_SBOX");

   -- FDWORD_PTR routines provide a default length of 6

   function F_ZZOZ (OFFSET : IMM)
		      return MEM;
   function FWORD_PTR (OFFSET : IMM)
		      return MEM renames F_ZZOZ;
   pragma Import (Intrinsic, F_ZZOZ, Link_Name => "F_ZZOZ");

   function F_SZOZ    (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM;
   function FWORD_PTR (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM renames F_SZOZ;
   pragma Import (Intrinsic, F_SZOZ, Link_Name => "F_SZOZ");

   function F_ZBOZ (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function FWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames F_ZBOZ;
   pragma Import (Intrinsic, F_ZBOZ, Link_Name => "F_ZBOZ");

   function F_SBOZ (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function FWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames F_SBOZ;
   pragma Import (Intrinsic, F_SBOZ, Link_Name => "F_SBOZ");

   function F_ZZOX (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM;
   function FWORD_PTR (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM renames F_ZZOX;
   pragma Import (Intrinsic, F_ZZOX, Link_Name => "F_ZZOX");

   function F_SZOX (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM;
   function FWORD_PTR (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM renames F_SZOX;
   pragma Import (Intrinsic, F_SZOX, Link_Name => "F_SZOX");

   function F_ZBOX (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function FWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames F_ZBOX;
   pragma Import (Intrinsic, F_ZBOX, Link_Name => "F_ZBOX");

   function F_SBOX (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function FWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames F_SBOX;
   pragma Import (Intrinsic, F_SBOX, Link_Name => "F_SBOX");

   -- QWORF_PTR routines provide a default length of 8

   function Q_ZZOZ (OFFSET : IMM)
		      return MEM;
   function QWORD_PTR (OFFSET : IMM)
		      return MEM renames Q_ZZOZ;
   pragma Import (Intrinsic, Q_ZZOZ, Link_Name => "Q_ZZOZ");

   function Q_SZOZ    (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM;
   function QWORD_PTR (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM renames Q_SZOZ;
   pragma Import (Intrinsic, Q_SZOZ, Link_Name => "Q_SZOZ");

   function Q_ZBOZ (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function QWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames Q_ZBOZ;
   pragma Import (Intrinsic, Q_ZBOZ, Link_Name => "Q_ZBOZ");

   function Q_SBOZ (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function QWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames Q_SBOZ;
   pragma Import (Intrinsic, Q_SBOZ, Link_Name => "Q_SBOZ");

   function Q_ZZOX (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM;
   function QWORD_PTR (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM renames Q_ZZOX;
   pragma Import (Intrinsic, Q_ZZOX, Link_Name => "Q_ZZOX");

   function Q_SZOX (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM;
   function QWORD_PTR (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM renames Q_SZOX;
   pragma Import (Intrinsic, Q_SZOX, Link_Name => "Q_SZOX");

   function Q_ZBOX (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function QWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames Q_ZBOX;
   pragma Import (Intrinsic, Q_ZBOX, Link_Name => "Q_ZBOX");

   function Q_SBOX (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function QWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames Q_SBOX;
   pragma Import (Intrinsic, Q_SBOX, Link_Name => "Q_SBOX");

   -- TWORQ_PTR routines provide a default length of 10

   function T_ZZOZ (OFFSET : IMM)
		      return MEM;
   function TWORD_PTR (OFFSET : IMM)
		      return MEM renames T_ZZOZ;
   pragma Import (Intrinsic, T_ZZOZ, Link_Name => "T_ZZOZ");

   function T_SZOZ    (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM;
   function TWORD_PTR (SEG    : SREG;
		      OFFSET : IMM)
		      return MEM renames T_SZOZ;
   pragma Import (Intrinsic, T_SZOZ, Link_Name => "T_SZOZ");

   function T_ZBOZ (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function TWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames T_ZBOZ;
   pragma Import (Intrinsic, T_ZBOZ, Link_Name => "T_ZBOZ");

   function T_SBOZ (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM;
   function TWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0)
		      return MEM renames T_SBOZ;
   pragma Import (Intrinsic, T_SBOZ, Link_Name => "T_SBOZ");

   function T_ZZOX (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM;
   function TWORD_PTR (OFFSET : IMM;
		     INDEX   : XREG)
		     return MEM renames T_ZZOX;
   pragma Import (Intrinsic, T_ZZOX, Link_Name => "T_ZZOX");

   function T_SZOX (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM;
   function TWORD_PTR (SEG    : SREG;
		      OFFSET : IMM;
		      INDEX : XREG)
		      return MEM renames T_SZOX;
   pragma Import (Intrinsic, T_SZOX, Link_Name => "T_SZOX");

   function T_ZBOX (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function TWORD_PTR (BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames T_ZBOX;
   pragma Import (Intrinsic, T_ZBOX, Link_Name => "T_ZBOX");

   function T_SBOX (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM;
   function TWORD_PTR (SEG    : SREG;
		      BASE   : REG;
		      OFFSET : IMM := 0;
		      INDEX  : XREG)
		      return MEM renames T_SBOX;
   pragma Import (Intrinsic, T_SBOX, Link_Name => "T_SBOX");


   -- Examples of memory operands using PTR calls

   --	  MOV (AX, PTR (16#1234_5678#));
   --	  MOV (EAX, PTR (EAX,EBX*4);
   --	  INC (BYTE_PTR (EBX,4));
   --	  MOV (DWORD_PTR (SS, EBP, 12));

   -- Note: although it is not permissible to define constants of type MEM,
   -- this effect can be attained through function renaming. For example,
   -- the declaration:

   --  function RTN_PT (OFFSET : IMM := 4;
   --			BASE   : REG := EBP) return MEM
   --  renames QWORD_PTR;

   -- defines a name RTN_PT that may be used in an instrinsic function call
   -- where MEM is required, with the parameters set as specified


   -- The remaining PTR routines are for use in addressing existing Ada
   -- objects. All objects addressed using these routines must be directly
   -- addressable. An object is directly adressable if:

   -- It is either global (declared at the library level), or local
   -- (declared in the current scope).

   -- It is either a static object, or a component of a static object.
   -- Static objects include all scalars, records whose components are
   -- themselves static, and arrays whose index subtypes are static.

   -- Note: certain non-static objects can be accessed by use of the OFFSET
   -- function which allows the address of an object to be loaded.


   -- All these PTR functions are generic templates which must be instantiated
   -- with an appropriate type, and then the function can be used to obtain
   -- a MEM value to be used in a memory reference instruction. As with the
   -- explicit functions, there is a general function where the length can be
   -- either left unspecified, or set to a required value, and a set of
   -- functions in which the length is implied. The general PTR function can
   -- be applied to any type, but the specific ones can only be applied to
   -- types of appropriate lengths. For example WORD_PTR requires a type
   -- whose SIZE is 16.

   -- Note: the names of these routines are supplied at instantiation time,
   -- and may of course be anything the programmer wants. The canonical names
   -- PTR, BYTE_PTR etc. are suggested. In addition, these generic functions
   -- are known to the compiler and should not be renamed. Similarly, 
   -- instantiations of these functions should not be renamed.

   generic
      type T is limited private;
   function PTR_TEMPLATE (OBJECT : T; LENGTH : NATURAL := 0) return MEM;
   -- The LENGTH must be a static expression


   -- Add an implicit SIZE parameter so codegen can tell which function
   -- was instantiated.  This parameter is not visible to the user in 
   -- the documentation, and because it is defaulted the user does not
   -- need to know about it.  Note that with this scheme, the size
   -- for all instantiations, including PTR_TEMPLATE, is the actual
   -- value for the second parameter.
   
   type SIZE_TYPE is new INTEGER;

   generic
      type T is limited private;              -- length = 1,  T'SIZE = 8
   function BYTE_PTR_TEMPLATE (OBJECT : T;
                               SIZE   : SIZE_TYPE := -1) return MEM;

   generic
      type T is limited private;              -- length = 2,  T'SIZE = 16
   function WORD_PTR_TEMPLATE (OBJECT : T;
                               SIZE   : SIZE_TYPE := -2) return MEM;

   generic
      type T is limited private;              -- length = 4,  T'SIZE = 32
   function DWORD_PTR_TEMPLATE (OBJECT : T;
                                SIZE   : SIZE_TYPE := -4) return MEM;

   generic
      type T is limited private;              -- length = 6,  T'SIZE = 48
   function FWORD_PTR_TEMPLATE (OBJECT : T;
                                SIZE   : SIZE_TYPE := -6) return MEM;

   generic
      type T is limited private;              -- length = 8,  T'SIZE = 64
   function QWORD_PTR_TEMPLATE (OBJECT : T;
                                SIZE   : SIZE_TYPE := -8) return MEM;

   generic
      type T is limited private;              -- length = 10, T'SIZE = 80
   function TWORD_PTR_TEMPLATE (OBJECT : T;
                                SIZE   : SIZE_TYPE := -10) return MEM;
   

   --	    ************************
   --	    * 5. OFFSET OPERATIONS *
   --	    ************************

   -- The offset operations allow obtaining the offset of Ada variables. For
   -- example, the instruction:

   --	MOV (EAX, OFFSET (JUNK));

   -- moves the offset of the variable junk into EAX. Note that in this model
   -- we assume that all Ada variables are in the same (DS) segment, so there
   -- is no need for a comparable SEGMENT function.

   -- The OFFSET operation can only be instantiated with variables that can
   -- be directly addressed (as described for MEM), or with records and arrays
   -- that are declared locally or globally (including dynamic records and
   -- arrays -- allowing such objects to be addressed from machine code).

   -- Note that when the offset of a dynamic object is required, it may be
   -- the case that an indirect reference is required, i.e. the required
   -- offset is stored in a memory location, that is itself directly
   -- addressable. In this case, what appears to be an immediate reference
   -- in the code may in fact generate a memory reference:

   --	  MOV (ESI, OFFSET (STATIC_ARR));	  -- MOV reg,immediate
   --	  MOV (ESI, OFFSET (DYNAM_ARR));	  -- MOV reg,memory

   -- Since no extra registers are required, and it is still the case that
   -- only a single instruction is generated, the programmer need not be
   -- concerned about which form is generated (the effect is the same in
   -- both cases, namely to load the address of the object).

   type OFS is limited private;
 
   -- Add a marker parameter so codegen can verify that the use
   -- of an OFFSET function is an instantiation of this function.
   generic
      type T is limited private;
   function OFFSET_TEMPLATE (OBJECT : T;
                             MARKER : SIZE_TYPE := -1) return OFS;

   --	    ***************************
   --	    * 6. Intrinsic OPERATIONS *
   --	    ***************************

   -- This section defines the actual instruction forms. The name is xxx
   -- where xxx is the standard Intel/Microsoft assembler mnemonic. Each of
   -- these calls generates a single corresponding machine instruction.
   -- In cases where xxx is an Ada reserved word (and, in, loop, or, out,
   -- xor), the procedure name is preceded by I_.

   procedure I_AAA;
   pragma Import (Intrinsic, I_AAA, Link_Name => "I_AAA");
   procedure AAA renames I_AAA;

   procedure I_AAD;
   pragma Import (Intrinsic, I_AAD, Link_Name => "I_AAD");
   procedure AAD renames I_AAD;

   procedure I_AAM;
   pragma Import (Intrinsic, I_AAM, Link_Name => "I_AAM");
   procedure AAM renames I_AAM;

   procedure I_AAS;
   pragma Import (Intrinsic, I_AAS, Link_Name => "I_AAS");
   procedure AAS renames I_AAS;

   procedure I_ADC_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_ADC_RR, Link_Name => "I_ADC_RR");
   procedure ADC (DEST : REG; SRC : REG) renames I_ADC_RR;
   procedure I_ADC_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_ADC_MR, Link_Name => "I_ADC_MR");
   procedure ADC (DEST : MEM; SRC : REG) renames I_ADC_MR;
   procedure I_ADC_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_ADC_RM, Link_Name => "I_ADC_RM");
   procedure ADC (DEST : REG; SRC : MEM) renames I_ADC_RM;
   procedure I_ADC_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_ADC_RI, Link_Name => "I_ADC_RI");
   procedure ADC (DEST : REG; SRC : IMM) renames I_ADC_RI;
   procedure I_ADC_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_ADC_MI, Link_Name => "I_ADC_MI");
   procedure ADC (DEST : MEM; SRC : IMM) renames I_ADC_MI;

   procedure I_ADD_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_ADD_RR, Link_Name => "I_ADD_RR");
   procedure ADD (DEST : REG; SRC : REG) renames I_ADD_RR;
   procedure I_ADD_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_ADD_MR, Link_Name => "I_ADD_MR");
   procedure ADD (DEST : MEM; SRC : REG) renames I_ADD_MR;
   procedure I_ADD_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_ADD_RM, Link_Name => "I_ADD_RM");
   procedure ADD (DEST : REG; SRC : MEM) renames I_ADD_RM;
   procedure I_ADD_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_ADD_RI, Link_Name => "I_ADD_RI");
   procedure ADD (DEST : REG; SRC : IMM) renames I_ADD_RI;
   procedure I_ADD_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_ADD_MI, Link_Name => "I_ADD_MI");
   procedure ADD (DEST : MEM; SRC : IMM) renames I_ADD_MI;

   procedure I_AND_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_AND_RR, Link_Name => "I_AND_RR");
   procedure I_AND (DEST : REG; SRC : REG) renames I_AND_RR;
   procedure I_AND_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_AND_MR, Link_Name => "I_AND_MR");
   procedure I_AND (DEST : MEM; SRC : REG) renames I_AND_MR;
   procedure I_AND_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_AND_RM, Link_Name => "I_AND_RM");
   procedure I_AND (DEST : REG; SRC : MEM) renames I_AND_RM;
   procedure I_AND_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_AND_RI, Link_Name => "I_AND_RI");
   procedure I_AND (DEST : REG; SRC : IMM) renames I_AND_RI;
   procedure I_AND_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_AND_MI, Link_Name => "I_AND_MI");
   procedure I_AND (DEST : MEM; SRC : IMM) renames I_AND_MI;

   procedure I_ARPL_MR (DEST : MEM; SRC : REG16);
   pragma Import (Intrinsic, I_ARPL_MR, Link_Name => "I_ARPL_MR");
   procedure ARPL (DEST : MEM; SRC : REG16) renames I_ARPL_MR;
   procedure I_ARPL_RR (DEST : REG16; SRC : REG16);
   pragma Import (Intrinsic, I_ARPL_RR, Link_Name => "I_ARPL_RR");
   procedure ARPL (DEST : REG16; SRC : REG16) renames I_ARPL_RR;

   procedure I_BOUND_RM (SRC : REG; BOUNDS : MEM);
   pragma Import (Intrinsic, I_BOUND_RM, Link_Name => "I_BOUND_RM");
   procedure BOUND (SRC : REG; BOUNDS : MEM) renames I_BOUND_RM;

   procedure I_BSF_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_BSF_RR, Link_Name => "I_BSF_RR");
   procedure BSF (DEST : REG; SRC : REG) renames I_BSF_RR;
   procedure I_BSF_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_BSF_RM, Link_Name => "I_BSF_RM");
   procedure BSF (DEST : REG; SRC : MEM) renames I_BSF_RM;

   procedure I_BSR_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_BSR_RR, Link_Name => "I_BSR_RR");
   procedure BSR (DEST : REG; SRC : REG) renames I_BSR_RR;
   procedure I_BSR_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_BSR_RM, Link_Name => "I_BSR_RM");
   procedure BSR (DEST : REG; SRC : MEM) renames I_BSR_RM;

   procedure I_BSWAP_R (DEST : REG32);
   pragma Import (Intrinsic, I_BSWAP_R, Link_Name => "I_BSWAP_R");
   procedure BSWAP (DEST : REG32) renames I_BSWAP_R;

   procedure I_BT_RR (SRC : REG; INDEX : REG);
   pragma Import (Intrinsic, I_BT_RR, Link_Name => "I_BT_RR");
   procedure BT (SRC : REG; INDEX : REG) renames I_BT_RR;
   procedure I_BT_MR (SRC : MEM; INDEX : REG);
   pragma Import (Intrinsic, I_BT_MR, Link_Name => "I_BT_MR");
   procedure BT (SRC : MEM; INDEX : REG) renames I_BT_MR;
   procedure I_BT_RI (SRC : REG; INDEX : IMM8);
   pragma Import (Intrinsic, I_BT_RI, Link_Name => "I_BT_RI");
   procedure BT (SRC : REG; INDEX : IMM8) renames I_BT_RI;
   procedure I_BT_MI (SRC : MEM; INDEX : IMM8);
   pragma Import (Intrinsic, I_BT_MI, Link_Name => "I_BT_MI");
   procedure BT (SRC : MEM; INDEX : IMM8) renames I_BT_MI;

   procedure I_BTC_RR (SRC : REG; INDEX : REG);
   pragma Import (Intrinsic, I_BTC_RR, Link_Name => "I_BTC_RR");
   procedure BTC (SRC : REG; INDEX : REG) renames I_BTC_RR;
   procedure I_BTC_MR (SRC : MEM; INDEX : REG);
   pragma Import (Intrinsic, I_BTC_MR, Link_Name => "I_BTC_MR");
   procedure BTC (SRC : MEM; INDEX : REG) renames I_BTC_MR;
   procedure I_BTC_RI (SRC : REG; INDEX : IMM8);
   pragma Import (Intrinsic, I_BTC_RI, Link_Name => "I_BTC_RI");
   procedure BTC (SRC : REG; INDEX : IMM8) renames I_BTC_RI;
   procedure I_BTC_MI (SRC : MEM; INDEX : IMM8);
   pragma Import (Intrinsic, I_BTC_MI, Link_Name => "I_BTC_MI");
   procedure BTC (SRC : MEM; INDEX : IMM8) renames I_BTC_MI;

   procedure I_BTR_RR (SRC : REG; INDEX : REG);
   pragma Import (Intrinsic, I_BTR_RR, Link_Name => "I_BTR_RR");
   procedure BTR (SRC : REG; INDEX : REG) renames I_BTR_RR;
   procedure I_BTR_MR (SRC : MEM; INDEX : REG);
   pragma Import (Intrinsic, I_BTR_MR, Link_Name => "I_BTR_MR");
   procedure BTR (SRC : MEM; INDEX : REG) renames I_BTR_MR;
   procedure I_BTR_RI (SRC : REG; INDEX : IMM8);
   pragma Import (Intrinsic, I_BTR_RI, Link_Name => "I_BTR_RI");
   procedure BTR (SRC : REG; INDEX : IMM8) renames I_BTR_RI;
   procedure I_BTR_MI (SRC : MEM; INDEX : IMM8);
   pragma Import (Intrinsic, I_BTR_MI, Link_Name => "I_BTR_MI");
   procedure BTR (SRC : MEM; INDEX : IMM8) renames I_BTR_MI;

   procedure I_BTS_RR (SRC : REG; INDEX : REG);
   pragma Import (Intrinsic, I_BTS_RR, Link_Name => "I_BTS_RR");
   procedure BTS (SRC : REG; INDEX : REG) renames I_BTS_RR;
   procedure I_BTS_MR (SRC : MEM; INDEX : REG);
   pragma Import (Intrinsic, I_BTS_MR, Link_Name => "I_BTS_MR");
   procedure BTS (SRC : MEM; INDEX : REG) renames I_BTS_MR;
   procedure I_BTS_RI (SRC : REG; INDEX : IMM8);
   pragma Import (Intrinsic, I_BTS_RI, Link_Name => "I_BTS_RI");
   procedure BTS (SRC : REG; INDEX : IMM8) renames I_BTS_RI;
   procedure I_BTS_MI (SRC : MEM; INDEX : IMM8);
   pragma Import (Intrinsic, I_BTS_MI, Link_Name => "I_BTS_MI");
   procedure BTS (SRC : MEM; INDEX : IMM8) renames I_BTS_MI;

   procedure I_CALL_L (TARGET : LBL);
   pragma Import (Intrinsic, I_CALL_L, Link_Name => "I_CALL_L");
   procedure CALL (TARGET : LBL) renames I_CALL_L;
   procedure I_CALL_M (TARGET : MEM);
   pragma Import (Intrinsic, I_CALL_M, Link_Name => "I_CALL_M");
   procedure CALL (TARGET : MEM) renames I_CALL_M;
   procedure I_CALL_II (TARGET_SEG : IMM16; TARGET_OFS : IMM);
   pragma Import (Intrinsic, I_CALL_II, Link_Name => "I_CALL_II");
   procedure CALL (TARGET_SEG : IMM16; TARGET_OFS : IMM) renames I_CALL_II;

   procedure I_CBW;
   pragma Import (Intrinsic, I_CBW, Link_Name => "I_CBW");
   procedure CBW renames I_CBW;

   procedure I_CDQ;
   pragma Import (Intrinsic, I_CDQ, Link_Name => "I_CDQ");
   procedure CDQ renames I_CDQ;

   procedure I_CLC;
   pragma Import (Intrinsic, I_CLC, Link_Name => "I_CLC");
   procedure CLC renames I_CLC;

   procedure I_CLD;
   pragma Import (Intrinsic, I_CLD, Link_Name => "I_CLD");
   procedure CLD renames I_CLD;

   procedure I_CLI;
   pragma Import (Intrinsic, I_CLI, Link_Name => "I_CLI");
   procedure CLI renames I_CLI;

   procedure I_CLTS;
   pragma Import (Intrinsic, I_CLTS, Link_Name => "I_CLTS");
   procedure CLTS renames I_CLTS;

   procedure I_CMC;
   pragma Import (Intrinsic, I_CMC, Link_Name => "I_CMC");
   procedure CMC renames I_CMC;

   procedure I_CMP_RR (SRC1 : REG; SRC2 : REG);
   pragma Import (Intrinsic, I_CMP_RR, Link_Name => "I_CMP_RR");
   procedure CMP (SRC1 : REG; SRC2 : REG) renames I_CMP_RR;
   procedure I_CMP_MR (SRC1 : MEM; SRC2 : REG);
   pragma Import (Intrinsic, I_CMP_MR, Link_Name => "I_CMP_MR");
   procedure CMP (SRC1 : MEM; SRC2 : REG) renames I_CMP_MR;
   procedure I_CMP_RM (SRC1 : REG; SRC2 : MEM);
   pragma Import (Intrinsic, I_CMP_RM, Link_Name => "I_CMP_RM");
   procedure CMP (SRC1 : REG; SRC2 : MEM) renames I_CMP_RM;
   procedure I_CMP_RI (SRC1 : REG; SRC2 : IMM);
   pragma Import (Intrinsic, I_CMP_RI, Link_Name => "I_CMP_RI");
   procedure CMP (SRC1 : REG; SRC2 : IMM) renames I_CMP_RI;
   procedure I_CMP_MI (SRC1 : MEM; SRC2 : IMM);
   pragma Import (Intrinsic, I_CMP_MI, Link_Name => "I_CMP_MI");
   procedure CMP (SRC1 : MEM; SRC2 : IMM) renames I_CMP_MI;

   procedure I_CMPS_MM (DEST : MEM; SRC : MEM);
   pragma Import (Intrinsic, I_CMPS_MM, Link_Name => "I_CMPS_MM");
   procedure CMPS (DEST : MEM; SRC : MEM) renames I_CMPS_MM;

   procedure I_CMPSB;
   pragma Import (Intrinsic, I_CMPSB, Link_Name => "I_CMPSB");
   procedure CMPSB renames I_CMPSB;

   procedure I_CMPSD;
   pragma Import (Intrinsic, I_CMPSD, Link_Name => "I_CMPSD");
   procedure CMPSD renames I_CMPSD;

   procedure I_CMPSW;
   pragma Import (Intrinsic, I_CMPSW, Link_Name => "I_CMPSW");
   procedure CMPSW renames I_CMPSW;

   procedure I_CMPXCHG_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_CMPXCHG_RR, Link_Name => "I_CMPXCHG_RR");
   procedure CMPXCHG (DEST : REG; SRC : REG) renames I_CMPXCHG_RR;
   procedure I_CMPXCHG_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_CMPXCHG_MR, Link_Name => "I_CMPXCHG_MR");
   procedure CMPXCHG (DEST : MEM; SRC : REG) renames I_CMPXCHG_MR;

   procedure I_CWD;
   pragma Import (Intrinsic, I_CWD, Link_Name => "I_CWD");
   procedure CWD renames I_CWD;

   procedure I_CWDE;
   pragma Import (Intrinsic, I_CWDE, Link_Name => "I_CWDE");
   procedure CWDE renames I_CWDE;

   procedure I_DAA;
   pragma Import (Intrinsic, I_DAA, Link_Name => "I_DAA");
   procedure DAA renames I_DAA;

   procedure I_DAS;
   pragma Import (Intrinsic, I_DAS, Link_Name => "I_DAS");
   procedure DAS renames I_DAS;

   procedure I_DEC_M (DEST : MEM);
   pragma Import (Intrinsic, I_DEC_M, Link_Name => "I_DEC_M");
   procedure DEC (DEST : MEM) renames I_DEC_M;
   procedure I_DEC_R (DEST : REG);
   pragma Import (Intrinsic, I_DEC_R, Link_Name => "I_DEC_R");
   procedure DEC (DEST : REG) renames I_DEC_R;

   procedure I_DIV_R (SRC : REG);
   pragma Import (Intrinsic, I_DIV_R, Link_Name => "I_DIV_R");
   procedure DIV (SRC : REG) renames I_DIV_R;
   procedure I_DIV_M (SRC : MEM);
   pragma Import (Intrinsic, I_DIV_M, Link_Name => "I_DIV_M");
   procedure DIV (SRC : MEM) renames I_DIV_M;

   procedure I_ENTER_II (FRAME_SIZE : IMM16; LEVEL : IMM8);
   pragma Import (Intrinsic, I_ENTER_II, Link_Name => "I_ENTER_II");
   procedure ENTER (FRAME_SIZE : IMM16; LEVEL : IMM8) renames I_ENTER_II;

   procedure I_HLT;
   pragma Import (Intrinsic, I_HLT, Link_Name => "I_HLT");
   procedure HLT renames I_HLT;

   procedure I_IDIV_R (SRC : REG);
   pragma Import (Intrinsic, I_IDIV_R, Link_Name => "I_IDIV_R");
   procedure IDIV (SRC : REG) renames I_IDIV_R;
   procedure I_IDIV_M (SRC : MEM);
   pragma Import (Intrinsic, I_IDIV_M, Link_Name => "I_IDIV_M");
   procedure IDIV (SRC : MEM) renames I_IDIV_M;

   procedure I_IMUL_R (SRC : REG);
   pragma Import (Intrinsic, I_IMUL_R, Link_Name => "I_IMUL_R");
   procedure IMUL (SRC : REG) renames I_IMUL_R;
   procedure I_IMUL_M (SRC : MEM);
   pragma Import (Intrinsic, I_IMUL_M, Link_Name => "I_IMUL_M");
   procedure IMUL (SRC : MEM) renames I_IMUL_M;
   procedure I_IMUL_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_IMUL_RR, Link_Name => "I_IMUL_RR");
   procedure IMUL (DEST : REG; SRC : REG) renames I_IMUL_RR;
   procedure I_IMUL_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_IMUL_RM, Link_Name => "I_IMUL_RM");
   procedure IMUL (DEST : REG; SRC : MEM) renames I_IMUL_RM;
   procedure I_IMUL_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_IMUL_RI, Link_Name => "I_IMUL_RI");
   procedure IMUL (DEST : REG; SRC : IMM) renames I_IMUL_RI;
   procedure I_IMUL_RRI (DEST : REG; SRC1 : REG; SRC2 : IMM);
   pragma Import (Intrinsic, I_IMUL_RRI, Link_Name => "I_IMUL_RRI");
   procedure IMUL (DEST : REG; SRC1 : REG; SRC2 : IMM) renames I_IMUL_RRI;
   procedure I_IMUL_RMI (DEST : REG; SRC1 : MEM; SRC2 : IMM);
   pragma Import (Intrinsic, I_IMUL_RMI, Link_Name => "I_IMUL_RMI");
   procedure IMUL (DEST : REG; SRC1 : MEM; SRC2 : IMM) renames I_IMUL_RMI;

   procedure I_IN_RI (DEST : REG; PORT : IMM8);
   pragma Import (Intrinsic, I_IN_RI, Link_Name => "I_IN_RI");
   procedure I_IN (DEST : REG; PORT : IMM8) renames I_IN_RI;
   procedure I_IN_RR (DEST : REG; PORT : REG_DX);
   pragma Import (Intrinsic, I_IN_RR, Link_Name => "I_IN_RR");
   procedure I_IN (DEST : REG; PORT : REG_DX) renames I_IN_RR;

   procedure I_INC_M (DEST : MEM);
   pragma Import (Intrinsic, I_INC_M, Link_Name => "I_INC_M");
   procedure INC (DEST : MEM) renames I_INC_M;
   procedure I_INC_R (DEST : REG);
   pragma Import (Intrinsic, I_INC_R, Link_Name => "I_INC_R");
   procedure INC (DEST : REG) renames I_INC_R;

   procedure I_INS_MR (DEST : MEM; PORT : REG_DX);
   pragma Import (Intrinsic, I_INS_MR, Link_Name => "I_INS_MR");
   procedure INS (DEST : MEM; PORT : REG_DX) renames I_INS_MR;

   procedure I_INSB;
   pragma Import (Intrinsic, I_INSB, Link_Name => "I_INSB");
   procedure INSB renames I_INSB;

   procedure I_INSD;
   pragma Import (Intrinsic, I_INSD, Link_Name => "I_INSD");
   procedure INSD renames I_INSD;

   procedure I_INSW;
   pragma Import (Intrinsic, I_INSW, Link_Name => "I_INSW");
   procedure INSW renames I_INSW;

   procedure I_INT_I (INTNO : IMM8);
   pragma Import (Intrinsic, I_INT_I, Link_Name => "I_INT_I");
   procedure INT (INTNO : IMM8) renames I_INT_I;

   procedure I_INTO;
   pragma Import (Intrinsic, I_INTO, Link_Name => "I_INTO");
   procedure INTO renames I_INTO;

   procedure I_INVD;
   pragma Import (Intrinsic, I_INVD, Link_Name => "I_INVD");
   procedure INVD renames I_INVD;

   procedure I_INVLPG_M (SRC : MEM);
   pragma Import (Intrinsic, I_INVLPG_M, Link_Name => "I_INVLPG_M");
   procedure INVLPG (SRC : MEM) renames I_INVLPG_M;

   procedure I_IRET;
   pragma Import (Intrinsic, I_IRET, Link_Name => "I_IRET");
   procedure IRET renames I_IRET;

   procedure I_IRETD;
   pragma Import (Intrinsic, I_IRETD, Link_Name => "I_IRETD");
   procedure IRETD renames I_IRETD;

   procedure I_JA_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JA_L, Link_Name => "I_JA_L");
   procedure JA (TARGET : LBL) renames I_JA_L;

   procedure I_JAE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JAE_L, Link_Name => "I_JAE_L");
   procedure JAE (TARGET : LBL) renames I_JAE_L;

   procedure I_JB_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JB_L, Link_Name => "I_JB_L");
   procedure JB (TARGET : LBL) renames I_JB_L;

   procedure I_JBE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JBE_L, Link_Name => "I_JBE_L");
   procedure JBE (TARGET : LBL) renames I_JBE_L;

   procedure JC (TARGET : LBL) renames JB;

   procedure I_JCXZ_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JCXZ_L, Link_Name => "I_JCXZ_L");
   procedure JCXZ (TARGET : LBL) renames I_JCXZ_L;

   procedure I_JE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JE_L, Link_Name => "I_JE_L");
   procedure JE (TARGET : LBL) renames I_JE_L;

   procedure I_JECXZ_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JECXZ_L, Link_Name => "I_JECXZ_L");
   procedure JECXZ (TARGET : LBL) renames I_JECXZ_L;

   procedure I_JG_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JG_L, Link_Name => "I_JG_L");
   procedure JG (TARGET : LBL) renames I_JG_L;

   procedure I_JGE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JGE_L, Link_Name => "I_JGE_L");
   procedure JGE (TARGET : LBL) renames I_JGE_L;

   procedure I_JL_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JL_L, Link_Name => "I_JL_L");
   procedure JL (TARGET : LBL) renames I_JL_L;

   procedure I_JLE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JLE_L, Link_Name => "I_JLE_L");
   procedure JLE (TARGET : LBL) renames I_JLE_L;

   procedure I_JMP_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JMP_L, Link_Name => "I_JMP_L");
   procedure JMP (TARGET : LBL) renames I_JMP_L;
   procedure I_JMP_M (TARGET : MEM);
   pragma Import (Intrinsic, I_JMP_M, Link_Name => "I_JMP_M");
   procedure JMP (TARGET : MEM) renames I_JMP_M;
   procedure I_JMP_II (TARGET_SEG : IMM16; TARGET_OFS : IMM);
   pragma Import (Intrinsic, I_JMP_II, Link_Name => "I_JMP_II");
   procedure JMP (TARGET_SEG : IMM16; TARGET_OFS : IMM) renames I_JMP_II;

   procedure JNA (TARGET : LBL) renames JBE;

   procedure JNAE (TARGET : LBL) renames JB;

   procedure JNB (TARGET : LBL) renames JAE;

   procedure JNBE (TARGET : LBL) renames JA;

   procedure JNC (TARGET : LBL) renames JAE;

   procedure I_JNE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JNE_L, Link_Name => "I_JNE_L");
   procedure JNE (TARGET : LBL) renames I_JNE_L;

   procedure JNG (TARGET : LBL) renames JLE;

   procedure JNGE (TARGET : LBL) renames JL;

   procedure JNL (TARGET : LBL) renames JGE;

   procedure JNLE (TARGET : LBL) renames JG;

   procedure I_JNO_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JNO_L, Link_Name => "I_JNO_L");
   procedure JNO (TARGET : LBL) renames I_JNO_L;

   procedure I_JNP_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JNP_L, Link_Name => "I_JNP_L");
   procedure JNP (TARGET : LBL) renames I_JNP_L;

   procedure I_JNS_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JNS_L, Link_Name => "I_JNS_L");
   procedure JNS (TARGET : LBL) renames I_JNS_L;

   procedure JNZ (TARGET : LBL) renames JNE;

   procedure I_JO_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JO_L, Link_Name => "I_JO_L");
   procedure JO (TARGET : LBL) renames I_JO_L;

   procedure I_JP_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JP_L, Link_Name => "I_JP_L");
   procedure JP (TARGET : LBL) renames I_JP_L;

   procedure JPE (TARGET : LBL) renames JP;

   procedure JPO (TARGET : LBL) renames JNP;

   procedure I_JS_L (TARGET : LBL);
   pragma Import (Intrinsic, I_JS_L, Link_Name => "I_JS_L");
   procedure JS (TARGET : LBL) renames I_JS_L;

   procedure JZ (TARGET : LBL) renames JE;

   procedure I_LAHF;
   pragma Import (Intrinsic, I_LAHF, Link_Name => "I_LAHF");
   procedure LAHF renames I_LAHF;

   procedure I_LAR_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_LAR_RM, Link_Name => "I_LAR_RM");
   procedure LAR (DEST : REG; SRC : MEM) renames I_LAR_RM;

   procedure I_LDS_RM (DEST : REG32; SRC : MEM);
   pragma Import (Intrinsic, I_LDS_RM, Link_Name => "I_LDS_RM");
   procedure LDS (DEST : REG32; SRC : MEM) renames I_LDS_RM;

   procedure I_LEA_RM (DEST : REG32; SRC : MEM);
   pragma Import (Intrinsic, I_LEA_RM, Link_Name => "I_LEA_RM");
   procedure LEA (DEST : REG32; SRC : MEM) renames I_LEA_RM;

   procedure I_LEAVE;
   pragma Import (Intrinsic, I_LEAVE, Link_Name => "I_LEAVE");
   procedure LEAVE renames I_LEAVE;

   procedure I_LES_RM (DEST : REG32; SRC : MEM);
   pragma Import (Intrinsic, I_LES_RM, Link_Name => "I_LES_RM");
   procedure LES (DEST : REG32; SRC : MEM) renames I_LES_RM;

   procedure I_LFS_RM (DEST : REG32; SRC : MEM);
   pragma Import (Intrinsic, I_LFS_RM, Link_Name => "I_LFS_RM");
   procedure LFS (DEST : REG32; SRC : MEM) renames I_LFS_RM;

   procedure I_LGDT_M (SRC : MEM);
   pragma Import (Intrinsic, I_LGDT_M, Link_Name => "I_LGDT_M");
   procedure LGDT (SRC : MEM) renames I_LGDT_M;

   procedure I_LGS_RM (DEST : REG32; SRC : MEM);
   pragma Import (Intrinsic, I_LGS_RM, Link_Name => "I_LGS_RM");
   procedure LGS (DEST : REG32; SRC : MEM) renames I_LGS_RM;

   procedure I_LIDT_M (SRC : MEM);
   pragma Import (Intrinsic, I_LIDT_M, Link_Name => "I_LIDT_M");
   procedure LIDT (SRC : MEM) renames I_LIDT_M;

   procedure I_LLDT_M (SRC : MEM);
   pragma Import (Intrinsic, I_LLDT_M, Link_Name => "I_LLDT_M");
   procedure LLDT (SRC : MEM) renames I_LLDT_M;
   procedure I_LLDT_R (SRC : REG);
   pragma Import (Intrinsic, I_LLDT_R, Link_Name => "I_LLDT_R");
   procedure LLDT (SRC : REG) renames I_LLDT_R;

   procedure I_LMSW_M (SRC : MEM);
   pragma Import (Intrinsic, I_LMSW_M, Link_Name => "I_LMSW_M");
   procedure LMSW (SRC : MEM) renames I_LMSW_M;
   procedure I_LMSW_R (SRC : REG);
   pragma Import (Intrinsic, I_LMSW_R, Link_Name => "I_LMSW_R");
   procedure LMSW (SRC : REG) renames I_LMSW_R;

   procedure I_LOCK;
   pragma Import (Intrinsic, I_LOCK, Link_Name => "I_LOCK");
   procedure LOCK renames I_LOCK;

   procedure I_LODS_M (SRC : MEM);
   pragma Import (Intrinsic, I_LODS_M, Link_Name => "I_LODS_M");
   procedure LODS (SRC : MEM) renames I_LODS_M;

   procedure I_LODSB;
   pragma Import (Intrinsic, I_LODSB, Link_Name => "I_LODSB");
   procedure LODSB renames I_LODSB;

   procedure I_LODSD;
   pragma Import (Intrinsic, I_LODSD, Link_Name => "I_LODSD");
   procedure LODSD renames I_LODSD;

   procedure I_LODSW;
   pragma Import (Intrinsic, I_LODSW, Link_Name => "I_LODSW");
   procedure LODSW renames I_LODSW;

   procedure I_LOOP_L (TARGET : LBL);
   pragma Import (Intrinsic, I_LOOP_L, Link_Name => "I_LOOP_L");
   procedure I_LOOP (TARGET : LBL) renames I_LOOP_L;

   procedure I_LOOPE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_LOOPE_L, Link_Name => "I_LOOPE_L");
   procedure LOOPE (TARGET : LBL) renames I_LOOPE_L;

   procedure I_LOOPNE_L (TARGET : LBL);
   pragma Import (Intrinsic, I_LOOPNE_L, Link_Name => "I_LOOPNE_L");
   procedure LOOPNE (TARGET : LBL) renames I_LOOPNE_L;

   procedure LOOPNZ (TARGET : LBL) renames LOOPNE;

   procedure LOOPZ (TARGET : LBL) renames LOOPE;

   procedure I_LSL_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_LSL_RM, Link_Name => "I_LSL_RM");
   procedure LSL (DEST : REG; SRC : MEM) renames I_LSL_RM;

   procedure I_LSS_RM (DEST : REG32; SRC : MEM);
   pragma Import (Intrinsic, I_LSS_RM, Link_Name => "I_LSS_RM");
   procedure LSS (DEST : REG32; SRC : MEM) renames I_LSS_RM;

   procedure I_LTR_M (SRC : MEM);
   pragma Import (Intrinsic, I_LTR_M, Link_Name => "I_LTR_M");
   procedure LTR (SRC : MEM) renames I_LTR_M;
   procedure I_LTR_R (SRC : REG);
   pragma Import (Intrinsic, I_LTR_R, Link_Name => "I_LTR_R");
   procedure LTR (SRC : REG) renames I_LTR_R;

   procedure I_MOV_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_MOV_RR, Link_Name => "I_MOV_RR");
   procedure MOV (DEST : REG; SRC : REG) renames I_MOV_RR;
   procedure I_MOV_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_MOV_RM, Link_Name => "I_MOV_RM");
   procedure MOV (DEST : REG; SRC : MEM) renames I_MOV_RM;
   procedure I_MOV_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_MOV_MR, Link_Name => "I_MOV_MR");
   procedure MOV (DEST : MEM; SRC : REG) renames I_MOV_MR;
   procedure I_MOV_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_MOV_MI, Link_Name => "I_MOV_MI");
   procedure MOV (DEST : MEM; SRC : IMM) renames I_MOV_MI;
   procedure I_MOV_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_MOV_RI, Link_Name => "I_MOV_RI");
   procedure MOV (DEST : REG; SRC : IMM) renames I_MOV_RI;
   procedure I_MOV_RO (DEST : REG; SRC : OFS);
   pragma Import (Intrinsic, I_MOV_RO, Link_Name => "I_MOV_RO");
   procedure MOV (DEST : REG; SRC : OFS) renames I_MOV_RO;
   procedure I_MOV_RL (DEST : REG; SRC : LBL);
   pragma Import (Intrinsic, I_MOV_RL, Link_Name => "I_MOV_RL");
   procedure MOV (DEST : REG; SRC : LBL) renames I_MOV_RL;
   procedure I_MOV_SR (DEST : SREG; SRC : REG16);
   pragma Import (Intrinsic, I_MOV_SR, Link_Name => "I_MOV_SR");
   procedure MOV (DEST : SREG; SRC : REG16) renames I_MOV_SR;
   procedure I_MOV_SM (DEST : SREG; SRC : MEM);
   pragma Import (Intrinsic, I_MOV_SM, Link_Name => "I_MOV_SM");
   procedure MOV (DEST : SREG; SRC : MEM) renames I_MOV_SM;
   procedure I_MOV_RS (DEST : REG16; SRC : SREG);
   pragma Import (Intrinsic, I_MOV_RS, Link_Name => "I_MOV_RS");
   procedure MOV (DEST : REG16; SRC : SREG) renames I_MOV_RS;
   procedure I_MOV_MS (DEST : MEM; SRC : SREG);
   pragma Import (Intrinsic, I_MOV_MS, Link_Name => "I_MOV_MS");
   procedure MOV (DEST : MEM; SRC : SREG) renames I_MOV_MS;
   procedure I_MOV_RC (DEST : REG32; SRC : CREG);
   pragma Import (Intrinsic, I_MOV_RC, Link_Name => "I_MOV_RC");
   procedure MOV (DEST : REG32; SRC : CREG) renames I_MOV_RC;
   procedure I_MOV_RD (DEST : REG32; SRC : DREG);
   pragma Import (Intrinsic, I_MOV_RD, Link_Name => "I_MOV_RD");
   procedure MOV (DEST : REG32; SRC : DREG) renames I_MOV_RD;
   procedure I_MOV_RT (DEST : REG32; SRC : TREG);
   pragma Import (Intrinsic, I_MOV_RT, Link_Name => "I_MOV_RT");
   procedure MOV (DEST : REG32; SRC : TREG) renames I_MOV_RT;
   procedure I_MOV_CR (DEST : CREG; SRC : REG32);
   pragma Import (Intrinsic, I_MOV_CR, Link_Name => "I_MOV_CR");
   procedure MOV (DEST : CREG; SRC : REG32) renames I_MOV_CR;
   procedure I_MOV_DR (DEST : DREG; SRC : REG32);
   pragma Import (Intrinsic, I_MOV_DR, Link_Name => "I_MOV_DR");
   procedure MOV (DEST : DREG; SRC : REG32) renames I_MOV_DR;
   procedure I_MOV_TR (DEST : TREG; SRC : REG32);
   pragma Import (Intrinsic, I_MOV_TR, Link_Name => "I_MOV_TR");
   procedure MOV (DEST : TREG; SRC : REG32) renames I_MOV_TR;

   procedure I_MOVS_MM (DEST : MEM; SRC : MEM);
   pragma Import (Intrinsic, I_MOVS_MM, Link_Name => "I_MOVS_MM");
   procedure MOVS (DEST : MEM; SRC : MEM) renames I_MOVS_MM;

   procedure I_MOVSB;
   pragma Import (Intrinsic, I_MOVSB, Link_Name => "I_MOVSB");
   procedure MOVSB renames I_MOVSB;

   procedure I_MOVSD;
   pragma Import (Intrinsic, I_MOVSD, Link_Name => "I_MOVSD");
   procedure MOVSD renames I_MOVSD;

   procedure I_MOVSW;
   pragma Import (Intrinsic, I_MOVSW, Link_Name => "I_MOVSW");
   procedure MOVSW renames I_MOVSW;

   procedure I_MOVSX_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_MOVSX_RR, Link_Name => "I_MOVSX_RR");
   procedure MOVSX (DEST : REG; SRC : REG) renames I_MOVSX_RR;
   procedure I_MOVSX_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_MOVSX_RM, Link_Name => "I_MOVSX_RM");
   procedure MOVSX (DEST : REG; SRC : MEM) renames I_MOVSX_RM;

   procedure I_MOVZX_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_MOVZX_RR, Link_Name => "I_MOVZX_RR");
   procedure MOVZX (DEST : REG; SRC : REG) renames I_MOVZX_RR;
   procedure I_MOVZX_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_MOVZX_RM, Link_Name => "I_MOVZX_RM");
   procedure MOVZX (DEST : REG; SRC : MEM) renames I_MOVZX_RM;

   procedure I_MUL_R (SRC : REG);
   pragma Import (Intrinsic, I_MUL_R, Link_Name => "I_MUL_R");
   procedure MUL (SRC : REG) renames I_MUL_R;
   procedure I_MUL_M (SRC : MEM);
   pragma Import (Intrinsic, I_MUL_M, Link_Name => "I_MUL_M");
   procedure MUL (SRC : MEM) renames I_MUL_M;

   procedure I_NEG_R (DEST : REG);
   pragma Import (Intrinsic, I_NEG_R, Link_Name => "I_NEG_R");
   procedure NEG (DEST : REG) renames I_NEG_R;
   procedure I_NEG_M (DEST : MEM);
   pragma Import (Intrinsic, I_NEG_M, Link_Name => "I_NEG_M");
   procedure NEG (DEST : MEM) renames I_NEG_M;

   procedure I_NOP;
   pragma Import (Intrinsic, I_NOP, Link_Name => "I_NOP");
   procedure NOP renames I_NOP;

   procedure I_NOT_R (DEST : REG);
   pragma Import (Intrinsic, I_NOT_R, Link_Name => "I_NOT_R");
   procedure I_NOT (DEST : REG) renames I_NOT_R;
   procedure I_NOT_M (DEST : MEM);
   pragma Import (Intrinsic, I_NOT_M, Link_Name => "I_NOT_M");
   procedure I_NOT (DEST : MEM) renames I_NOT_M;

   procedure I_OR_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_OR_RR, Link_Name => "I_OR_RR");
   procedure I_OR (DEST : REG; SRC : REG) renames I_OR_RR;
   procedure I_OR_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_OR_MR, Link_Name => "I_OR_MR");
   procedure I_OR (DEST : MEM; SRC : REG) renames I_OR_MR;
   procedure I_OR_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_OR_RM, Link_Name => "I_OR_RM");
   procedure I_OR (DEST : REG; SRC : MEM) renames I_OR_RM;
   procedure I_OR_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_OR_RI, Link_Name => "I_OR_RI");
   procedure I_OR (DEST : REG; SRC : IMM) renames I_OR_RI;
   procedure I_OR_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_OR_MI, Link_Name => "I_OR_MI");
   procedure I_OR (DEST : MEM; SRC : IMM) renames I_OR_MI;

   procedure I_OUT_IR (PORT : IMM8; SRC : REG);
   pragma Import (Intrinsic, I_OUT_IR, Link_Name => "I_OUT_IR");
   procedure I_OUT (PORT : IMM8; SRC : REG) renames I_OUT_IR;
   procedure I_OUT_RR (PORT : REG_DX; SRC : REG);
   pragma Import (Intrinsic, I_OUT_RR, Link_Name => "I_OUT_RR");
   procedure I_OUT (PORT : REG_DX; SRC : REG) renames I_OUT_RR;

   procedure I_OUTS_RM (PORT : REG_DX; SRC : MEM);
   pragma Import (Intrinsic, I_OUTS_RM, Link_Name => "I_OUTS_RM");
   procedure OUTS (PORT : REG_DX; SRC : MEM) renames I_OUTS_RM;

   procedure I_OUTSB;
   pragma Import (Intrinsic, I_OUTSB, Link_Name => "I_OUTSB");
   procedure OUTSB renames I_OUTSB;

   procedure I_OUTSD;
   pragma Import (Intrinsic, I_OUTSD, Link_Name => "I_OUTSD");
   procedure OUTSD renames I_OUTSD;

   procedure I_OUTSW;
   pragma Import (Intrinsic, I_OUTSW, Link_Name => "I_OUTSW");
   procedure OUTSW renames I_OUTSW;

   procedure I_POP_R (SRC : REG);
   pragma Import (Intrinsic, I_POP_R, Link_Name => "I_POP_R");
   procedure POP (SRC : REG) renames I_POP_R;
   procedure I_POP_M (SRC : MEM);
   pragma Import (Intrinsic, I_POP_M, Link_Name => "I_POP_M");
   procedure POP (SRC : MEM) renames I_POP_M;
   procedure I_POP_S (SRC : SREG);
   pragma Import (Intrinsic, I_POP_S, Link_Name => "I_POP_S");
   procedure POP (SRC : SREG) renames I_POP_S;

   procedure I_POPA;
   pragma Import (Intrinsic, I_POPA, Link_Name => "I_POPA");
   procedure POPA renames I_POPA;

   procedure I_POPAD;
   pragma Import (Intrinsic, I_POPAD, Link_Name => "I_POPAD");
   procedure POPAD renames I_POPAD;

   procedure I_POPF;
   pragma Import (Intrinsic, I_POPF, Link_Name => "I_POPF");
   procedure POPF renames I_POPF;

   procedure I_POPFD;
   pragma Import (Intrinsic, I_POPFD, Link_Name => "I_POPFD");
   procedure POPFD renames I_POPFD;

   procedure I_PUSH_R (SRC : REG);
   pragma Import (Intrinsic, I_PUSH_R, Link_Name => "I_PUSH_R");
   procedure PUSH (SRC : REG) renames I_PUSH_R;
   procedure I_PUSH_M (SRC : MEM);
   pragma Import (Intrinsic, I_PUSH_M, Link_Name => "I_PUSH_M");
   procedure PUSH (SRC : MEM) renames I_PUSH_M;
   procedure I_PUSH_S (SRC : SREG);
   pragma Import (Intrinsic, I_PUSH_S, Link_Name => "I_PUSH_S");
   procedure PUSH (SRC : SREG) renames I_PUSH_S;
   procedure I_PUSH_I (SRC : IMM);
   pragma Import (Intrinsic, I_PUSH_I, Link_Name => "I_PUSH_I");
   procedure PUSH (SRC : IMM) renames I_PUSH_I;

   procedure I_PUSHA;
   pragma Import (Intrinsic, I_PUSHA, Link_Name => "I_PUSHA");
   procedure PUSHA renames I_PUSHA;

   procedure I_PUSHAD;
   pragma Import (Intrinsic, I_PUSHAD, Link_Name => "I_PUSHAD");
   procedure PUSHAD renames I_PUSHAD;

   procedure I_PUSHF;
   pragma Import (Intrinsic, I_PUSHF, Link_Name => "I_PUSHF");
   procedure PUSHF renames I_PUSHF;

   procedure I_PUSHFD;
   pragma Import (Intrinsic, I_PUSHFD, Link_Name => "I_PUSHFD");
   procedure PUSHFD renames I_PUSHFD;

   procedure I_RCL_RI (DEST : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_RCL_RI, Link_Name => "I_RCL_RI");
   procedure RCL (DEST : REG; COUNT : CTR) renames I_RCL_RI;
   procedure I_RCL_RR (DEST : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_RCL_RR, Link_Name => "I_RCL_RR");
   procedure RCL (DEST : REG; COUNT : REG_CL) renames I_RCL_RR;
   procedure I_RCL_MI (DEST : MEM; COUNT : CTR);
   pragma Import (Intrinsic, I_RCL_MI, Link_Name => "I_RCL_MI");
   procedure RCL (DEST : MEM; COUNT : CTR) renames I_RCL_MI;
   procedure I_RCL_MR (DEST : MEM; COUNT : REG_CL);
   pragma Import (Intrinsic, I_RCL_MR, Link_Name => "I_RCL_MR");
   procedure RCL (DEST : MEM; COUNT : REG_CL) renames I_RCL_MR;

   procedure I_RCR_RI (DEST : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_RCR_RI, Link_Name => "I_RCR_RI");
   procedure RCR (DEST : REG; COUNT : CTR) renames I_RCR_RI;
   procedure I_RCR_RR (DEST : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_RCR_RR, Link_Name => "I_RCR_RR");
   procedure RCR (DEST : REG; COUNT : REG_CL) renames I_RCR_RR;
   procedure I_RCR_MI (DEST : MEM; COUNT : CTR);
   pragma Import (Intrinsic, I_RCR_MI, Link_Name => "I_RCR_MI");
   procedure RCR (DEST : MEM; COUNT : CTR) renames I_RCR_MI;
   procedure I_RCR_MR (DEST : MEM; COUNT : REG_CL);
   pragma Import (Intrinsic, I_RCR_MR, Link_Name => "I_RCR_MR");
   procedure RCR (DEST : MEM; COUNT : REG_CL) renames I_RCR_MR;

   procedure I_REP;
   pragma Import (Intrinsic, I_REP, Link_Name => "I_REP");
   procedure REP renames I_REP;

   procedure REPE renames REP;

   procedure I_REPNE;
   pragma Import (Intrinsic, I_REPNE, Link_Name => "I_REPNE");
   procedure REPNE renames I_REPNE;

   procedure REPNZ renames REPNE;

   procedure I_RET;
   pragma Import (Intrinsic, I_RET, link_name => "I_RET");
   procedure RET renames I_RET;
   procedure I_RET_I (ESP_ADJUST : IMM16);
   pragma Import (Intrinsic, I_RET_I, Link_Name => "I_RET_I");
   procedure RET (ESP_ADJUST : IMM16) renames I_RET_I;

   procedure I_RETF;
   pragma Import (Intrinsic, I_RETF, Link_Name => "I_RETF");
   procedure RETF renames I_RETF;
   procedure I_RETF_I (ESP_ADJUST : IMM16);
   pragma Import (Intrinsic, I_RETF_I, Link_Name => "I_RETF_I");
   procedure RETF (ESP_ADJUST : IMM16) renames I_RETF_I;

   procedure I_ROL_RI (DEST : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_ROL_RI, Link_Name => "I_ROL_RI");
   procedure ROL (DEST : REG; COUNT : CTR) renames I_ROL_RI;
   procedure I_ROL_RR (DEST : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_ROL_RR, Link_Name => "I_ROL_RR");
   procedure ROL (DEST : REG; COUNT : REG_CL) renames I_ROL_RR;
   procedure I_ROL_MI (DEST : MEM; COUNT : CTR);
   pragma Import (Intrinsic, I_ROL_MI, Link_Name => "I_ROL_MI");
   procedure ROL (DEST : MEM; COUNT : CTR) renames I_ROL_MI;
   procedure I_ROL_MR (DEST : MEM; COUNT : REG_CL);
   pragma Import (Intrinsic, I_ROL_MR, Link_Name => "I_ROL_MR");
   procedure ROL (DEST : MEM; COUNT : REG_CL) renames I_ROL_MR;

   procedure I_ROR_RI (DEST : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_ROR_RI, Link_Name => "I_ROR_RI");
   procedure ROR (DEST : REG; COUNT : CTR) renames I_ROR_RI;
   procedure I_ROR_RR (DEST : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_ROR_RR, Link_Name => "I_ROR_RR");
   procedure ROR (DEST : REG; COUNT : REG_CL) renames I_ROR_RR;
   procedure I_ROR_MI (DEST : MEM; COUNT : CTR);
   pragma Import (Intrinsic, I_ROR_MI, Link_Name => "I_ROR_MI");
   procedure ROR (DEST : MEM; COUNT : CTR) renames I_ROR_MI;
   procedure I_ROR_MR (DEST : MEM; COUNT : REG_CL);
   pragma Import (Intrinsic, I_ROR_MR, Link_Name => "I_ROR_MR");
   procedure ROR (DEST : MEM; COUNT : REG_CL) renames I_ROR_MR;

   procedure I_SAHF;
   pragma Import (Intrinsic, I_SAHF, Link_Name => "I_SAHF");
   procedure SAHF renames I_SAHF;

   procedure I_SAL_RI (DEST : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_SAL_RI, Link_Name => "I_SAL_RI");
   procedure SAL (DEST : REG; COUNT : CTR) renames I_SAL_RI;
   procedure I_SAL_RR (DEST : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SAL_RR, Link_Name => "I_SAL_RR");
   procedure SAL (DEST : REG; COUNT : REG_CL) renames I_SAL_RR;
   procedure I_SAL_MI (DEST : MEM; COUNT : CTR);
   pragma Import (Intrinsic, I_SAL_MI, Link_Name => "I_SAL_MI");
   procedure SAL (DEST : MEM; COUNT : CTR) renames I_SAL_MI;
   procedure I_SAL_MR (DEST : MEM; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SAL_MR, Link_Name => "I_SAL_MR");
   procedure SAL (DEST : MEM; COUNT : REG_CL) renames I_SAL_MR;

   procedure I_SAR_RI (DEST : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_SAR_RI, Link_Name => "I_SAR_RI");
   procedure SAR (DEST : REG; COUNT : CTR) renames I_SAR_RI;
   procedure I_SAR_RR (DEST : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SAR_RR, Link_Name => "I_SAR_RR");
   procedure SAR (DEST : REG; COUNT : REG_CL) renames I_SAR_RR;
   procedure I_SAR_MI (DEST : MEM; COUNT : CTR);
   pragma Import (Intrinsic, I_SAR_MI, Link_Name => "I_SAR_MI");
   procedure SAR (DEST : MEM; COUNT : CTR) renames I_SAR_MI;
   procedure I_SAR_MR (DEST : MEM; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SAR_MR, Link_Name => "I_SAR_MR");
   procedure SAR (DEST : MEM; COUNT : REG_CL) renames I_SAR_MR;

   procedure I_SBB_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_SBB_RR, Link_Name => "I_SBB_RR");
   procedure SBB (DEST : REG; SRC : REG) renames I_SBB_RR;
   procedure I_SBB_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_SBB_MR, Link_Name => "I_SBB_MR");
   procedure SBB (DEST : MEM; SRC : REG) renames I_SBB_MR;
   procedure I_SBB_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_SBB_RM, Link_Name => "I_SBB_RM");
   procedure SBB (DEST : REG; SRC : MEM) renames I_SBB_RM;
   procedure I_SBB_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_SBB_RI, Link_Name => "I_SBB_RI");
   procedure SBB (DEST : REG; SRC : IMM) renames I_SBB_RI;
   procedure I_SBB_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_SBB_MI, Link_Name => "I_SBB_MI");
   procedure SBB (DEST : MEM; SRC : IMM) renames I_SBB_MI;

   procedure I_SCAS_M (DEST : MEM);
   pragma Import (Intrinsic, I_SCAS_M, Link_Name => "I_SCAS_M");
   procedure SCAS (DEST : MEM) renames I_SCAS_M;

   procedure I_SCASB;
   pragma Import (Intrinsic, I_SCASB, Link_Name => "I_SCASB");
   procedure SCASB renames I_SCASB;

   procedure I_SCASD;
   pragma Import (Intrinsic, I_SCASD, Link_Name => "I_SCASD");
   procedure SCASD renames I_SCASD;

   procedure I_SCASW;
   pragma Import (Intrinsic, I_SCASW, Link_Name => "I_SCASW");
   procedure SCASW renames I_SCASW;

   procedure I_SETA_R (DEST : REG);
   pragma Import (Intrinsic, I_SETA_R, Link_Name => "I_SETA_R");
   procedure SETA (DEST : REG) renames I_SETA_R;
   procedure I_SETA_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETA_M, Link_Name => "I_SETA_M");
   procedure SETA (DEST : MEM) renames I_SETA_M;

   procedure I_SETAE_R (DEST : REG);
   pragma Import (Intrinsic, I_SETAE_R, Link_Name => "I_SETAE_R");
   procedure SETAE (DEST : REG) renames I_SETAE_R;
   procedure I_SETAE_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETAE_M, Link_Name => "I_SETAE_M");
   procedure SETAE (DEST : MEM) renames I_SETAE_M;

   procedure I_SETB_R (DEST : REG);
   pragma Import (Intrinsic, I_SETB_R, Link_Name => "I_SETB_R");
   procedure SETB (DEST : REG) renames I_SETB_R;
   procedure I_SETB_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETB_M, Link_Name => "I_SETB_M");
   procedure SETB (DEST : MEM) renames I_SETB_M;

   procedure I_SETBE_R (DEST : REG);
   pragma Import (Intrinsic, I_SETBE_R, Link_Name => "I_SETBE_R");
   procedure SETBE (DEST : REG) renames I_SETBE_R;
   procedure I_SETBE_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETBE_M, Link_Name => "I_SETBE_M");
   procedure SETBE (DEST : MEM) renames I_SETBE_M;

   procedure SETC (DEST : REG) renames SETB;
   procedure SETC (DEST : MEM) renames SETB;

   procedure I_SETE_R (DEST : REG);
   pragma Import (Intrinsic, I_SETE_R, Link_Name => "I_SETE_R");
   procedure SETE (DEST : REG) renames I_SETE_R;
   procedure I_SETE_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETE_M, Link_Name => "I_SETE_M");
   procedure SETE (DEST : MEM) renames I_SETE_M;

   procedure I_SETG_R (DEST : REG);
   pragma Import (Intrinsic, I_SETG_R, Link_Name => "I_SETG_R");
   procedure SETG (DEST : REG) renames I_SETG_R;
   procedure I_SETG_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETG_M, Link_Name => "I_SETG_M");
   procedure SETG (DEST : MEM) renames I_SETG_M;

   procedure I_SETGE_R (DEST : REG);
   pragma Import (Intrinsic, I_SETGE_R, Link_Name => "I_SETGE_R");
   procedure SETGE (DEST : REG) renames I_SETGE_R;
   procedure I_SETGE_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETGE_M, Link_Name => "I_SETGE_M");
   procedure SETGE (DEST : MEM) renames I_SETGE_M;

   procedure I_SETL_R (DEST : REG);
   pragma Import (Intrinsic, I_SETL_R, Link_Name => "I_SETL_R");
   procedure SETL (DEST : REG) renames I_SETL_R;
   procedure I_SETL_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETL_M, Link_Name => "I_SETL_M");
   procedure SETL (DEST : MEM) renames I_SETL_M;

   procedure I_SETLE_R (DEST : REG);
   pragma Import (Intrinsic, I_SETLE_R, Link_Name => "I_SETLE_R");
   procedure SETLE (DEST : REG) renames I_SETLE_R;
   procedure I_SETLE_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETLE_M, Link_Name => "I_SETLE_M");
   procedure SETLE (DEST : MEM) renames I_SETLE_M;

   procedure SETNA (DEST : REG) renames SETBE;
   procedure SETNA (DEST : MEM) renames SETBE;

   procedure SETNAE (DEST : REG) renames SETB;
   procedure SETNAE (DEST : MEM) renames SETB;

   procedure SETNB (DEST : REG) renames SETAE;
   procedure SETNB (DEST : MEM) renames SETAE;

   procedure SETNBE (DEST : REG) renames SETA;
   procedure SETNBE (DEST : MEM) renames SETA;

   procedure SETNC (DEST : REG) renames SETAE;
   procedure SETNC (DEST : MEM) renames SETAE;

   procedure I_SETNE_R (DEST : REG);
   pragma Import (Intrinsic, I_SETNE_R, Link_Name => "I_SETNE_R");
   procedure SETNE (DEST : REG) renames I_SETNE_R;
   procedure I_SETNE_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETNE_M, Link_Name => "I_SETNE_M");
   procedure SETNE (DEST : MEM) renames I_SETNE_M;

   procedure SETNG (DEST : REG) renames SETLE;
   procedure SETNG (DEST : MEM) renames SETLE;

   procedure SETNGE (DEST : REG) renames SETL;
   procedure SETNGE (DEST : MEM) renames SETL;

   procedure SETNL (DEST : REG) renames SETGE;
   procedure SETNL (DEST : MEM) renames SETGE;

   procedure SETNLE (DEST : REG) renames SETG;
   procedure SETNLE (DEST : MEM) renames SETG;

   procedure I_SETNO_R (DEST : REG);
   pragma Import (Intrinsic, I_SETNO_R, Link_Name => "I_SETNO_R");
   procedure SETNO (DEST : REG) renames I_SETNO_R;
   procedure I_SETNO_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETNO_M, Link_Name => "I_SETNO_M");
   procedure SETNO (DEST : MEM) renames I_SETNO_M;

   procedure I_SETNP_R (DEST : REG);
   pragma Import (Intrinsic, I_SETNP_R, Link_Name => "I_SETNP_R");
   procedure SETNP (DEST : REG) renames I_SETNP_R;
   procedure I_SETNP_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETNP_M, Link_Name => "I_SETNP_M");
   procedure SETNP (DEST : MEM) renames I_SETNP_M;

   procedure I_SETNS_R (DEST : REG);
   pragma Import (Intrinsic, I_SETNS_R, Link_Name => "I_SETNS_R");
   procedure SETNS (DEST : REG) renames I_SETNS_R;
   procedure I_SETNS_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETNS_M, Link_Name => "I_SETNS_M");
   procedure SETNS (DEST : MEM) renames I_SETNS_M;

   procedure SETNZ (DEST : REG) renames SETNE;
   procedure SETNZ (DEST : MEM) renames SETNE;

   procedure I_SETO_R (DEST : REG);
   pragma Import (Intrinsic, I_SETO_R, Link_Name => "I_SETO_R");
   procedure SETO (DEST : REG) renames I_SETO_R;
   procedure I_SETO_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETO_M, Link_Name => "I_SETO_M");
   procedure SETO (DEST : MEM) renames I_SETO_M;

   procedure I_SETP_R (DEST : REG);
   pragma Import (Intrinsic, I_SETP_R, Link_Name => "I_SETP_R");
   procedure SETP (DEST : REG) renames I_SETP_R;
   procedure I_SETP_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETP_M, Link_Name => "I_SETP_M");
   procedure SETP (DEST : MEM) renames I_SETP_M;

   procedure SETPE (DEST : REG) renames SETP;
   procedure SETPE (DEST : MEM) renames SETP;

   procedure SETPO (DEST : REG) renames SETNP;
   procedure SETPO (DEST : MEM) renames SETNP;

   procedure I_SETS_R (DEST : REG);
   pragma Import (Intrinsic, I_SETS_R, Link_Name => "I_SETS_R");
   procedure SETS (DEST : REG) renames I_SETS_R;
   procedure I_SETS_M (DEST : MEM);
   pragma Import (Intrinsic, I_SETS_M, Link_Name => "I_SETS_M");
   procedure SETS (DEST : MEM) renames I_SETS_M;

   procedure SETZ (DEST : REG) renames SETE;
   procedure SETZ (DEST : MEM) renames SETE;

   procedure I_SGDT_M (DEST : MEM);
   pragma Import (Intrinsic, I_SGDT_M, Link_Name => "I_SGDT_M");
   procedure SGDT (DEST : MEM) renames I_SGDT_M;

   procedure SHL (DEST : REG; COUNT : CTR) renames SAL;
   procedure SHL (DEST : REG; COUNT : REG_CL) renames SAL;   -- note 5
   procedure SHL (DEST : MEM; COUNT : CTR) renames SAL;
   procedure SHL (DEST : MEM; COUNT : REG_CL) renames SAL;   -- note 5

   procedure I_SHLD_RRI (DEST : REG; SRC : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_SHLD_RRI, Link_Name => "I_SHLD_RRI");
   procedure SHLD (DEST : REG; SRC : REG; COUNT : CTR) renames I_SHLD_RRI;
   procedure I_SHLD_RRR (DEST : REG; SRC : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SHLD_RRR, Link_Name => "I_SHLD_RRR");
   procedure SHLD (DEST : REG; SRC : REG; COUNT : REG_CL) renames I_SHLD_RRR;
   procedure I_SHLD_MRI (DEST : MEM; SRC : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_SHLD_MRI, Link_Name => "I_SHLD_MRI");
   procedure SHLD (DEST : MEM; SRC : REG; COUNT : CTR) renames I_SHLD_MRI;
   procedure I_SHLD_MRR (DEST : MEM; SRC : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SHLD_MRR, Link_Name => "I_SHLD_MRR");
   procedure SHLD (DEST : MEM; SRC : REG; COUNT : REG_CL) renames I_SHLD_MRR;

   procedure I_SHR_RI (DEST : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_SHR_RI, Link_Name => "I_SHR_RI");
   procedure SHR (DEST : REG; COUNT : CTR) renames I_SHR_RI;
   procedure I_SHR_RR (DEST : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SHR_RR, Link_Name => "I_SHR_RR");
   procedure SHR (DEST : REG; COUNT : REG_CL) renames I_SHR_RR;
   procedure I_SHR_MI (DEST : MEM; COUNT : CTR);
   pragma Import (Intrinsic, I_SHR_MI, Link_Name => "I_SHR_MI");
   procedure SHR (DEST : MEM; COUNT : CTR) renames I_SHR_MI;
   procedure I_SHR_MR (DEST : MEM; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SHR_MR, Link_Name => "I_SHR_MR");
   procedure SHR (DEST : MEM; COUNT : REG_CL) renames I_SHR_MR;

   procedure I_SHRD_RRI (DEST : REG; SRC : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_SHRD_RRI, Link_Name => "I_SHRD_RRI");
   procedure SHRD (DEST : REG; SRC : REG; COUNT : CTR) renames I_SHRD_RRI;
   procedure I_SHRD_RRR (DEST : REG; SRC : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SHRD_RRR, Link_Name => "I_SHRD_RRR");
   procedure SHRD (DEST : REG; SRC : REG; COUNT : REG_CL) renames I_SHRD_RRR;
   procedure I_SHRD_MRI (DEST : MEM; SRC : REG; COUNT : CTR);
   pragma Import (Intrinsic, I_SHRD_MRI, Link_Name => "I_SHRD_MRI");
   procedure SHRD (DEST : MEM; SRC : REG; COUNT : CTR) renames I_SHRD_MRI;
   procedure I_SHRD_MRR (DEST : MEM; SRC : REG; COUNT : REG_CL);
   pragma Import (Intrinsic, I_SHRD_MRR, Link_Name => "I_SHRD_MRR");
   procedure SHRD (DEST : MEM; SRC : REG; COUNT : REG_CL) renames I_SHRD_MRR;

   procedure I_SIDT_M (DEST : MEM);
   pragma Import (Intrinsic, I_SIDT_M, Link_Name => "I_SIDT_M");
   procedure SIDT (DEST : MEM) renames I_SIDT_M;

   procedure I_SLDT_M (DEST : MEM);
   pragma Import (Intrinsic, I_SLDT_M, Link_Name => "I_SLDT_M");
   procedure SLDT (DEST : MEM) renames I_SLDT_M;
   procedure I_SLDT_R (DEST : REG);
   pragma Import (Intrinsic, I_SLDT_R, Link_Name => "I_SLDT_R");
   procedure SLDT (DEST : REG) renames I_SLDT_R;

   procedure I_SMSW_M (DEST : MEM);
   pragma Import (Intrinsic, I_SMSW_M, Link_Name => "I_SMSW_M");
   procedure SMSW (DEST : MEM) renames I_SMSW_M;
   procedure I_SMSW_R (DEST : REG);
   pragma Import (Intrinsic, I_SMSW_R, Link_Name => "I_SMSW_R");
   procedure SMSW (DEST : REG) renames I_SMSW_R;

   procedure I_STC;
   pragma Import (Intrinsic, I_STC, Link_Name => "I_STC");
   procedure STC renames I_STC;

   procedure I_STD;
   pragma Import (Intrinsic, I_STD, Link_Name => "I_STD");
   procedure STD renames I_STD;

   procedure I_STI;
   pragma Import (Intrinsic, I_STI, Link_Name => "I_STI");
   procedure STI renames I_STI;

   procedure I_STOS_M (DEST : MEM);
   pragma Import (Intrinsic, I_STOS_M, Link_Name => "I_STOS_M");
   procedure STOS (DEST : MEM) renames I_STOS_M;

   procedure I_STOSB;
   pragma Import (Intrinsic, I_STOSB, Link_Name => "I_STOSB");
   procedure STOSB renames I_STOSB;

   procedure I_STOSD;
   pragma Import (Intrinsic, I_STOSD, Link_Name => "I_STOSD");
   procedure STOSD renames I_STOSD;

   procedure I_STOSW;
   pragma Import (Intrinsic, I_STOSW, Link_Name => "I_STOSW");
   procedure STOSW renames I_STOSW;

   procedure I_STR_M (DEST : MEM);
   pragma Import (Intrinsic, I_STR_M, Link_Name => "I_STR_M");
   procedure STR (DEST : MEM) renames I_STR_M;
   procedure I_STR_R (DEST : REG);
   pragma Import (Intrinsic, I_STR_R, Link_Name => "I_STR_R");
   procedure STR (DEST : REG) renames I_STR_R;

   procedure I_SUB_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_SUB_RR, Link_Name => "I_SUB_RR");
   procedure SUB (DEST : REG; SRC : REG) renames I_SUB_RR;
   procedure I_SUB_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_SUB_MR, Link_Name => "I_SUB_MR");
   procedure SUB (DEST : MEM; SRC : REG) renames I_SUB_MR;
   procedure I_SUB_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_SUB_RM, Link_Name => "I_SUB_RM");
   procedure SUB (DEST : REG; SRC : MEM) renames I_SUB_RM;
   procedure I_SUB_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_SUB_RI, Link_Name => "I_SUB_RI");
   procedure SUB (DEST : REG; SRC : IMM) renames I_SUB_RI;
   procedure I_SUB_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_SUB_MI, Link_Name => "I_SUB_MI");
   procedure SUB (DEST : MEM; SRC : IMM) renames I_SUB_MI;

   procedure I_TEST_RR (SRC1 : REG; SRC2 : REG);
   pragma Import (Intrinsic, I_TEST_RR, Link_Name => "I_TEST_RR");
   procedure TEST (SRC1 : REG; SRC2 : REG) renames I_TEST_RR;
   procedure I_TEST_MR (SRC1 : MEM; SRC2 : REG);
   pragma Import (Intrinsic, I_TEST_MR, Link_Name => "I_TEST_MR");
   procedure TEST (SRC1 : MEM; SRC2 : REG) renames I_TEST_MR;
   procedure I_TEST_RM (SRC1 : REG; SRC2 : MEM);
   pragma Import (Intrinsic, I_TEST_RM, Link_Name => "I_TEST_RM");
   procedure TEST (SRC1 : REG; SRC2 : MEM) renames I_TEST_RM;
   procedure I_TEST_RI (SRC1 : REG; SRC2 : IMM);
   pragma Import (Intrinsic, I_TEST_RI, Link_Name => "I_TEST_RI");
   procedure TEST (SRC1 : REG; SRC2 : IMM) renames I_TEST_RI;
   procedure I_TEST_MI (SRC1 : MEM; SRC2 : IMM);
   pragma Import (Intrinsic, I_TEST_MI, Link_Name => "I_TEST_MI");
   procedure TEST (SRC1 : MEM; SRC2 : IMM) renames I_TEST_MI;

   procedure I_VERR_M (SRC : MEM);
   pragma Import (Intrinsic, I_VERR_M, Link_Name => "I_VERR_M");
   procedure VERR (SRC : MEM) renames I_VERR_M;
   procedure I_VERR_R (SRC : REG);
   pragma Import (Intrinsic, I_VERR_R, Link_Name => "I_VERR_R");
   procedure VERR (SRC : REG) renames I_VERR_R;

   procedure I_VERW_M (SRC : MEM);
   pragma Import (Intrinsic, I_VERW_M, Link_Name => "I_VERW_M");
   procedure VERW (SRC : MEM) renames I_VERW_M;
   procedure I_VERW_R (SRC : REG);
   pragma Import (Intrinsic, I_VERW_R, Link_Name => "I_VERW_R");
   procedure VERW (SRC : REG) renames I_VERW_R;

   procedure I_WAIT;
   pragma Import (Intrinsic, I_WAIT, Link_Name => "I_WAIT");
   procedure WAIT renames I_WAIT;

   procedure I_WBINVD;
   pragma Import (Intrinsic, I_WBINVD, Link_Name => "I_WBINVD");
   procedure WBINVD renames I_WBINVD;

   procedure I_XADD_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_XADD_RR, Link_Name => "I_XADD_RR");
   procedure XADD (DEST : REG; SRC : REG) renames I_XADD_RR;
   procedure I_XADD_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_XADD_MR, Link_Name => "I_XADD_MR");
   procedure XADD (DEST : MEM; SRC : REG) renames I_XADD_MR;

   procedure I_XCHG_MR (DEST1 : MEM; DEST2 : REG);
   pragma Import (Intrinsic, I_XCHG_MR, Link_Name => "I_XCHG_MR");
   procedure XCHG (DEST1 : MEM; DEST2 : REG) renames I_XCHG_MR;
   procedure I_XCHG_RR (DEST1 : REG; DEST2 : REG);
   pragma Import (Intrinsic, I_XCHG_RR, Link_Name => "I_XCHG_RR");
   procedure XCHG (DEST1 : REG; DEST2 : REG) renames I_XCHG_RR;
   procedure I_XCHG_RM (DEST1 : REG; DEST2 : MEM);
   pragma Import (Intrinsic, I_XCHG_RM, Link_Name => "I_XCHG_RM");
   procedure XCHG (DEST1 : REG; DEST2 : MEM) renames I_XCHG_RM;

   procedure I_XLAT_M (SRC : MEM);
   pragma Import (Intrinsic, I_XLAT_M, Link_Name => "I_XLAT_M");
   procedure XLAT (SRC : MEM) renames I_XLAT_M;

   procedure I_XLATB;
   pragma Import (Intrinsic, I_XLATB, Link_Name => "I_XLATB");
   procedure XLATB renames I_XLATB;

   procedure I_XOR_RR (DEST : REG; SRC : REG);
   pragma Import (Intrinsic, I_XOR_RR, Link_Name => "I_XOR_RR");
   procedure I_XOR (DEST : REG; SRC : REG) renames I_XOR_RR;
   procedure I_XOR_MR (DEST : MEM; SRC : REG);
   pragma Import (Intrinsic, I_XOR_MR, Link_Name => "I_XOR_MR");
   procedure I_XOR (DEST : MEM; SRC : REG) renames I_XOR_MR;
   procedure I_XOR_RM (DEST : REG; SRC : MEM);
   pragma Import (Intrinsic, I_XOR_RM, Link_Name => "I_XOR_RM");
   procedure I_XOR (DEST : REG; SRC : MEM) renames I_XOR_RM;
   procedure I_XOR_RI (DEST : REG; SRC : IMM);
   pragma Import (Intrinsic, I_XOR_RI, Link_Name => "I_XOR_RI");
   procedure I_XOR (DEST : REG; SRC : IMM) renames I_XOR_RI;
   procedure I_XOR_MI (DEST : MEM; SRC : IMM);
   pragma Import (Intrinsic, I_XOR_MI, Link_Name => "I_XOR_MI");
   procedure I_XOR (DEST : MEM; SRC : IMM) renames I_XOR_MI;

   -- Note 1: In CALL/JMP instructions with a memory operand, the transfer
   -- is NEAR if the memory type is 4 bytes, and FAR if it is 6 bytes.

   -- Note 2: The CALL/JMP forms with two immediate operands generate a direct
   -- FAR call or jump with the specified segment and offset values.

   -- Note 3: In these forms of string instructions, the operands serve only
   -- to specify the operand size and possible segment override. Since operand
   -- size is more conveniently specified using the specialized versions where
   -- the name implies the size, these forms are usually used only when a
   -- segment override (applicable only to the source operand) is required.

   -- Note 4: The only register which can be used to specify a port number in
   -- an I/O operation is DX.

   -- Note 5: The only register which can be used to specify a shift count
   -- in a shift or rotate instruction is CL.

   -- Note 6: RET generates a NEAR RET, and RETF generates a FAR RET (the
   -- distinction in the mnemonics is needed because, unlike the situation
   -- in assembly language, there is no concept of the current procedure mode).

   -- Note 7: In the XLAT instruction, the operand serves only to specify a
   -- possible segment override for the translation table. If no segment
   -- overlap is required, then XLATB is more convenient.

   -- Note 8: These instructions are available only on the 80486 processor.


   --	    ********************************
   --	    * 7. FLOATING-POINT OPERATIONS *
   --	    ********************************

   -- Each of the following operations generates a single floating-point
   -- instruction. Explicit FWAIT instructions must be coded where needed.

   procedure I_F2XM1;
   pragma Import (Intrinsic, I_F2XM1, Link_Name => "I_F2XM1");
   procedure F2XM1 renames I_F2XM1;

   procedure I_FABS;
   pragma Import (Intrinsic, I_FABS, Link_Name => "I_FABS");
   procedure FABS renames I_FABS;

   procedure I_FADD;
   pragma Import (Intrinsic, I_FADD, Link_Name => "I_FADD");
   procedure FADD renames I_FADD;
   procedure I_FADD_M (SRC : MEM);
   pragma Import (Intrinsic, I_FADD_M, Link_Name => "I_FADD_M");
   procedure FADD (SRC : MEM) renames I_FADD_M;
   procedure I_FADD_FF (DEST : FREG; SRC : FREG);
   pragma Import (Intrinsic, I_FADD_FF, Link_Name => "I_FADD_FF");
   procedure FADD (DEST : FREG; SRC : FREG) renames I_FADD_FF;

   procedure I_FADDP_FF (DEST : FREG; SRC : FREG_ST);
   pragma Import (Intrinsic, I_FADDP_FF, Link_Name => "I_FADDP_FF");
   procedure FADDP (DEST : FREG; SRC : FREG_ST) renames I_FADDP_FF;

   procedure I_FBLD_M (SRC : MEM);
   pragma Import (Intrinsic, I_FBLD_M, Link_Name => "I_FBLD_M");
   procedure FBLD (SRC : MEM) renames I_FBLD_M;

   procedure I_FBSTP_M (SRC : MEM);
   pragma Import (Intrinsic, I_FBSTP_M, Link_Name => "I_FBSTP_M");
   procedure FBSTP (SRC : MEM) renames I_FBSTP_M;

   procedure I_FCHS;
   pragma Import (Intrinsic, I_FCHS, Link_Name => "I_FCHS");
   procedure FCHS renames I_FCHS;

   procedure I_FCLEX;
   pragma Import (Intrinsic, I_FCLEX, Link_Name => "I_FCLEX");
   procedure FCLEX renames I_FCLEX;

   procedure I_FCOM;
   pragma Import (Intrinsic, I_FCOM, Link_Name => "I_FCOM");
   procedure FCOM renames I_FCOM;
   procedure I_FCOM_F (SRC : FREG);
   pragma Import (Intrinsic, I_FCOM_F, Link_Name => "I_FCOM_F");
   procedure FCOM (SRC : FREG) renames I_FCOM_F;

   procedure I_FCOMP;
   pragma Import (Intrinsic, I_FCOMP, Link_Name => "I_FCOMP");
   procedure FCOMP renames I_FCOMP;
   procedure I_FCOMP_F (SRC : FREG);
   pragma Import (Intrinsic, I_FCOMP_F, Link_Name => "I_FCOMP_F");
   procedure FCOMP (SRC : FREG) renames I_FCOMP_F;

   procedure I_FCOMPP;
   pragma Import (Intrinsic, I_FCOMPP, Link_Name => "I_FCOMPP");
   procedure FCOMPP renames I_FCOMPP;

   procedure I_FCOS;
   pragma Import (Intrinsic, I_FCOS, Link_Name => "I_FCOS");
   procedure FCOS renames I_FCOS;

   procedure I_FDECSTP;
   pragma Import (Intrinsic, I_FDECSTP, Link_Name => "I_FDECSTP");
   procedure FDECSTP renames I_FDECSTP;

   procedure I_FDISI;
   pragma Import (Intrinsic, I_FDISI, Link_Name => "I_FDISI");
   procedure FDISI renames I_FDISI;

   procedure I_FDIV;
   pragma Import (Intrinsic, I_FDIV, Link_Name => "I_FDIV");
   procedure FDIV renames I_FDIV;
   procedure I_FDIV_M (SRC : MEM);
   pragma Import (Intrinsic, I_FDIV_M, Link_Name => "I_FDIV_M");
   procedure FDIV (SRC : MEM) renames I_FDIV_M;
   procedure I_FDIV_FF (DEST : FREG; SRC : FREG);
   pragma Import (Intrinsic, I_FDIV_FF, Link_Name => "I_FDIV_FF");
   procedure FDIV (DEST : FREG; SRC : FREG) renames I_FDIV_FF;

   procedure I_FDIVP_FF (DEST : FREG; SRC : FREG_ST);
   pragma Import (Intrinsic, I_FDIVP_FF, Link_Name => "I_FDIVP_FF");
   procedure FDIVP (DEST : FREG; SRC : FREG_ST) renames I_FDIVP_FF;

   procedure I_FDIVR;
   pragma Import (Intrinsic, I_FDIVR, Link_Name => "I_FDIVR");
   procedure FDIVR renames I_FDIVR;
   procedure I_FDIVR_M (SRC : MEM);
   pragma Import (Intrinsic, I_FDIVR_M, Link_Name => "I_FDIVR_M");
   procedure FDIVR (SRC : MEM) renames I_FDIVR_M;
   procedure I_FDIVR_FF (DEST : FREG; SRC : FREG);
   pragma Import (Intrinsic, I_FDIVR_FF, Link_Name => "I_FDIVR_FF");
   procedure FDIVR (DEST : FREG; SRC : FREG) renames I_FDIVR_FF;

   procedure I_FDIVRP_FF (DEST : FREG; SRC : FREG_ST);
   pragma Import (Intrinsic, I_FDIVRP_FF, Link_Name => "I_FDIVRP_FF");
   procedure FDIVRP (DEST : FREG; SRC : FREG_ST) renames I_FDIVRP_FF;

   procedure I_FENI;
   pragma Import (Intrinsic, I_FENI, Link_Name => "I_FENI");
   procedure FENI renames I_FENI;

   procedure I_FFREE_F (DEST : FREG);
   pragma Import (Intrinsic, I_FFREE_F, Link_Name => "I_FFREE_F");
   procedure FFREE (DEST : FREG) renames I_FFREE_F;

   procedure I_FIADD_M (SRC : MEM);
   pragma Import (Intrinsic, I_FIADD_M, Link_Name => "I_FIADD_M");
   procedure FIADD (SRC : MEM) renames I_FIADD_M;

   procedure I_FICOM_M (SRC : MEM);
   pragma Import (Intrinsic, I_FICOM_M, Link_Name => "I_FICOM_M");
   procedure FICOM (SRC : MEM) renames I_FICOM_M;

   procedure I_FICOMP_M (SRC : MEM);
   pragma Import (Intrinsic, I_FICOMP_M, Link_Name => "I_FICOMP_M");
   procedure FICOMP (SRC : MEM) renames I_FICOMP_M;

   procedure I_FIDIV_M (SRC : MEM);
   pragma Import (Intrinsic, I_FIDIV_M, Link_Name => "I_FIDIV_M");
   procedure FIDIV (SRC : MEM) renames I_FIDIV_M;

   procedure I_FIDIVR_M (SRC : MEM);
   pragma Import (Intrinsic, I_FIDIVR_M, Link_Name => "I_FIDIVR_M");
   procedure FIDIVR (SRC : MEM) renames I_FIDIVR_M;

   procedure I_FILD_M (SRC : MEM);
   pragma Import (Intrinsic, I_FILD_M, Link_Name => "I_FILD_M");
   procedure FILD (SRC : MEM) renames I_FILD_M;

   procedure I_FIMUL_M (SRC : MEM);
   pragma Import (Intrinsic, I_FIMUL_M, Link_Name => "I_FIMUL_M");
   procedure FIMUL (SRC : MEM) renames I_FIMUL_M;

   procedure I_FINCSTP;
   pragma Import (Intrinsic, I_FINCSTP, Link_Name => "I_FINCSTP");
   procedure FINCSTP renames I_FINCSTP;

   procedure I_FINIT;
   pragma Import (Intrinsic, I_FINIT, Link_Name => "I_FINIT");
   procedure FINIT renames I_FINIT;

   procedure I_FIST_M (DEST : MEM);
   pragma Import (Intrinsic, I_FIST_M, Link_Name => "I_FIST_M");
   procedure FIST (DEST : MEM) renames I_FIST_M;

   procedure I_FISTP_M (DEST : MEM);
   pragma Import (Intrinsic, I_FISTP_M, Link_Name => "I_FISTP_M");
   procedure FISTP (DEST : MEM) renames I_FISTP_M;

   procedure I_FISUB_M (SRC : MEM);
   pragma Import (Intrinsic, I_FISUB_M, Link_Name => "I_FISUB_M");
   procedure FISUB (SRC : MEM) renames I_FISUB_M;

   procedure I_FISUBR_M (SRC : MEM);
   pragma Import (Intrinsic, I_FISUBR_M, Link_Name => "I_FISUBR_M");
   procedure FISUBR (SRC : MEM) renames I_FISUBR_M;

   procedure I_FLD_M (SRC : MEM);
   pragma Import (Intrinsic, I_FLD_M, Link_Name => "I_FLD_M");
   procedure FLD (SRC : MEM) renames I_FLD_M;
   procedure I_FLD_F (SRC : FREG);
   pragma Import (Intrinsic, I_FLD_F, Link_Name => "I_FLD_F");
   procedure FLD (SRC : FREG) renames I_FLD_F;

   procedure I_FLDCW_M (SRC : MEM);
   pragma Import (Intrinsic, I_FLDCW_M, Link_Name => "I_FLDCW_M");
   procedure FLDCW (SRC : MEM) renames I_FLDCW_M;

   procedure I_FLDENV_M (SRC : MEM);
   pragma Import (Intrinsic, I_FLDENV_M, Link_Name => "I_FLDENV_M");
   procedure FLDENV (SRC : MEM) renames I_FLDENV_M;

   procedure I_FLD1;
   pragma Import (Intrinsic, I_FLD1, Link_Name => "I_FLD1");
   procedure FLD1 renames I_FLD1;

   procedure I_FLDL2E;
   pragma Import (Intrinsic, I_FLDL2E, Link_Name => "I_FLDL2E");
   procedure FLDL2E renames I_FLDL2E;

   procedure I_FLDL2T;
   pragma Import (Intrinsic, I_FLDL2T, Link_Name => "I_FLDL2T");
   procedure FLDL2T renames I_FLDL2T;

   procedure I_FLDLG2;
   pragma Import (Intrinsic, I_FLDLG2, Link_Name => "I_FLDLG2");
   procedure FLDLG2 renames I_FLDLG2;

   procedure I_FLDLN2;
   pragma Import (Intrinsic, I_FLDLN2, Link_Name => "I_FLDLN2");
   procedure FLDLN2 renames I_FLDLN2;

   procedure I_FLDP1;
   pragma Import (Intrinsic, I_FLDP1, Link_Name => "I_FLDP1");
   procedure FLDP1 renames I_FLDP1;

   procedure I_FLDZ;
   pragma Import (Intrinsic, I_FLDZ, Link_Name => "I_FLDZ");
   procedure FLDZ renames I_FLDZ;

   procedure I_FMUL;
   pragma Import (Intrinsic, I_FMUL, Link_Name => "I_FMUL");
   procedure FMUL renames I_FMUL;
   procedure I_FMUL_M (SRC : MEM);
   pragma Import (Intrinsic, I_FMUL_M, Link_Name => "I_FMUL_M");
   procedure FMUL (SRC : MEM) renames I_FMUL_M;
   procedure I_FMUL_FF (DEST : FREG; SRC : FREG);
   pragma Import (Intrinsic, I_FMUL_FF, Link_Name => "I_FMUL_FF");
   procedure FMUL (DEST : FREG; SRC : FREG) renames I_FMUL_FF;

   procedure I_FMULP_FF (DEST : FREG; SRC : FREG_ST);
   pragma Import (Intrinsic, I_FMULP_FF, Link_Name => "I_FMULP_FF");
   procedure FMULP (DEST : FREG; SRC : FREG_ST) renames I_FMULP_FF;

   procedure I_FNCLEX;
   pragma Import (Intrinsic, I_FNCLEX, Link_Name => "I_FNCLEX");
   procedure FNCLEX renames I_FNCLEX;

   procedure I_FNDISI;
   pragma Import (Intrinsic, I_FNDISI, Link_Name => "I_FNDISI");
   procedure FNDISI renames I_FNDISI;

   procedure I_FNENI;
   pragma Import (Intrinsic, I_FNENI, Link_Name => "I_FNENI");
   procedure FNENI renames I_FNENI;

   procedure I_FNINIT;
   pragma Import (Intrinsic, I_FNINIT, Link_Name => "I_FNINIT");
   procedure FNINIT renames I_FNINIT;

   procedure I_FNOP;
   pragma Import (Intrinsic, I_FNOP, Link_Name => "I_FNOP");
   procedure FNOP renames I_FNOP;

   procedure I_FNSAVE_M (DEST : MEM);
   pragma Import (Intrinsic, I_FNSAVE_M, Link_Name => "I_FNSAVE_M");
   procedure FNSAVE (DEST : MEM) renames I_FNSAVE_M;

   procedure I_FNSTCW_M (DEST : MEM);
   pragma Import (Intrinsic, I_FNSTCW_M, Link_Name => "I_FNSTCW_M");
   procedure FNSTCW (DEST : MEM) renames I_FNSTCW_M;

   procedure I_FNSTENV_M (DEST : MEM);
   pragma Import (Intrinsic, I_FNSTENV_M, Link_Name => "I_FNSTENV_M");
   procedure FNSTENV (DEST : MEM) renames I_FNSTENV_M;

   procedure I_FNSTSW_M (DEST : MEM);
   pragma Import (Intrinsic, I_FNSTSW_M, Link_Name => "I_FNSTSW_M");
   procedure FNSTSW (DEST : MEM) renames I_FNSTSW_M;
   procedure I_FNSTSW_R (DEST : REG_AX);
   pragma Import (Intrinsic, I_FNSTSW_R, Link_Name => "I_FNSTSW_R");
   procedure FNSTSW (DEST : REG_AX) renames I_FNSTSW_R;

   procedure I_FPATAN;
   pragma Import (Intrinsic, I_FPATAN, Link_Name => "I_FPATAN");
   procedure FPATAN renames I_FPATAN;

   procedure I_FPREM;
   pragma Import (Intrinsic, I_FPREM, Link_Name => "I_FPREM");
   procedure FPREM renames I_FPREM;

   procedure I_FPREM1;
   pragma Import (Intrinsic, I_FPREM1, Link_Name => "I_FPREM1");
   procedure FPREM1 renames I_FPREM1;

   procedure I_FPTAN;
   pragma Import (Intrinsic, I_FPTAN, Link_Name => "I_FPTAN");
   procedure FPTAN renames I_FPTAN;

   procedure I_FRNDINT;
   pragma Import (Intrinsic, I_FRNDINT, Link_Name => "I_FRNDINT");
   procedure FRNDINT renames I_FRNDINT;

   procedure I_FRSTOR_M (SRC : MEM);
   pragma Import (Intrinsic, I_FRSTOR_M, Link_Name => "I_FRSTOR_M");
   procedure FRSTOR (SRC : MEM) renames I_FRSTOR_M;

   procedure I_FSAVE_M (DEST : MEM);
   pragma Import (Intrinsic, I_FSAVE_M, Link_Name => "I_FSAVE_M");
   procedure FSAVE (DEST : MEM) renames I_FSAVE_M;

   procedure I_FSCALE;
   pragma Import (Intrinsic, I_FSCALE, Link_Name => "I_FSCALE");
   procedure FSCALE renames I_FSCALE;

   procedure I_FSETPM;
   pragma Import (Intrinsic, I_FSETPM, Link_Name => "I_FSETPM");
   procedure FSETPM renames I_FSETPM;

   procedure I_FSIN;
   pragma Import (Intrinsic, I_FSIN, Link_Name => "I_FSIN");
   procedure FSIN renames I_FSIN;

   procedure I_FSINCOS;
   pragma Import (Intrinsic, I_FSINCOS, Link_Name => "I_FSINCOS");
   procedure FSINCOS renames I_FSINCOS;

   procedure I_FSQRT;
   pragma Import (Intrinsic, I_FSQRT, Link_Name => "I_FSQRT");
   procedure FSQRT renames I_FSQRT;

   procedure I_FST_F (DEST : FREG);
   pragma Import (Intrinsic, I_FST_F, Link_Name => "I_FST_F");
   procedure FST (DEST : FREG) renames I_FST_F;
   procedure I_FST_M (DEST : MEM);
   pragma Import (Intrinsic, I_FST_M, Link_Name => "I_FST_M");
   procedure FST (DEST : MEM) renames I_FST_M;

   procedure I_FSTCW_M (DEST : MEM);
   pragma Import (Intrinsic, I_FSTCW_M, Link_Name => "I_FSTCW_M");
   procedure FSTCW (DEST : MEM) renames I_FSTCW_M;

   procedure I_FSTENV_M (DEST : MEM);
   pragma Import (Intrinsic, I_FSTENV_M, Link_Name => "I_FSTENV_M");
   procedure FSTENV (DEST : MEM) renames I_FSTENV_M;

   procedure I_FSTP_M (DEST : MEM);
   pragma Import (Intrinsic, I_FSTP_M, Link_Name => "I_FSTP_M");
   procedure FSTP (DEST : MEM) renames I_FSTP_M;
   procedure I_FSTP_F (DEST : FREG);
   pragma Import (Intrinsic, I_FSTP_F, Link_Name => "I_FSTP_F");
   procedure FSTP (DEST : FREG) renames I_FSTP_F;

   procedure I_FSTSW_M (DEST : MEM);
   pragma Import (Intrinsic, I_FSTSW_M, Link_Name => "I_FSTSW_M");
   procedure FSTSW (DEST : MEM) renames I_FSTSW_M;
   procedure I_FSTSW_R (DEST : REG_AX);
   pragma Import (Intrinsic, I_FSTSW_R, Link_Name => "I_FSTSW_R");
   procedure FSTSW (DEST : REG_AX) renames I_FSTSW_R;

   procedure I_FSUB;
   pragma Import (Intrinsic, I_FSUB, Link_Name => "I_FSUB");
   procedure FSUB renames I_FSUB;
   procedure I_FSUB_M (SRC : MEM);
   pragma Import (Intrinsic, I_FSUB_M, Link_Name => "I_FSUB_M");
   procedure FSUB (SRC : MEM) renames I_FSUB_M;
   procedure I_FSUB_FF (DEST : FREG; SRC : FREG);
   pragma Import (Intrinsic, I_FSUB_FF, Link_Name => "I_FSUB_FF");
   procedure FSUB (DEST : FREG; SRC : FREG) renames I_FSUB_FF;

   procedure I_FSUBP_FF (DEST : FREG; SRC : FREG_ST);
   pragma Import (Intrinsic, I_FSUBP_FF, Link_Name => "I_FSUBP_FF");
   procedure FSUBP (DEST : FREG; SRC : FREG_ST) renames I_FSUBP_FF;

   procedure I_FSUBR;
   pragma Import (Intrinsic, I_FSUBR, Link_Name => "I_FSUBR");
   procedure FSUBR renames I_FSUBR;
   procedure I_FSUBR_M (SRC : MEM);
   pragma Import (Intrinsic, I_FSUBR_M, Link_Name => "I_FSUBR_M");
   procedure FSUBR (SRC : MEM) renames I_FSUBR_M;
   procedure I_FSUBR_FF (DEST : FREG; SRC : FREG);
   pragma Import (Intrinsic, I_FSUBR_FF, Link_Name => "I_FSUBR_FF");
   procedure FSUBR (DEST : FREG; SRC : FREG) renames I_FSUBR_FF;

   procedure I_FSUBRP_FF (DEST : FREG; SRC : FREG_ST);
   pragma Import (Intrinsic, I_FSUBRP_FF, Link_Name => "I_FSUBRP_FF");
   procedure FSUBRP (DEST : FREG; SRC : FREG_ST) renames I_FSUBRP_FF;

   procedure I_FTST;
   pragma Import (Intrinsic, I_FTST, Link_Name => "I_FTST");
   procedure FTST renames I_FTST;

   procedure I_FUCOM;
   pragma Import (Intrinsic, I_FUCOM, Link_Name => "I_FUCOM");
   procedure FUCOM renames I_FUCOM;
   procedure I_FUCOM_F (SRC : FREG);
   pragma Import (Intrinsic, I_FUCOM_F, Link_Name => "I_FUCOM_F");
   procedure FUCOM (SRC : FREG) renames I_FUCOM_F;

   procedure I_FUCOMP;
   pragma Import (Intrinsic, I_FUCOMP, Link_Name => "I_FUCOMP");
   procedure FUCOMP renames I_FUCOMP;
   procedure I_FUCOMP_F (SRC : FREG);
   pragma Import (Intrinsic, I_FUCOMP_F, Link_Name => "I_FUCOMP_F");
   procedure FUCOMP (SRC : FREG) renames I_FUCOMP_F;

   procedure I_FUCOMPP;
   pragma Import (Intrinsic, I_FUCOMPP, Link_Name => "I_FUCOMPP");
   procedure FUCOMPP renames I_FUCOMPP;

   procedure FWAIT renames WAIT;

   procedure I_FXAM;
   pragma Import (Intrinsic, I_FXAM, Link_Name => "I_FXAM");
   procedure FXAM renames I_FXAM;

   procedure I_FXCH;
   pragma Import (Intrinsic, I_FXCH, Link_Name => "I_FXCH");
   procedure FXCH renames I_FXCH;
   procedure I_FXCH_F (DEST : FREG);
   pragma Import (Intrinsic, I_FXCH_F, Link_Name => "I_FXCH_F");
   procedure FXCH (DEST : FREG) renames I_FXCH_F;

   procedure I_FXTRACT;
   pragma Import (Intrinsic, I_FXTRACT, Link_Name => "I_FXTRACT");
   procedure FXTRACT renames I_FXTRACT;

   procedure I_FYL2X;
   pragma Import (Intrinsic, I_FYL2X, Link_Name => "I_FYL2X");
   procedure FYL2X renames I_FYL2X;

   procedure I_FYL2P1;
   pragma Import (Intrinsic, I_FYL2P1, Link_Name => "I_FYL2P1");
   procedure FYL2P1 renames I_FYL2P1;

   -- Note 9: In floating point operations with two operands of type FREG,
   -- one of the two operands is required to be ST(0) (which can be written
   -- simply as ST, since ST(0) is the default).

   -- Note 10: The only permissible src register for these operations is ST.

   -- Note 11: These operations are only available on the 8087, and thus
   -- should not normally be used on an 80286 or 80386 processor.

   -- Note 12: The only permissible register for an FSTSW or FNSTSW
   -- operation is AX.


   --	    **********************************
   --	    * 8. INTERSPERSED ADA STATEMENTS *
   --	    **********************************

   -- Generally Ada statements should be assumed to destroy all registers and
   -- the flags. The exceptions are as follows:

   -- A go to statement generates a simple direct jump instruction, so that,
   -- for example, the two following statements are equivalent in effect:

   --	    goto LABEL1;
   --       JMP (LABEL1'ADDRESS);

   -- A call to a subprogram with no parameters generates a single direct
   -- NEAR call instruction.

   -- A loop/end loop combination generates a simple direct jump, so that,
   -- for example, the following two instructions are equivalent:

   --	    <<LOOP_LBL>>
   --	       ...
   --          JMP (LOOP_LBL'ADDRESS);

   --	    loop
   --	       ...
   --	    end loop;

   -- The latter form is considered preferable, since it maintains the
   -- structured format of the Ada source.

   -- An if statement generates a conditional jump and (if an else part is
   -- present) an unconditional jump, provided that the condition is a test
   -- of flag settings, as described in the next section. This means, for
   -- example, that the following two constructs are equivalent:

   --          JZ (ELSEPART'ADDRESS);
   --	       ...
   --          JMP (PASTIF'ADDRESS);
   --	    <<ELSEPART>>
   --	       ...
   --	    <<PASTIF>>

   --	    if F_NZ then
   --	       ...
   --	    else
   --	       ...
   --	    end if;

   -- The latter form is considered preferable, since it maintains the
   -- structured format of the Ada source.

   -- A while loop generates a conditional jump at the top of the loop and
   -- an unconditional jump at the bottom of the loop, provided that the
   -- condition is a test of flag settings, as described in the next section.
   -- This means, for example, that the following constructs are equivalent:

   --	    <<LOOP_LBL>>
   --          JNZ (LOOP_EXIT'ADDRESS);
   --	       ...
   --          JMP (LOOP_LBL'ADDRESS);
   --	    <<LOOP_EXIT>>
   --	       ...

   --	    while F_Z loop
   --	       ...
   --	    end loop;

   -- The latter form is considered preferable, since it maintains the
   -- structured format of the Ada source.

   -- An exit statement within a loop generates a conditional jump out of the
   -- loop provided that the condition is a test of flag settings, as described
   -- in the next section. This means, for example, that the following two
   -- constructs are equivalent:

   --	    <<LOOP_LBL>>
   --	       ...
   --          JNZ (LOOP_EXIT'ADDRESS);
   --	       ...
   --          JMP (LOOP_LBL'ADDRESS);
   --	    <<LOOP_EXIT>>
   --	       ...

   --	    loop
   --	       ...
   --	       exit when F_NZ;
   --	       ...
   --	    end loop;

   -- The latter form is considered preferable, since it maintains the
   -- structured format of the Ada source.

   -- Note: any other Ada operations, other than calls to instrinsic routines
   -- defined in this package, or as described in one of the above specialized
   -- cases, should be considered to destroy all registers and all flags.

   -- Note: with the exception of specialized loops constructed using LOOP,
   -- JCXZ, JCEXZ, LOOPE, LOOPZ, LOOPNE, LOOPNZ, virtually all control forms
   -- can, and should, be coded using their Ada forms. This will result in
   -- much more readable code.


   --	    ***************************
   --	    * 9. FLAG TEST OPERATIONS *
   --	    ***************************

   -- Functions returning Boolean indicating state of flags. These are
   -- exclusively intended for use in "if", "while" or "exit" tests, which
   -- compile to a single appropriate jump on condition instruction, as
   -- described in the previous section.

   function F_A   return BOOLEAN;
   function F_AE  return BOOLEAN;
   function F_B   return BOOLEAN;
   function F_BE  return BOOLEAN;
   function F_C   return BOOLEAN renames F_B;
   function F_E   return BOOLEAN;
   function F_G   return BOOLEAN;
   function F_GE  return BOOLEAN;
   function F_L   return BOOLEAN;
   function F_LE  return BOOLEAN;
   function F_NA  return BOOLEAN renames F_BE;
   function F_NAE return BOOLEAN renames F_B;
   function F_NB  return BOOLEAN renames F_AE;
   function F_NBE return BOOLEAN renames F_A;
   function F_NC  return BOOLEAN renames F_AE;
   function F_NE  return BOOLEAN;
   function F_NG  return BOOLEAN renames F_LE;
   function F_NGE return BOOLEAN renames F_L;
   function F_NL  return BOOLEAN renames F_GE;
   function F_NLE return BOOLEAN renames F_G;
   function F_NO  return BOOLEAN;
   function F_NP  return BOOLEAN;
   function F_NS  return BOOLEAN;
   function F_NZ  return BOOLEAN renames F_NE;
   function F_O   return BOOLEAN;
   function F_P   return BOOLEAN;
   function F_PE  return BOOLEAN renames F_P;
   function F_PO  return BOOLEAN renames F_NP;
   function F_S   return BOOLEAN;
   function F_Z   return BOOLEAN renames F_E;


   --	    ******************************************
   --	    * 10. SUPPRESSING SUBPROGRAM PROLOG CODE *
   --	    ******************************************

   -- If the following call is used as the very first statement in a
   -- subprogram, immediately following the begin:

   procedure I_SUPPRESS_PROLOG_EPILOG;
   pragma Import (Intrinsic, I_SUPPRESS_PROLOG_EPILOG, link_name => "I_SUPPRESS_PROLOG_EPILOG");
   procedure SUPPRESS_PROLOG_EPILOG renames I_SUPPRESS_PROLOG_EPILOG;

   -- then the specified procedure is compiled without generating the standard
   -- prolog and epilog code. This is typically used when an entire procedure
   -- is coded using the instrinsics, and the programmer takes responsibility
   -- for generating any required prolog and epilog code.

   -- Subprograms containing this call may have parameters, and the use of
   -- functions returning values is also permitted, but in these cases the
   -- programmer is responsible for conforming to the Alsys standard calling
   -- sequence (to access parameters or return a function result).


   --	    *********************************************
   --	    * 11. PRIVATE DECLARATIONS OF OPERAND TYPES *
   --	    *********************************************

private

   -- The full declarations for the types MEM and OFS are dummy
   -- declarations to satisfy the syntactic requirements. In practice,
   -- since all uses of these types are optimized at compile time, no
   -- values of these types are ever generated.

   type MEM is new INTEGER;

   type OFS is new INTEGER;

   -- Intrinsic pragmas.
   pragma Import (Intrinsic, ST, Link_Name => "ST");
   pragma Import (Intrinsic, "*", Link_Name => "*");
   pragma Import (Intrinsic, F_A, Link_Name => "F_A");
   pragma Import (Intrinsic, F_AE, Link_Name => "F_AE");
   pragma Import (Intrinsic, F_B, Link_Name => "F_B");
   pragma Import (Intrinsic, F_BE, Link_Name => "F_BE");
   pragma Import (Intrinsic, F_E, Link_Name => "F_E");
   pragma Import (Intrinsic, F_G, Link_Name => "F_G");
   pragma Import (Intrinsic, F_GE, Link_Name => "F_GE");
   pragma Import (Intrinsic, F_L, Link_Name => "F_L");
   pragma Import (Intrinsic, F_LE, Link_Name => "F_LE");
   pragma Import (Intrinsic, F_NE, Link_Name => "F_NE");
   pragma Import (Intrinsic, F_NO, Link_Name => "F_NO");
   pragma Import (Intrinsic, F_NP, Link_Name => "F_NP");
   pragma Import (Intrinsic, F_NS, Link_Name => "F_NS");
   pragma Import (Intrinsic, F_O, Link_Name => "F_O");
   pragma Import (Intrinsic, F_P, Link_Name => "F_P");
   pragma Import (Intrinsic, F_S, Link_Name => "F_S");
   pragma Import (Intrinsic, SUPPRESS_PROLOG_EPILOG, Link_Name => "SUPPRESS_PROLOG_EPILOG");

end MACHINE_OPERATIONS_386;
