-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.protected_operations_pkg.spc,v $ 
-- $Revision: 1.12 $ $Date: 1996/11/05 16:52:52 $ $Author: bethc $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.Protected_Types_Pkg;  use System.RTS.Protected_Types_Pkg;  
with Ada.Interrupts;
with Ada.Exceptions;
with System.RTS.Interface_Types;

package System.RTS.Protected_Operations_Pkg Is

    Pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- This package contains the support for protected operations
-- (subprograms and entry calls). For protected procedures, there are
-- two cases, a special case for a procedure in an object with no
-- entries and the general case.  Included in the former are all
-- calls to protected functions, as there is implicit no change in
-- state of the po that needs to be accounted for.  The types are
-- defined in package rts_protected_types_pkg.

--==========================

    subtype Code_Address is Interface_Types.Code_Address;

    procedure Do_Or_Queue(
      ECD : ECD_Access );
    --* OVERVIEW
    -- Called by all RTS entry call interfaces. Also, called
    -- recursively in requeue cases.  Handles both Small and Large
    -- models.  Note that requeue may be to an object of a different
    -- model.  Locks are managed by caller.
    --* EFFECTS
    -- Leaves protected record with no work remaining on entries with
    -- True barriers.

    procedure Do_Entryless_Protected_Subp_Call(
      PO          : PO_Access;
      Params      : Param_Block_Ptr;
      Pr_Subp     : Code_Address );
    --* OVERVIEW
    -- Do_Entryless_Protected_Subp_Call is invoked to execute a protected
    -- function.  It is also used to invoke a protected procedure
    -- when there are no entries in the protected record type.
    --
    --* REQUIRES
    -- The return value for a function is contained within the
    -- parameter block, as an "out" parameter.  The generated code
    -- of the called procedure will be responsible for assignment.
    --
    --* EFFECTS 
    -- Execute the function at Pr_Subp, passing both PO and
    -- Params as parameters, propagating any exceptions.
    --

    procedure Do_Protected_Procedure_Call(
      PO          : PO_Access;
      Params      : Param_Block_Ptr;
      Pr_Subp     : Code_Address );
    --* OVERVIEW
    -- Do_Protected_Procedure_Call is invoked to execute a protected
    -- procedure within a protected record type that has entries.
    --
    --* EFFECTS 
    -- Execute the function at Pr_Subp, passing both PO and
    -- Params as parameters, propagating any exceptions.  At the
    -- completion of the procedure, the queues of the protected
    -- object will be scanned for available executable entries.
    --

    procedure Do_Protected_Entry_Call(
	ECD : ECD_Access
    );
    --* OVERVIEW
    --  Called for an unconditional entry call.
    --* EFFECTS
    --  Seizes and releases lock of the protected object around calls
    --  to Do_Or_Queue.  Suspends after releasing lock if call is queued.

    function Do_Conditional_Protected_Call(
	ECD : ECD_Access
    ) return Boolean;
    --* OVERVIEW
    --  Called for a conditional entry call on a protected object.

    function Do_Timed_Protected_Call(
	ECD : ECD_Access
    ) return Boolean;
    --* OVERVIEW
    --  Called for a timed entry call with an absolute expiration.
    --* EFFECTS
    --  Seizes and releases lock of the protected object around call
    --  to Do_Or_Queue.  Starts delay if call was queued abortably.
    --  Call cancellation is performed by Delay Manager if delay expires.

    function Do_Async_Protected_Call(
      ECD          : ECD_Access) return Boolean;
       --Master       : Master_Handle

    --* OVERVIEW
    --  Called for an asynchronous select statement.
    --* EFFECTS
    --  Seizes and releases lock of the protected object around call
    --  to Do_Or_Queue.

    function Do_Protected_Entry_Count(
      PO           : PO_Access;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index ) return Natural;
    --* OVERVIEW
    --  Corresponds to the attribute E'COUNT for Large model.
    --* REQUIRES
    --  Do_Protected_Entry_Count must be called from within the
    --  protected object.
    --* EFFECTS
    --  Examines the queue of the PO to determine the count attribute
    --  which, if non-zero, is held in the ECD at the head of the
    --  queue.

    function Do_Protected_Entry_Caller_Priority(
      PO           : PO_Access;
      Name_Index   : Entry_Name_Index;
      Member_Index : Entry_Member_Index ) return System.Priority;
    --* OVERVIEW
    --  Implements implementation-defined attribute 'Caller_
    --  Priority used within the RTS.
    --* REQUIRES
    --  Lock is held on PO.  Used currently in Selective_Accept entry
    --  of a task's protected TCB.  Therefore, this procedure is
    --  called from the generated code of a protected entry of PO.
    --* EFFECTS
    --  Examines the queue of the PO to determine the priority
    --  of the ECD at the head of the 1-D queue designated by the
    --  index pair passed in.

    procedure Scan_Queues(
      PO : PO_Access );
    --* OVERVIEW
    --  Called from Do_Or_Queue, a core RTS routines used by all
    --  entry call interfaces.  Also called from the epilogue of an
    --  interrupt handler.  Requires that the lock of PO is held.
    --* EFFECTS
    --  Evaluates barrier and performs entry body if barrier is true.
    --  Visits entries in an order determined by scheduling policy.
    --  Iterates until no more work can be done. Barriers can open up
    --  as more calls are serviced.

    procedure Small_Scan_Queues(
      PO : PO_Access );
    --* OVERVIEW
    -- Called by Scan_Queues for the Small model.

    procedure Large_Scan_Queues(
      PO : PO_Access );
    --* OVERVIEW
    -- Called by Scan_Queues for the Large model.

    procedure Cancel_Call(
      ECD : ECD_Access );
    --* EFFECTS
    -- Cancels the call represented by the specified ECD.  The call
    -- can only be cancelled upon becoming queued abortably.  Due to
    -- requeue the Called_PO is may be unstable. Therefore, this
    -- procedure loops tracking the call through a series of
    -- protected objects.

    procedure Cancel_Call_Already_Locked(
      ECD : ECD_Access );
    --* OVERVIEW
    -- Used in Pr_TCB_Type.End_of_Rendezvous to cancel call queued on 
    -- Caller_In_Rendezvous.
    --* EFFECTS
    -- Same as Cancel_Call except lock is assumed to be held by caller.

    procedure Cancel_Call_For_Abort(
	ECD : ECD_Access
    );	
    --* OVERVIEW
    -- Used by abort to cancel an entry call.  Might not succeed if
    -- priority of aborting task is too high or call is queued 
    -- nonabortably.

    procedure Move_Call_For_Set_Priority(
	ECD : ECD_Access;
	Priority : System.Priority
    );

    procedure Record_Exception( 
	ECD    : ECD_Access;
	Exc_ID : Ada.Exceptions.Exception_ID
    );
    -- * OVERVIEW
    -- Called to store an exception occurrence in an ECD.

    procedure Raise_Exception_In_Callers(
        PO    	  :  PO_Access;
	Exc_ID    :  Ada.Exceptions.Exception_ID
    );
    --* EFFECTS
    -- Sets exception-related fields in all ECDs on all queues of
    -- this protected object, and clears all queues. If Current is
    -- not null, sets the exception in that ECD (which is not on any
    -- queue).

    procedure Add_To_Queue( ECD : ECD_Access );
    --* OVERVIEW
    -- Called to add an ECD to an entry queue when barrier is false.
    -- Also called to move an ECD from one queue to another within a
    -- protected object.  Requires that the lock of the corresponding
    -- PO is held.

    procedure Remove_From_Queue( ECD : ECD_Access );
     --* Removes ECD from simple queue.  No queue scanning performed.

end System.RTS.Protected_Operations_Pkg;
