-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.sec_stack_pkg.spc,v $ 
-- $Revision: 1.5 $ $Date: 95/11/10 13:51:56 $ $Author: akira $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.RTS.TCB;
package System.RTS.TGT.Sec_Stack_Pkg is

    --* OVERVIEW
    -- Space on the secondary stack should be managed as a mark-release heap.
    -- System.RTS.Secondary_Stack_Pkg should be implemented in such a way
    -- that one can obtain the highwater mark of the secondary stack
    -- utilization (Mark), allocate a given number of storage units of memory
    -- on the secondary stack (Allocate), and release memory so that 
    -- the secondary stack is restored back to a given mark (Release).

    type Mark_Ptr is private;

    Null_Mark_Ptr : constant Mark_Ptr;

    -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * --

    procedure Initialize (Size : in Natural; 
		Current_TCB : System.RTS.TCB.Ref_TCB := Null);
       -- Allocate and initialize the initial chunk for a secondary stack.
       -- Initializes the SS components of current TCB for this new chunk.  
       -- This function should be called for each created thread to initialize
       -- its secondary stack.

    -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * --

    procedure Free( Current_TCB : System.RTS.TCB.Ref_TCB := Null);
       -- Release the entire secondary stack.  This procedure should be called
       -- to when the thread is being destroyed.

    -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * --

    procedure Mark( Mark_Save : Mark_Ptr );

    pragma Export( Ada,
		   Mark,
		   "rts_ss_mark" );

        --* OVERVIEW
        -- Mark allows an application to "remember" or "mark"
	-- its current point of secondary stack utilization.
        --
        --* EFFECTS
        -- Mark will save the current 'Mark' of secondary stack
        -- utilization in the area addressed by Mark_Save.
        -- A subsequent call to Release with this same mark will
        -- nominally restore the secondary stack to the same point,
        -- releasing any secondary stack allocated in the interim.
        --
        -- Given the state of the secondary stack:
	--
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            |
        --        |        |            |
        --        |        |            |
        --   direction     |            |
        --      of         |            |
        --     stack       |------------| Highwater mark (M)
        --     growth      |            |
        --        |        |            |
        --        |        |            |
        --        V        |            |
        --                 |            |
        --                 |            |
        --                 |            |
        --                       .
        --                       .
        --                       .
	--
	--      Mark( Mark_Save => Save_Address );
	--
        -- This should return in the area addressed by Save_Address
	-- a representation of the highwater mark (M).

    -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * --

    function Allocate( Size : in Natural ) return Address;

    pragma Export( Ada,
		   Allocate,
		   "rts_ss_allocate" );

        --* OVERVIEW
        -- Allocate provides the basic mechanism of secondary stack
        -- memory allocation.
        --
        --* EFFECTS
        -- Allocate will reserve a block of space equal to the size
        -- requested and aligned to the proper target dependent boundary
	-- and return its address.  If sufficient space is not available,
        -- STORAGE_ERROR will be raised and the state of the secondary
        -- stack remains unchanged.
        --
	-- Before Allocate:
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            |
        --        |        |            |
        --        |        |------------| Highwater mark (M)
        --   direction     |            |
        --      of         |            |
        --     stack       |            |
        --     growth      |            |
        --        |        |            |
        --        |        |            |
        --        V        |            |
        --                 |            |
        --                 |            |
        --                 |            |
        --                       .
        --                       .
        --                       .
	--
	--      New_Stack := Allocate( Size => Needed_Stack );
	--
	-- After Allocate:
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            |
        --        |        |            |
        --        |        |------------| <-- New_Stack
        --   direction     |            |
        --      of         | Newly      |
        --     stack       |  Allocated |
        --     growth      |   Stack    |
        --        |        |    Space   |
        --        |        |            |
        --        V        |------------| Highwater mark 
        --                 |            | (M+Needed_Stack+Alignment)
        --                 |            |
        --                 |            |
        --                       .
        --                       .
        --                       .
	--
        -- The memory address after proper alignment of the allocated
	-- secondary stack space is returned in New_Stack and the 
	-- internal highwater mark set to (M+Needed_Stack+Alignment)
	-- storage units.

    -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * --

    procedure Release( Mark_Save : in Mark_Ptr );

    pragma Export( Ada,
		   Release,
		   "rts_ss_release" );

        --* OVERVIEW
        -- Release provides for the release of memory allocated
        -- from the secondary stack.
        --
        --* EFFECTS
	-- Memory allocated between the time the Mark_Save was established
	-- (via a call to Mark) and the call to Release will be released.
        -- Release will return the secondary stack to the condition it was
	-- at the point where the 'Mark' given by Mark_Save was established.
        --
        -- Before Release:
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            |
        --        |        |            |
        --        |        |            | (Previous_Mark)
        --   direction     |            |
        --      of         |            |
        --     stack       |            |
        --     growth      |            |
        --        |        |            |
        --        |        |            |
        --        V        |------------| Highwater mark (M)
        --                 |            |
        --                 |            |
        --                 |            |
        --                       .
        --                       .
        --                       .
	--
	--      Release( Mark_Save => Save_Address );
	--
	-- After Release:
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            |
        --        |        |            |
        --        |        |------------| Highwater mark (Previous_Mark)
        --   direction     |            |
        --      of         |            |
        --     stack       |            |
        --     growth      |            |
        --        |        |            |
        --        |        |            |
        --        V        |            | (M)
        --                 |            |
        --                 |            |
        --                 |            |
        --                       .
        --                       .
        --                       .
	--

    -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * --

    function Copy_And_Release( From      : in Address;
                               Mark_Save : in Mark_Ptr;
                               Size      : in Natural ) return Address;

    pragma Export( Ada,
		   Copy_And_Release,
		   "rts_ss_copy_and_release" );

        --* OVERVIEW
        -- Copy_And_Release provides the functionality of "stripping"
        -- the secondary stack back to a given point, similar to Release.
        -- However, it also allows for "except for this data object",
        -- in that a given data object may be made an exception to the
        -- releasing mechanism.
        --
        --* EFFECTS
        -- Copy_And_Release will release all secondary stack back to
        -- Mark_Save, except that space of Size will be reserved on the
	-- secondary stack for the object given at address From.
	-- That object will be copied starting at the point Mark_Save.
	-- Copy_And_Release will return the starting address of the copied 
	-- object.
        -- Copy_And_Release may actually result in an allocation,
        -- as the object to be copied need not necessarily reside in the
        -- secondary stack.  If sufficient space is not available to
        -- accommodate this copy, STORAGE_ERROR will be raised and the
        -- secondary stack state will remain unchanged.
        --
        -- Note that releasing to Mark_Save, allocating Size on the 
	-- secondary stack, and coping the Size object at From will not
        -- be correct if the object to be copied was on the secondary
	-- stack at a point beyond the Mark_Save.
        --
        -- Case 1 : Item at From is on the stack
	--
	-- Before Copy_And_Release
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            | (Previous_Mark)
        --        |        |            |
        --        |        |            | 
        --   direction     |- -- -- -- -| <-- Copy_From (address)
        --      of         | Item       |
        --     stack       |  To        |
        --     growth      |   Copy     |
        --        |        |- -- -- -- -| (Copy_From+Copy_Size)
        --        |        |            |
        --        V        |            |
        --                 |            |
        --                 |------------| Highwater mark (M)
        --                 |            |
        --                       .
        --                       .
        --                       .
	--
	--      Item_Address := Copy_And_Release( 
	--                          From      => Copy_From,
        --                          Mark_Save => Previous_Mark,
        --                          Size      => Copy_Size );
	--
	-- After Copy_And_Release
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |- -- -- -- -| (Previous_Mark) <-- Item_Address
        --        |        |            |
        --        |        |            | 
        --   direction     |            | 
        --      of         |------------| Highwater mark 
        --     stack       |            |
        --     growth      |            |
        --        |        |            |
        --        |        |            |
        --        V        |            |
        --                 |            |
        --                 |            |
        --                 |            |
        --                       .
        --                       .
        --                       .
	--
	-- Copy_And_Release will return the address of the Item in
	-- Item_Address as well as clear all but Item from the stack
	-- and set the new highwater mark.
        --
        --
        -- Case 2 : Item at From is not on the stack
	--
	-- Before Copy_And_Release
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            | (Previous_Mark)
        --        |        |            |
        --        |        |            | 
        --   direction     |------------| Highwater mark (M)
        --      of         |            |
        --     stack       |            |
        --     growth      |            |
        --        |        |            | 
        --        |        |            |
        --        V        |            |
        --                 |            |
        --                 |            | 
        --                 |            |
        --                       .
        --                       .
        --                       .
	--
	-- Somewhere else in memory:
	--
        --                 +------------+ <-- Copy_From
        --                 |            | 
        --                 |            |
        --                 |            | 
        --                 |            |
        --                 |            | 
        --                 |            |
        --                 |            | 
        --                 +------------+ <-- Copy_From+Copy_Size
	--
	--
	--      Item_Address := Copy_And_Release( 
	--                          From      => Copy_From,
        --                          Mark_Save => Previous_Mark,
        --                          Size      => Copy_Size );
	--
	-- After Copy_And_Release
        --                       .
        --                       .
        --                       .
        --                 |            |
        --                 |            | (Previous_Mark) <-- Item_Address
        --        |        |            |
        --        |        |            | 
        --   direction     |            | (M)
        --      of         |            |
        --     stack       |            |
        --     growth      |            |
        --        |        |------------| Highwater mark
        --        |        |            |
        --        V        |            |
        --                 |            |
        --                 |            | 
        --                 |            |
	--
        --                       .
        --                       .
        --                       .
	-- Copy_And_Release will return the address of the Item in
	-- Item_Address as well as clear all but Item from the stack
	-- and set the new highwater mark.  Note that the new highwater
	-- mark is greater than the highwater mark before Copy_And_Release
	-- which means that there was an allocation.

    -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * -- * --

private
    type Mark_Object;
    -- Representation of a secondary stack "mark" to be completed in the body

    type Mark_Ptr is access all Mark_Object;

    Null_Mark_Ptr : constant Mark_Ptr := null;

end System.RTS.TGT.Sec_Stack_Pkg;
