-- $Source: /home/dsd/d/aa/monoBANK/AA/system.rts.tgt.wide_char_io.spc,v $
-- $Revision: 1.2 $ $Date: 96/01/16 15:59:17 $ $Author: akira $
--        Copyright (C) 1995 by Intermetrics, Inc.   

with Ada.IO_Exceptions;

package System.RTS.TGT.Wide_Char_IO is  -- IO operations underlying Wide_Text_IO

    pragma suppress( elaboration_check );

    --| OVERVIEW
    --| This package specifies the IO operations underlying Text_IO.
    --| It calls routines which act upon the actual external files,
    --| and also manipulates the file types which are used to represent
    --| the external files.  It also takes care of input and output buffering.

    subtype ELEMENT_TYPE is WIDE_CHARACTER;
    --| Wide_Text_IO operates upon file elements of type WIDE_CHARACTER.

    type FILE_TYPE is limited private;
    --| The file type object used to represent the external file.

    type FILE_MODE is (IN_FILE, OUT_FILE, APPEND_FILE);
    --| Text_IO external files may be mode IN_FILE (read) or
    --| OUT_FILE, APPEND_FILE (write).

    -- file management
    
    ------------------------------------------------------------------------
    
    procedure CREATE (
        FILE     : in out FILE_TYPE;    --| File to be associated with
					--|   new external file
        MODE     : in FILE_MODE := OUT_FILE;
					--| Mode of new file
        NAME     : in STRING := "";     --| Name of new file
        FORM     : in STRING := ""      --| Form of new file
        );
    
        --| OVERVIEW
        --| This procedure associates the given file type FILE with an
	--| external file having the given name NAME and form FORM, and
	--| sets the mode of the external file to the given mode
	--| MODE.  The file is left open.  If no name is given, a temporary
	--| file is created. If no mode is given, the default mode is OUT_FILE.
        --| This routine requires that the given file is not already open.
        --| If it is already open, STATUS_ERROR is raised.  If the string
        --| given as name does not allow the identification of an external
        --| file, NAME_ERROR is raised.  If the environment does not support
        --| creation of an external file with the specified name and mode,
        --| USE_ERROR is raised.
    
    
    ------------------------------------------------------------------------
    
    procedure OPEN (
        FILE     : in out FILE_TYPE;    --| File to be associated with the
                                        --|   external file.
        MODE     : in FILE_MODE;        --| Mode of external file.
        NAME     : in STRING;           --| Name of external file.
        FORM     : in STRING := ""      --| Form of external file.
        );
    
        --| OVERVIEW
        --| This procedure associates the given file type FILE with an
	--| existing external file having the given name NAME and form
	--| FORM, and sets the mode of the external file to the given mode
	--| MODE.  The file is left open. This procedure requires that the
	--| file is not already open.  If it is, STATUS_ERROR is raised.
	--| If the string given as name does not allow the identification
	--| of an external file, NAME_ERROR is raised.  If the environment
	--| does not support the opening of an external file with the given
	--| name and mode, USE_ERROR is raised.
    
    ------------------------------------------------------------------------
    
    procedure CLOSE (
        FILE     : in out FILE_TYPE     --| File associated with external file
        );
    
        --| OVERVIEW
        --| The given file type FILE is disassociated from its associated
	--| external file.  The external file is left closed, and if it
	--| is a temporary file, it is deleted.
        --| This procedure requires that the given file is open.  If it is
	--| not, STATUS_ERROR is raised.
    
    ------------------------------------------------------------------------
    
    procedure DELETE (
        FILE     : in out FILE_TYPE     --| File associated with external file
                                        --|   to be deleted
        );
    
        --| OVERVIEW
        --| Deletes the external file associated with the given file type
	--| FILE.  If the given file is not open, STATUS_ERROR is raised.
	--| If the environment does not support deletion of the external file
        --| associated with the given file, USE_ERROR is raised.
    
    ------------------------------------------------------------------------
    
    procedure RESET (
        FILE     : in out FILE_TYPE;    --| File associated with external file
                                        --|   which is to be reset.
        MODE     : in FILE_MODE         --| Mode which the file's current mode
                                        --|   is to be changed to.
        );
    
        --| OVERVIEW
        --| The current position in the external file associated with
	--| the file type FILE is reset to the beginning of the file,
	--| and the mode of the external file is changed to the given mode
	--| MODE.  If the file is not open, STATUS_ERROR is raised.
	--| If the mode of file is unable to be changed, or the current
	--| position in the file is unable to be reset to the beginning
	--| of the file, USE_ERROR is raised.
    
    --------------------------------------------------------------------
    
    function  NAME (
        FILE     : in FILE_TYPE         --| File associated with external file
                                        --|   whose name is to be returned.
        ) return STRING;                --| Name of external file.
    
        --| OVERVIEW
        --| The name of the external file associated with the given file is
        --| returned.  If the given file is not open, STATUS_ERROR is raised.
    
    ----------------------------------------------------------------------
    
    function  FORM (
        FILE     : in FILE_TYPE         --| File associated with external file
                                        --|   whose form is to be returned.
        ) return STRING;                --| Form of external file.
        
        --| OVERVIEW
        --| The form string of the external file associated with the given
	--| file is returned.  If the given file is not open, STATUS_ERROR
	--| is raised.
    
    ----------------------------------------------------------------------
    
    procedure OPEN_STANDARD_INPUT (
        FILE     : in out FILE_TYPE     --| File which is to be associated
                                        --|   with external file.
        );
    
        --| OVERVIEW
        --| The given file is associated with the external file representing
        --| terminal input.  If the file is already open, STATUS_ERROR 
	--| is raised.
    
    --------------------------------------------------------------------

    procedure OPEN_STANDARD_OUTPUT (
        FILE     : in out FILE_TYPE     --| File to be associated with
                                        --|   external file.
        );

        --| OVERVIEW
        --| Associates a file with the external file representing terminal
        --| output.  STATUS_ERROR is raised if the given file is already open.
    
    --------------------------------------------------------------------

    procedure OPEN_STANDARD_ERROR (
        FILE     : in out FILE_TYPE     --| File to be associated with
                                        --|   external file.
        );

        --| OVERVIEW
        --| Associates a file with the external file representing terminal
        --| error.  STATUS_ERROR is raised if the given file is already open.
    
    --------------------------------------------------------------------
    
    procedure READ (
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   where element is to be read from.
        ITEM     : out ELEMENT_TYPE     --| Where element is to be put.
        );
    
        --| OVERVIEW
        --| The element at the current position in the external file
        --| associated with the given file is read into ITEM, and the
        --| current position in the file is advanced.
        --| If position in external file associated with given file is at end
        --| of file, END_ERROR will be raised.
    
    --------------------------------------------------------------------

    procedure WRITE (
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   which will be written into.
        ITEM     : in ELEMENT_TYPE      --| Element to be written into file.
        );
    
        --| OVERVIEW
        --| A data element is output to the output buffer associated with the
        --| given file.  If the file is not open, STATUS_ERROR will be raised.
	--| If the mode of the external file associated with the given file
	--| is not Out_File, MODE_ERROR will be raised.
    
    
    --------------------------------------------------------------------
    
    procedure WRITE(
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   which is to be written to.
        ITEM     : in WIDE_STRING       --| String which is to be written to
                                        --|   file.
        );
    
        --| OVERVIEW
        --| A string is output to the output buffer associated with the given
        --| file.  If the file is not open, STATUS_ERROR is raised.
	--| If the mode of the external file associated with the given
	--| file is not Out_File, MODE_ERROR is raised.
    
    --------------------------------------------------------------------
    
    procedure FLUSH (
        FILE     : in File_Type         --| File whose output buffer is to be
                                        --|   flushed.
        );
    
        --| OVERVIEW
        --| The output buffer associated with the given file is written out
        --| to the external file associated with the given file.
        --| If all the elements in the output buffer are unable to be written
        --| to the external file associated with the given file, DATA_ERROR is
        --| raised.
    
    --------------------------------------------------------------------
    
    function  END_OF_FILE (
        FILE     : in FILE_TYPE         --| File associated with external file
                                        --|   where end of file is going to be
                                        --|   checked.
        ) return BOOLEAN;               --| TRUE if at end of file,
					--| FALSE otherwise.
    
        --| OVERVIEW
        --| This file return TRUE if the external file associated with the
        --| given file is at end of file, and FALSE otherwise.
    
    ----------------------------------------------------------------------
    
    procedure GET_LINE (
        FILE     : in FILE_TYPE;        --| File associated with external file
                                        --|   to be read from.
        ITEM     : out wide_string;     --| Buffer to be filled
        LAST     : out natural          --| Last index read
        );

        --| OVERVIEW
        --| The external file associated with the given file is read
        --| up to and including the carriage return character or until
        --| range of string ITEM is exhausted.  The current position in
        --| the file is advanced.  LAST is the position in ITEM
        --| where the last character read is stored.
        --| If the file is not open, STATUS_ERROR will be raised.
        --| If the mode of the external file associated with the given file
        --| is not IN_FILE, MODE_ERROR will be raised.
 
    
    ----------------------------------------------------------------------
   
    procedure Get_Immediate(
        FILE      : in FILE_TYPE;        --| File associated with external file
                                         --|   to be read from.
        WAIT      : in boolean;          --| Wait or No_Wait
        ITEM      : out wide_character;  --| character to be filled
        AVAILABLE : out boolean          --| if Item is available when
                                         --|   when No_Wait
        );

        --| OVERVIEW
        --| Reads the next character, wither control or graphic, from
        --| the specified FILE.  FILE is assumed to be In_File and the
        --| current location is assumed to be not at the end of the file.
        --| The procedure, if the value of WAIT is false, will return with
        --| AVAILABLE set to true and ITEM containing the character read or
        --| AVAILABLE set to false and ITEM unspecified.  If the value of
        --| WAIT is true then this procedure will wait until the ITEM is
        --| read and AVAILABLE is meaningless.

    ----------------------------------------------------------------------

    -- Exceptions
    
    STATUS_ERROR : exception renames Ada.IO_EXCEPTIONS.STATUS_ERROR;
    MODE_ERROR   : exception renames Ada.IO_EXCEPTIONS.MODE_ERROR;
    NAME_ERROR   : exception renames Ada.IO_EXCEPTIONS.NAME_ERROR;
    USE_ERROR    : exception renames Ada.IO_EXCEPTIONS.USE_ERROR;
    DEVICE_ERROR : exception renames Ada.IO_EXCEPTIONS.DEVICE_ERROR;
    END_ERROR    : exception renames Ada.IO_EXCEPTIONS.END_ERROR;
    DATA_ERROR   : exception renames Ada.IO_EXCEPTIONS.DATA_ERROR;

private

    type FILE_RECORD; -- to be completed in package body

    type FILE_TYPE is access FILE_RECORD;

end System.RTS.TGT.Wide_Char_IO;
