--   Copyright 1995, Intermetrics, Inc.
--
with Interfaces.Java;
with java.lang.String; use java.lang.String;
package image is
    -- Returns the character corresponding to the given digit
    function digitimage(digit : Integer) return Character;

    -- Returns the image of the given long integer value
    function integer_image(x : Long_Integer) return String;

    -- Returns the image, in a %g-like format, of the given long float value
    function double_image(x : Long_Float) return String;

    -- A very simple unit test
    procedure main(argv : String_array);
end;

with java.lang.System;
with java.io.PrintStream;

with ada.strings.unbounded;  use ada.strings.unbounded;
package body image is

    --
    -- DigitImage:
    --   Returns the character corresponding to the given digit
    --
    function digitimage(digit : Integer) return Character is
    begin
	return character'val(character'pos('0') + digit);
    end;

    --
    -- Integer_Image:
    --   Returns the image of the given long integer value, as an
    --   unbounded string, for use by Double_Image
    --
    -- This implementation using Unbounded_String isn't very efficient, but
    -- it's easy.
    --
    function integer_image(x : Long_Integer) return Unbounded_String is
	result : Unbounded_String;
	value : Long_Integer;
    begin
	if x = 0 then
	    return To_Unbounded_String("0");
	elsif x > 0 then
	    value := -x;
	else
	    value := x;
	end if;
        while value /= 0 loop
	    result := digitimage(-Integer(value rem 10)) & result;
	    value := value / 10;
	end loop;
	if x < 0 then
	    result := '-' & result;
	end if;
	return result;
    end integer_image;

    --
    -- Integer_Image:
    --   Returns the image of the given long integer value
    --
    function integer_image(x : Long_Integer) return String is
    begin
	if x = 0 then
	    return "0";
	else
	    return To_String(integer_image(x));
	end if;
    end;

    --
    -- Double_Image:
    --   Returns the image, in a %g-like format, of the given long float value
    --
    function double_image(x : Long_Float) return String is
	v : Long_Float := abs(x);
	exp : Integer := 0;
	shift : Integer := 0;
	result : Unbounded_String;
	int_val : Long_Integer;
    begin
	--
	-- First, handle zero.
	--
	if v = 0.0 then
	    return "0.";
	end if;

	--
	-- Next, we check to see if the number is "too large" or "too small".
	-- If it is, convert it a mantissa and a power-of-ten exponent, with
	-- the mantissa in the range [1.0, 10.0).
	-- We use 1e12 and 1e-6 as thresholds for "too large" and "too small".
	--
	if v > 1.0e12 then
	    while v > 1.0e9 loop
		v := v / 1.0e9;
		exp := exp + 9;
	    end loop;
	    while v > 10.0 loop
		v := v / 10.0;
		exp := exp + 1;
	    end loop;
	elsif v < 1.0e-6 then
	    while v < 1.0e-9 loop
		v := v * 1.0e9;
		exp := exp - 9;
	    end loop;
	    while v < 1.0 loop
		v := v * 10.0;
		exp := exp - 1;
	    end loop;
	end if;

	--
	-- Now, multiply the number by powers of ten so that it is around
	-- 10^13 or so.  Then convert to an Integer and stick the decimal
	-- point in the right place.
	-- 
	if v < 10.0 then
	    v := v * 1.0e13;
	    shift := 13;
	else
	    while v < 1.0e13 loop
	        v := v * 10.0;
	        shift := shift + 1;
	    end loop;
	end if;

	int_val := Long_Integer(v);

	-- Make an attempt to get rid of trailing zeros.
	while shift > 0 and then (int_val rem 10 = 0) loop
	    int_val := int_val / 10;
	    shift := shift - 1;
	end loop;

	result := integer_image(int_val);

	--
	-- Insert the decimal point <shift> digits to the left of the end.
	-- First, though, we need to make sure the image is long enough.
	-- If it isn't, we need to prepend enough zeros, plus one more so
	-- that there is always a zero before the decimal point.
	--
	if Length(result) <= shift then
	    result := (shift - Length(result) + 1) * '0' & result;
	end if;
	insert(result, Length(result) - shift + 1, ".");

	-- Append an exponent, if needed.
	if exp /= 0 then
	    -- 1.e8 looks ugly.  Add a zero if necessary.
	    if tail(result, 1) = "." then
		result := result & '0';
	    end if;
	    result :=
	      result & 'e' & Unbounded_String'(integer_image(Long_Integer(exp)));
	end if;

	-- Prepend the sign
	if x < 0.0 then
	    result := '-' & result;
	end if;

	return To_String(result);
    end;

    procedure main(argv : String_Array) is
	use Interfaces.Java;
	use java.lang.System;
	use java.io.PrintStream;
    begin
	print(stdout, Ada_To_Java_String(integer_image(0)));
	println(stdout, +" (should be 0)");
	print(stdout, Ada_To_Java_String(integer_image(9979)));
	println(stdout, +" (should be 9979)");
	print(stdout, Ada_To_Java_String(integer_image(-1389237)));
	println(stdout, +" (should be -1389237)");

	print(stdout, Ada_To_Java_String(double_image(0.0)));
	println(stdout, +" (should be 0.)");
	print(stdout, Ada_To_Java_String(double_image(2.0)));
	println(stdout, +" (should be 2.)");
	print(stdout, Ada_To_Java_String(double_image(0.2)));
	println(stdout, +" (should be 0.2)");
	print(stdout, Ada_To_Java_String(double_image(10000.0)));
	println(stdout, +" (should be 10000.)");
	print(stdout, Ada_To_Java_String(double_image(2.45)));
	println(stdout, +" (should be 2.45)");
	print(stdout, Ada_To_Java_String(double_image(3.45e12)));
	println(stdout, +" (should be 3.45e12)");
	print(stdout, Ada_To_Java_String(double_image(-1.043e-5)));
	println(stdout, +" (should be -0.00001043)");
	print(stdout, Ada_To_Java_String(double_image(-1.043e-7)));
	println(stdout, +" (should be -1.043e-7)");
    end main;
end;
