-- pragma Suppress(All_Checks);

-- /*
--  *  Life.ada          by S. Tucker Taft
--  *  Copyright (c) 1995 Intermetrics, Inc.
--  *
--  * derived from:
--  *  Life.java         by Joshua J. Marketos
--  *  Copyright (c) 1995 Joshua J. Marketos 
--  *
--  *  This program is free software; you can redistribute it and/or modify
--  *  it under the terms of the GNU General Public License as published by
--  *  the Free Software Foundation; either version 2 of the License, or
--  *  (at your option) any later version.
--  *
--  *  This program is distributed in the hope that it will be useful,
--  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  *  GNU General Public License for more details.
--  *
--  *  You should have received a copy of the GNU General Public License
--  *  along with this program; if not, write to the Free Software
--  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--  *
--  */

with java.awt.Graphics; use java.awt.Graphics;
with java.awt.Color; use java.awt.Color;
package LR_Colony is
  Pixels_Per_Cell : constant := 6;
  Width_In_Cells : constant := 60;
  Height_In_Cells : constant := 30;

  Display_Label : constant Boolean := True;

  Background_Color : constant Color_Ptr := blue;
  Cell_Color : constant Color_Ptr := gray;
  Erase_Color : constant Color_Ptr := yellow;

  Label_Color : constant Color_Ptr := white;
  Label_Text_Color : constant Color_Ptr := black;
  Label_Text : constant String := 
    "Conway's game of Life, written using AdaMagic/Java";

  type XCoord is range 0..Width_In_Cells-1;
  type YCoord is range 0..Height_In_Cells-1;
  type Coord_Delta is range -1 .. +1;

  function XAdd(X : XCoord; D : Coord_Delta) return XCoord;
  function YAdd(Y : YCoord; D : Coord_Delta) return YCoord;
  pragma Inline(XAdd, YAdd);
  function "+"(X : XCoord; D : Coord_Delta) return XCoord renames XAdd;
  function "+"(Y : YCoord; D : Coord_Delta) return YCoord renames YAdd;

  type LR_Colony_Obj is private;
  subtype Colony_Obj is LR_Colony_Obj;

  procedure init_colony(C : in out Colony_Obj);

  type Colony_Iterator(C : access Colony_Obj) is limited private;
  type Coord_Pair is record
      X : XCoord;
      Y : YCoord;
  end record;

  function More(Iter : Colony_Iterator) return Boolean;
  procedure get_next(Iter : in out Colony_Iterator; Where : out Coord_Pair);

  procedure add_cell (C : in out Colony_Obj; XX : XCoord; YY : YCoord);

  procedure print_col(C : Colony_Obj; G : Graphics_Ptr);

private

  type Cell_Count is range 0..1000;
  subtype Cell_Number is Cell_Count range 1..Cell_Count'Last;

  type XCoord_Array is array(Cell_Number) of XCoord;
  type YCoord_Array is array(Cell_Number) of YCoord;

  type LR_Colony_Obj is record
    XC : XCoord_Array;
    YC : YCoord_Array;
    MySize : Cell_Count := 0;
  end record;

  type Colony_Iterator(C : access Colony_Obj) is limited record
    Current : Cell_Count := 0;
  end record;

end LR_Colony;

with java.awt.Color; use java.awt.Color;
package body LR_Colony is

  function XAdd(X : XCoord; D : Coord_Delta) return XCoord is
      Result : constant XCoord'Base := X + XCoord'Base(D);
  begin
      if Result < XCoord'First then
	  return Result + XCoord'Base(Width_In_Cells);
      elsif Result > XCoord'Last then
	  return Result - XCoord'Base(Width_In_Cells);
      else
	  return Result;
      end if;
  end XAdd;

  function YAdd(Y : YCoord; D : Coord_Delta) return YCoord is
      Result : constant YCoord'Base := Y + YCoord'Base(D);
  begin
      if Result < YCoord'First then
	  return Result + YCoord'Base(Height_In_Cells);
      elsif Result > YCoord'Last then
	  return Result - YCoord'Base(Height_In_Cells);
      else
	  return Result;
      end if;
  end YAdd;

  procedure init_colony(C : in out Colony_Obj) is
  begin
      C.MySize := 0;
  end init_colony;

  function More(Iter : Colony_Iterator) return Boolean is
  begin
    return Iter.current < Iter.C.mysize;
  end More;

  procedure get_next(Iter : in out Colony_Iterator; Where : out Coord_Pair) is
  begin
      Iter.current := Iter.current + 1;
      Where.x := Iter.C.xc(Iter.current);
      Where.y := Iter.C.yc(Iter.current);
  end get_next;

  procedure add_cell (C : in out Colony_Obj; XX : XCoord; YY : YCoord) is
  begin
      C.Mysize := C.Mysize + 1;
      C.XC(C.Mysize) := XX;
      C.YC(C.Mysize) := YY;
  end add_cell;

  procedure print_col(C : Colony_Obj; G : Graphics_Ptr) is
    XX, YY : Integer;
  begin

    -- For each live cell, display it in the "Live Cell" color.
    setColor(g, Cell_Color);
    for A in 1..C.MySize loop
      XX := Integer(C.XC(A)) * Pixels_Per_Cell + Pixels_Per_Cell/2;
      YY := Integer(C.YC(A)) * Pixels_Per_Cell + Pixels_Per_Cell/2;
      fillRect(g, XX, YY, Pixels_Per_Cell-1, Pixels_Per_Cell-1);
    end loop;
  end print_col;

end LR_Colony;


with LR_Colony; use LR_Colony;
with java.awt.Graphics; use java.awt.Graphics;
with java.awt.Color; use java.awt.Color;
package LR_lifeCol is

  type LR_lifeCol_Obj is private;
  subtype lifeCol_Obj is LR_lifeCol_Obj;

  procedure do_gen(L : in out lifeCol_Obj; G : Graphics_Ptr);

  procedure add_cell(L : in out lifeCol_Obj; X : XCoord; Y : YCoord);

  procedure clear(L : in out lifeCol_Obj);

private

  subtype Neighbor_Count is Integer range 0..18;
  type Neighbor_Matrix is array(XCoord, YCoord) of Neighbor_Count;

  type Colony_Pair is array(Boolean) of aliased Colony_Obj;

  type LR_lifeCol_Obj is record
    A : Colony_Pair;
    Current : Boolean := False;
    NB : Neighbor_Matrix; -- := (others => (others => 0));
  end record;

end LR_lifeCol;

package body LR_lifeCol is

  use type XCoord, YCoord;

  procedure inc_col(L : in out lifeCol_Obj; Which : Boolean) is

    -- Count number of neighbors for each cell

    xpxo : XCoord;
    ypyo : YCoord;
    Iter : Colony_Iterator(L.A(Which)'access);
    Where : Coord_Pair;
    X : XCoord;
    Y : YCoord;
  begin
    -- Iterate over all the live cells
    while More(Iter) loop
	Get_Next(Iter, Where);
	
	X := Where.X; Y := Where.Y;

	-- If cell itself is alive, bump by 9

	-- (Ignore cell if already processed)

	if L.NB(X, Y) < 9 then
	    -- Only process cells the first time we encounter them.
	    -- (They might be in colony twice due to initial random generation).
	
	    L.NB(X, Y) := L.NB(X, Y) + 9;

	    -- Now bump all of its neighbors (and itself once more)
	    for XO in Coord_Delta loop
	      xpxo := X + XO;
	      for YO in Coord_Delta loop
		ypyo := Y + YO;
		L.NB(xpxo,ypyo) := L.NB(xpxo, ypyo) + 1;
	      end loop;
	    end loop;
	end if;
    end loop;
  end inc_col;

  procedure stack_col(L : in out lifeCol_Obj; 

    -- Update cells that are born or die

    Current : Boolean; G : Graphics_Ptr) is

    xpxo : XCoord;
    ypyo : YCoord;
    X1, Y1 : Integer;
    Current_Iter : Colony_Iterator(L.A(Current)'access);
    Where : Coord_Pair;
    X : XCoord;
    Y : YCoord;
    Next : constant Boolean := not Current;
  begin

    -- Start with a clean slate in new colony
    init_colony(L.A(Next));

    -- For each cell in the old colony, look at each of its neighbors
    while More(Current_Iter) loop
      Get_Next(Current_Iter, Where);
      X := Where.X; Y := Where.Y;
      for XO in Coord_Delta loop
	xpxo := X + XO;
	for YO in Coord_Delta loop
	  ypyo := Y + YO;

	  -- See whether cell should come alive, die, or stay the same
	  case L.NB(xpxo, ypyo) is

	  when 12 | 13 | 3 | 7 =>  -- "7" is non-standard
	    -- Was alive with 2 or 3 neighbors, or was dead with 3 neighbors
	    -- Should be alive in new colony
	    add_cell(L.A(Next), xpxo, ypyo);
	  when 0 .. 2 | 4 .. 6 | 8 =>
	    -- Wrong number of neighbors, stays dead in new colony
	    -- (or if zero, we may have already processed this cell)
	    null;
	  when others =>
	    -- Anything else, the cell is dead in new colony.

	    -- clear out a cell (change to the "Erase" color)
	    x1 := Integer(xpxo) * Pixels_Per_Cell + Pixels_Per_Cell/2;
	    y1 := Integer(ypyo) * Pixels_Per_Cell + Pixels_Per_Cell/2;
	    setColor(g, Erase_Color);
	    fillRect(g, x1, y1, Pixels_Per_Cell-1, Pixels_Per_Cell-1);
	  end case;

	  -- Indicate that we have handled this cell
	  L.NB(xpxo,ypyo) := 0;
	end loop;
      end loop;
    end loop;
  end stack_col;

  procedure add_cell(L : in out lifeCol_Obj; X : XCoord; Y : YCoord) is
  begin
    add_cell(L.a(L.current), x, y);
  end add_cell;

  procedure do_gen(L : in out lifeCol_Obj; G : Graphics_Ptr) is
  begin
    -- Count number of neighbors for each cell
    inc_col(L, L.current);

    -- Update cells that are born or die
    stack_col(L, L.Current, g);

    -- Switch to new colony
    L.Current := not L.Current;

    -- And display it
    print_col(L.a(L.current), g);
  end do_gen;

  procedure clear(L : in out lifeCol_Obj) is
  begin
    -- Indicate there are no live cells
    init_colony(L.a(L.current));

    -- Clear out the neighbor matrix
    for X in XCoord loop
      for Y in YCoord loop
	L.NB(X, Y) := 0;
      end loop;
    end loop;
  end clear;
end LR_LifeCol;


-- /**
--  * Life
--  *
--  * $id$
--  */

with java.applet.Applet; use java.applet.Applet;
with LR_Colony; use LR_Colony;
with LR_lifeCol; use LR_lifeCol;
with java.awt.Color; use java.awt.Color;
with java.awt.Graphics; use java.awt.Graphics;
with java.awt.Event; use java.awt.Event;
with java.lang.Thread; use java.lang.Thread;
with java.lang.Math; use java.lang.Math;
with java.lang.Runnable; use java.lang.Runnable;
with java.lang.String; use java.lang.String;
package LifeRect is
  type LifeRect_Obj is new Applet_Obj with private;

  procedure run(L : access LifeRect_Obj);

  procedure paint(L : access LifeRect_Obj; G : Graphics_Ptr);

  procedure update(L : access LifeRect_Obj; G : Graphics_Ptr);

  function mouseUp(L : access LifeRect_Obj; evt : Event_Ptr; x, y : Integer)
    return Boolean;

  function mouseDown(L : access LifeRect_Obj; evt : Event_Ptr; X, Y : Integer)
    return Boolean;

  procedure start(L : access LifeRect_Obj);

  procedure stop(L : access LifeRect_Obj);

  procedure init(L : access LifeRect_Obj);

  procedure main(s : String_Array);
private
  type lifeCol_Ptr is access all lifeCol_Obj;  
    -- Default Constructor can't handle complicated obj yet

  type LifeRect_Obj is new Applet_Obj with record

      Runnable : aliased Runnable_Obj;  -- means "implements Runnable"

      kicker : Thread_Ptr := null;

      
       -- Whether or not the thread has been paused by the user.
       
      ThreadDead : boolean := false;

      elvis : boolean := false;

      a : lifeCol_Ptr;
  end record;

  pragma Convention(Java, run);  -- So matches Runnable.run
end LifeRect;

with Interfaces.Java;
with java.io.PrintStream; use java.io.PrintStream;
with java.lang.System; use java.lang.System;
with java.lang.Runnable; use java.lang.Runnable;
with java.awt.Image; use java.awt.Image;
with java.awt.Font; use java.awt.Font;
-- with java.awt.Window; use java.awt.Window;
package body LifeRect is

   -- Run the image loop. This methods is called by class Thread.
   -- @see java.lang.Thread

  Label_Height : constant := 40;
  Display_Width : constant := (Width_In_Cells+1) * Pixels_Per_Cell;
  Display_Height : constant := (Height_In_Cells+1) * Pixels_Per_Cell;
  Applet_Height : constant := Display_Height + 
    Boolean'Pos(Display_Label) * Label_Height;

  Offscreen : Image_Ptr;
  BufferG : Graphics_Ptr;
   
  function "+"(s : String) return String_Ptr
	renames Interfaces.Java."+";


  procedure run(L : access LifeRect_Obj) is

    xx : XCoord;
    yy : YCoord;
  begin
    println(stdout, "made it into the subprogram run");
    setPriority(currentThread, MIN_PRIORITY);

    clear(L.A.all);

    for I in 1 .. Width_in_Cells * Height_in_Cells / 10 loop
      xx := XCoord(Long_Float(XCoord'Last) * random);
      yy := YCoord(Long_Float(YCoord'Last) * random);
      add_cell(L.A.all, xx, yy);
    end loop;
    L.elvis := false;

    while L.kicker /= null loop
      repaint(L);
      sleep(millis => 10);  -- avoid locking up Netscape
    end loop;

  end run;

   -- Initialize the applet.
   
  procedure init(L : access LifeRect_Obj) is
  begin
    println(stdout, "about to resize");
    resize(L, Display_Width, Applet_Height);
  end init;

  -- The intended font is TimesRoman.  For a change, try Helvetica.

  procedure paint(L : access LifeRect_Obj; G : Graphics_Ptr) is
    Image_Done : Boolean;
  begin
    if Offscreen = null or else not L.elvis then
      -- (Re)display the screen from scratch
      if Offscreen = null then
	Offscreen := createImage(L, Display_Width,
				 Applet_Height);
      end if;
      if BufferG = null then
	BufferG := getGraphics(Offscreen);
      end if;
      setColor(BufferG, Background_Color);
      fillRect(BufferG, 0, 0, Display_Width, Display_Height);
      L.elvis := true;

      if Display_Label then
        -- Display a label if desired
        setColor(BufferG, Background_Color);
        drawRect(BufferG, 0, 0, Display_Width-1, Display_Height-1);

        setColor(BufferG, Label_Color);
        fillRect(BufferG, x => 1, y => Display_Height+1, 
          width => Display_Width-2, height => Label_Height-2);

        setColor(BufferG, Label_Text_Color);
        setFont(BufferG, new_Font(+"TimesRoman", PLAIN, 14));
        drawString(BufferG, +Label_Text, x => 14,
	    y => Display_Height + Label_Height/2 + 4);
      end if;

    end if;

    -- Compute the next generation
    do_gen(L.a.all, BufferG);

    -- Now draw the updated image (not clear what the bool result means)
    Image_Done := drawImage(G, Offscreen, 0, 0, null);
    
  end paint;

   -- Update without erasing the background.
  procedure update(L : access LifeRect_Obj; G : Graphics_Ptr) is
  begin
    paint(L, g);
  end update;

   -- The user clicked in the applet. Start the clock!
  function mouseUp(L : access LifeRect_Obj; evt : Event_Ptr; x, y : Integer)
    return Boolean is
  begin
    repaint(L); 
    return True;
  end mouseUp;


     -- Start the applet by forking an animation thread.
    procedure start(L : access LifeRect_Obj) is
	use java.lang;
    begin
        println(stdout, "made it into start");

	-- Create the colonies, etc.
	if L.a = null then L.a := new LifeCol_Obj; end if;

	-- Create the thread
 	if L.kicker = null then
             L.kicker := new_Thread(L.Runnable'Access, +"Life thread");
             start(L.kicker);
        end if;
    end start;

     -- Stop the applet. The thread will exit because kicker is set to null.
    procedure stop(L : access LifeRect_Obj) is
    begin
        println(stdout, "made it into stop");
        L.kicker := null;
    end stop;

     -- Pause the thread when the user clicks the mouse in the applet.

    function mouseDown(L : access LifeRect_Obj; 
      evt : Event_Ptr; X, Y : Integer)
    return Boolean is
    begin
	println(stdout, "mouse down");
        if L.ThreadDead then
            start(L);
        else
            stop(L);
	end if;
        L.ThreadDead := not L.ThreadDead;
	return True;
    end mouseDown;

    procedure main(s : String_Array) is
	L : aliased LifeRect_Obj;
    begin
	println(stdout, "finished default init");
        LifeRect.init(L'access);
	println(stdout, "finished call on init");
	LifeRect.start(L'access);
    end main;

end LifeRect;
